/***************************************************************************
                          buttongrid.cpp  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifdef WIN32
#include <windows.h>
#endif

#include <assert.h>
#include <stdlib.h>
#include <qpainter.h>
#include <qapplication.h>
#include <qdatetime.h>
//#include <qmutex.h>
#include <qthread.h>

#include "buttongrid.h"
#include "designoptions.h"
/*
 * Draws button-like 3D element.
 * x, y - left & top
 * w, h - width & height
 * d - depth of 3D effect
 * Main - main color
 * Ligth - color of light part
 * Shadow - color of shadow part
 */
void drawElement(QPainter & p, int x, int y, int w, int h, int d,
  const QColor & Main, const QColor & Light, const QColor & Shadow)
{
  QBrush bMain(Main), bLight(Light), bShadow(Shadow);
  p.fillRect(x + d, y + d, w - 2*d, h - 2*d, bMain);
  for (int i = 0; i < d; i ++)
  {
    p.fillRect((x + i), (y + i), 1, (h - i - i - 1), bLight);
    p.fillRect((x + i), (y + i), (w - i - i - 1), 1, bLight);

    p.fillRect((x + w - i - 1), (y + i + 1), 1, (h - i - i - 1), bShadow);
    p.fillRect((x + i + 1), (y + h - i - 1), (w - i - i - 1), 1, bShadow);

    p.fillRect((x + w - i - 1), (y + i), 1, 1, bMain);
    p.fillRect((x + i), (y + h - i - 1), 1, 1, bMain);

  }
}

void drawEmpty(QPainter & p, int x, int y, int w, int h, int d,
  const QColor & Main, const QColor & Light, const QColor & Shadow,
  const QColor Black = Qt::black)
{
  QBrush bBg(Black), bMain(Main), bLight(Light), bShadow(Shadow);
  p.fillRect(x, y, w, h, bBg);
  for (int i = 0; i < d; i ++)
  {
    p.fillRect((x - i - 1), (y - i - 1), 1, (h + i + i + 1), bShadow);
    p.fillRect((x - i - 1), (y - i - 1), (w + i + i + 1), 1, bShadow);

    p.fillRect((x + w + i), (y - i), 1, (h + i + i + 1), bLight);
    p.fillRect((x - i), (y + h + i), (w + i + i + 1), 1, bLight);

    p.fillRect((x + w + i), (y - i - 1), 1, 1, bMain);
    p.fillRect((x - i - 1), (y + h + i), 1, 1, bMain);
  }
}

class ButtonGridAnimateThread: public QThread
{
public:
  ButtonGridAnimateThread() {}

  void animate(ButtonGrid * Grid, int XPercent, int YPercent, int StepNum)
  {
    FGrid = Grid; FXPercent = XPercent; FYPercent = YPercent; FStepNum = StepNum;
    start();
  }

  static void pubMSleep(unsigned long msecs) { msleep(msecs); }
  static void pubUSleep(unsigned long usecs) { usleep(usecs); }

protected:
  virtual void run() { FGrid->doAnimateFigure(FXPercent, FYPercent, FStepNum); }
private:
  ButtonGrid * FGrid;
  int FXPercent, FYPercent, FStepNum;
};

ButtonGrid::ButtonGrid(QWidget *parent, const char *name ) :
  QWidget(parent, name, WRepaintNoErase),
  FElementPixmap(0),
  FElementEmptyPixmap(0),
  FAppliedPixmap(0),
  FFigure(0),
  FHasColor(true),
  FElementWidth(1), FElementHeight(1), FIndentX(5), FIndentY(5),
  FUpdateCounter(0),
  FElementIndentX(0), FElementIndentY(0),
  FGridInterval(0), FColorNumber(0)
  /*,
  FMutex(new QMutex(true)),
  FThread(new ButtonGridAnimateThread())*/
{
}

ButtonGrid::~ButtonGrid()
{
  qDebug("ButtonGrid::~ButtonGrid begin");
  /*if (FThread->running())
  {
    FThread->wait();
  }

  delete FThread;*/

  if (FElementPixmap)
    delete[] FElementPixmap;

  if (FElementEmptyPixmap)
    delete[] FElementEmptyPixmap;

  if (FAppliedPixmap)
    delete FAppliedPixmap;

  //delete FMutex;

  qDebug("ButtonGrid::~ButtonGrid end");
}

/**  */
void ButtonGrid::paintEvent( QPaintEvent * )
{
  //QMutexLocker locker(FMutex);
  bitBlt(this, 0, 0, & FPixmap);
}
/**  */
void ButtonGrid::setSizes(int ByWidth, int ByHeight)
{
  //QMutexLocker locker(FMutex);
  FElements.setSizes(ByWidth, ByHeight);
  updateGeometry();
  resize((ByWidth - FElementIndentX) * FElementWidth + FIndentX * 2,
    (ByHeight - FElementIndentY) * FElementHeight + FIndentY * 2);
}

bool ButtonGrid::getHasColor()
{
  return FHasColor;
}

void ButtonGrid::setHasColor(bool Value)
{
  //QMutexLocker locker(FMutex);
  FHasColor = Value;
  resize(sizeHint().width(), sizeHint().height());
}

void ButtonGrid::rebuildElementPixmap()
{
  //QMutexLocker locker(FMutex);
  QPainter p;

  int depthW = (FElementWidth + 4) / 8,
    depthH = (FElementHeight + 4) / 8;
  int depth = depthW < depthH ? depthW : depthH;

  FIndentX = depth * 2;
  FIndentY = depth * 2;

  if (FSourceElements.isNull())
  {
    for (int i = 1; i < FColorNumber + 1; i ++)
    {
      FElementPixmap[i].resize(FElementWidth, FElementHeight);
      p.begin( & (FElementPixmap[i]), this);
      if (FHasColor)
      {
        int c = (i - 1) % (FColorNumber) + 1;
        drawElement(p, 0, 0, FElementWidth, FElementHeight, depth,
          QColor(127*(c/9), 127*((c%9)/3), 127*(c%3)),
          QColor(240, 240, 240),
          QColor(64, 64, 64)
          );
      }
      else
        drawElement(p, 0, 0, FElementWidth, FElementHeight, depth,
          QColor(192, 192, 192),
          QColor(240, 240, 240),
          QColor(64, 64, 64)
          );
      p.end();

      FElementEmptyPixmap[i].resize(FElementWidth, FElementHeight);
      p.begin( & (FElementEmptyPixmap[i]), this);
      if (FHasColor)
      {
        int c = (i - 1) % (FColorNumber) + 1;
        drawElement(p, 0, 0, FElementWidth, FElementHeight, depth,
          QColor(63*(c/9), 63*((c%9)/3), 63*(c%3)),
          QColor(240, 240, 240),
          QColor(64, 64, 64)
          );
        /*drawEmpty(p, 0 + depth, 0 + depth, FElementWidth - 2 * depth, FElementHeight - 2 * depth, depth,
          QColor(255*(c/4), 255*((c%4)/2), 255*(c%2)),
          QColor(240, 240, 240),
          QColor(64, 64, 64),
          QColor(255*(c/4), 255*((c%4)/2), 255*(c%2))
          );*/
      }
      else
        drawEmpty(p, 0 + depth, 0 + depth, FElementWidth - 2 * depth, FElementHeight - 2 * depth, depth,
          QColor(192, 192, 192),
          QColor(240, 240, 240),
          QColor(64, 64, 64),
          QColor(192, 192, 192));
      p.end();
    }

    FElementPixmap[0].resize(FElementWidth, FElementHeight);
    p.begin( & (FElementPixmap[0]), this);
    p.fillRect(0, 0, FElementWidth, FElementHeight, QBrush(black));
    p.end();

  }
  else
  {
    for (int i = 1; i < FColorNumber + 1; i ++)
    {
      FElementPixmap[i].resize(FElementWidth, FElementHeight);
      bitBlt(& FElementPixmap[i], 0, 0, & FSourceElements,
        (i - 1) * FElementWidth, 0, FElementWidth, FElementHeight);
      FElementEmptyPixmap[i].resize(FElementWidth, FElementHeight);
      bitBlt(& FElementEmptyPixmap[i], 0, 0, & FSourceElements,
        (i - 1) * FElementWidth, 0, FElementWidth, FElementHeight, Qt::NotCopyROP);
    }
  }
}

void ButtonGrid::resize ( int w, int h )
{
  //QMutexLocker locker(FMutex);
  QWidget::resize(w, h);
  if (FElements.getWidth() == 0 ||
    FElements.getHeight() == 0)
    return;

  FPixmap.resize(w, h);
  FBackground.resize(w - 2 * FIndentX, h - 2 * FIndentY);
  QPainter p;
  p.begin(& FBackground, this);
  p.fillRect(0, 0, w - 2 * FIndentX, h - 2 * FIndentY, QBrush(black));

  if (! FSourceBackground.isNull())
  {
    for (int x = 0; x < FBackground.width(); x += FSourceBackground.width())
    for (int y = 0; y < FBackground.height(); y += FSourceBackground.height())
    {
      bitBlt(& FBackground, x, y, & FSourceBackground);
    }
  }

  if (FGridInterval > 0)
  {
    for (int h = 0; h < FElements.getHeight() - FElementIndentY; h ++)
    for (int w = 0; w < FElements.getWidth() - FElementIndentX; w ++)
    {
      if (w % FGridInterval == 0)
      {
        p.fillRect(w * FElementWidth + FElementWidth / 2,
          h * FElementHeight + FElementHeight / 2,
          2, 2, QBrush(white)
        );
      }
    }
  }

  p.end();

  repaintPixmap();
}

void ButtonGrid::setElementAt(int X, int Y, int Value)
{
  //QMutexLocker locker(FMutex);
  if (X >= FElements.getWidth() || X < 0 ||
    Y >= FElements.getHeight() && Y < 0)
  {
    qDebug("X=%d Y=%d\n", X, Y);
    assert(false);
  }

  FElements.setElementAt(X, Y, Value);

  if (FElementPixmap && X >= FElementIndentX && Y >= FElementIndentY)
  {
    if (Value)
      bitBlt(& FPixmap, FIndentX + (X - FElementIndentX) * FElementWidth,
        FIndentY + (Y - FElementIndentY) * FElementHeight,
        &(FElementPixmap[Value]));
    else
      bitBlt(& FPixmap, FIndentX + (X - FElementIndentX) * FElementWidth,
        FIndentY + (Y - FElementIndentY) * FElementHeight,
        & FBackground, (X - FElementIndentX) * FElementWidth,
        (Y - FElementIndentY) * FElementHeight,
        FElementWidth, FElementHeight);
  }
}

int ButtonGrid::getElementAt(int X, int Y) const
{
  return FElements.getElementAt(X, Y);
}

void ButtonGrid::repaintPixmap()
{
  //QMutexLocker locker(FMutex);
  QPainter p;
  p.begin(& FPixmap, this);
  drawEmpty(p, FIndentX, FIndentY,
    (getWidth() - FElementIndentX) * FElementWidth,
    (getHeight() - FElementIndentY) * FElementHeight,
    FIndentX,
    QColor(192, 192, 192), QColor(255, 255, 255),
    QColor(128, 128, 128));
  p.end();

  for (int x = FElementIndentX; x < FElements.getWidth(); x ++)
  for (int y = FElementIndentY; y < FElements.getHeight(); y ++)
  {
    if (FElements.getElementAt(x, y))
      bitBlt(& FPixmap, getXCoord(x), getYCoord(y),
        &(FElementPixmap[FElements.getElementAt(x, y)]));
    else
      bitBlt(& FPixmap, getXCoord(x), getYCoord(y),
        & FBackground, (x - FElementIndentX) * FElementWidth,
        (y - FElementIndentY) * FElementHeight,
        FElementWidth, FElementHeight);
  }

  drawFigure();

  if (FAppliedPixmap)
    bitBlt(& FPixmap, FAppliedX, FAppliedY, FAppliedPixmap);
}

void ButtonGrid::setElementIndents(int ElementIndentX, int ElementIndentY)
{
  //QMutexLocker locker(FMutex);
  FElementIndentX = ElementIndentX;
  FElementIndentY = ElementIndentY;
  updateGeometry();
  resize(sizeHint().width(), sizeHint().height());
}

void ButtonGrid::setAppearance(QString backgroundFileName, QString elementsFileName,
  int ElementSize, int GridInterval, bool HasColor, int ColorNumber, int AnimationSpeed)
{
  //QMutexLocker locker(FMutex);
  FColorNumber = ColorNumber;
  if (FElementPixmap)
    delete[] FElementPixmap;
  if (FElementEmptyPixmap)
    delete[] FElementEmptyPixmap;

  FElementPixmap = new QPixmap[FColorNumber + 2];
  FElementEmptyPixmap = new QPixmap[FColorNumber + 2];

  if (backgroundFileName.isEmpty())
  {
    FSourceBackground.resize(0, 0);
  }
  else
  {
    FSourceBackground.load(backgroundFileName);
  }

  if (elementsFileName.isEmpty())
  {
    FSourceElements.resize(0, 0);
    FElementWidth = ElementSize;
    FElementHeight = ElementSize;
  }
  else
  {
    FSourceElements.load(elementsFileName);
    FElementWidth = FSourceElements.height();
    FElementHeight = FSourceElements.height();
    qDebug("Element height %d", (int) FSourceElements.height());
  }

  FGridInterval = GridInterval;
  FHasColor = HasColor;
  FAnimationSpeed = AnimationSpeed;

  rebuildElementPixmap();
  updateGeometry();
  resize(sizeHint().width(), sizeHint().height());
}

void ButtonGrid::drawPixmap(QPixmap * AppliedPixmap, int X, int Y)
{
  if (FAppliedPixmap)
  {
    delete FAppliedPixmap;
    FAppliedPixmap = 0;
  }

  FAppliedPixmap = AppliedPixmap;
  FAppliedX = X;
  FAppliedY = Y;
  repaintPixmap();
  refresh();
}

/*void ButtonGrid::animate(IReadMatrix * figure, int StartX, int StartY,
    int StartXPercent, int StartYPercent,
    int XPercent, int YPercent, int StepNum,
    int PieceX1, int PieceY1, int PieceX2, int PieceY2)
{
  QPaintDevice * pd = new QPixmap(
    (PieceX2 - PieceX1 + 1) * FElementWidth +
    abs(XPercent) * FElementWidth / 100,
    (PieceY2 - PieceY1 + 1) * FElementHeight +
    abs(YPercent) * FElementHeight / 100
  );

  int xprefix = (XPercent < 0 ? - XPercent : 0) * FElementWidth / 100;
  int yprefix = (YPercent < 0 ? - YPercent : 0) * FElementHeight / 100;
  bitBlt(pd, 0, 0, & FPixmap,
    getXCoord(StartX) + StartXPercent * FElementWidth / 100 - xprefix,
    getYCoord(StartY) + StartYPercent * FElementWidth / 100 - yprefix,
    (PieceX2 - PieceX1 + 1) * FElementWidth +
    abs(XPercent) * FElementWidth / 100,
    (PieceY2 - PieceY1 + 1) * FElementHeight +
    abs(YPercent) * FElementHeight / 100
  );

  //QPaintDevice * pd = this;

  int w, h;
  int erasexdiff = (XPercent == 0) ? 0 : (XPercent > 0 ? -1 : 1);
  int eraseydiff = (YPercent == 0) ? 0 : (YPercent > 0 ? -1 : 1);


  QTime cl = QTime::currentTime();
  for (int Step = 1; Step <= StepNum; Step ++)
  {
    // Drawing pieces at new position
    for (w = PieceX1; w <= PieceX2; w ++)
    for (h = PieceY1; h <= PieceY2; h ++)
    {
      if (figure->getElementAt(w, h))
      {
        bitBlt(pd,
          (w - PieceX1) * FElementWidth + xprefix +
          XPercent * Step * FElementWidth / StepNum / 100,
          (h - PieceY1) * FElementHeight + yprefix +
          YPercent * Step * FElementHeight / StepNum / 100,
          &(FElementPixmap[figure->getElementAt(w, h)])
        );
      }
    }

    int ew = abs(XPercent * Step * FElementWidth / StepNum / 100 -
      XPercent * (Step - 1) * FElementWidth / StepNum / 100);
    int erasewidth = erasexdiff == 0 ? FElementWidth : ew;
    int eh = abs(YPercent * Step * FElementHeight / StepNum / 100 -
      YPercent * (Step - 1) * FElementHeight / StepNum / 100);
    int eraseheight = eraseydiff == 0 ? FElementHeight : eh;
    int erasepixelxdiff = erasexdiff == -1 ? FElementWidth - erasewidth : 0;
    int erasepixelydiff = eraseydiff == -1 ? FElementHeight - eraseheight : 0;

    // Erasing pieces at old position

    for (w = PieceX1; w <= PieceX2; w ++)
    for (h = PieceY1; h <= PieceY2; h ++)
    {
      int erasew = w + erasexdiff;
      int eraseh = h + eraseydiff;
      bool neederase =
        figure->getElementAt(w, h) &&
        (erasew < PieceX1 || erasew > PieceX2 ||
        eraseh < PieceY1 || eraseh > PieceY2 ||
        figure->getElementAt(erasew, eraseh) == 0);

      if (neederase)
      {
        int X0 = (erasew - PieceX1) * FElementWidth +
          XPercent * Step * FElementWidth / StepNum / 100 +
          erasepixelxdiff;
        int Y0 = (eraseh - PieceY1) * FElementHeight +
          YPercent * Step * FElementHeight / StepNum / 100 +
          erasepixelydiff;

        int X = getXCoord(StartX) +
          StartXPercent * FElementWidth / 100 + X0;
        int Y = getYCoord(StartY) +
          StartYPercent * FElementHeight / 100 + Y0;

        bitBlt(pd, X0 + xprefix, Y0 + yprefix, & FBackground, X - FIndentX, Y - FIndentY,
          erasewidth, eraseheight);
      }
    }

    bitBlt(this,
      getXCoord(StartX) + StartXPercent * FElementWidth / 100 - xprefix,
      getYCoord(StartY) + StartYPercent * FElementWidth / 100 - yprefix,
      pd);
    qApp->flush();
    QTime ncl = QTime::currentTime();
    while (ncl >= cl && cl.msecsTo(ncl) < 1)
    {
      ncl = QTime::currentTime();
    }

    cl = ncl;
  }

  bitBlt(& FPixmap,
    getXCoord(StartX) + StartXPercent * FElementWidth / 100 - xprefix,
    getYCoord(StartY) + StartYPercent * FElementWidth / 100 - yprefix,
    pd);
  delete pd;
}
*/

void ButtonGrid::setFigure(IReadMatrix * Figure, int FigureXPer, int FigureYPer)
{
  //QMutexLocker locker(FMutex);
  FFigure = Figure;
  FFigureXPer = FigureXPer;
  FFigureYPer = FigureYPer;
}

void ButtonGrid::animateFigure(int XPercent, int YPercent, int StepNum)
{
  //if (StepNum == 1)
  {
    doAnimateFigure(XPercent, YPercent, StepNum);
  }
  /*else
  {
    if (FThread->running())
      FThread->wait();
    FThread->animate(this, XPercent, YPercent, StepNum);
  }*/
}

void ButtonGrid::doAnimateFigure(int XPercent, int YPercent, int StepNum)
{
/*#ifdef WIN32
  RequestWakeupLatency(LT_LOWEST_LATENCY);
#endif*/
  if (FAnimationSpeed == DesignOptions_MaxAnimationSpeed)
    StepNum = 1;
  //if (StepNum >= 1) qApp->lock();
  //QMutexLocker locker(FMutex);
  int PieceX1 = 0;
  int PieceY1 = 0;
  int PieceX2 = FFigure->getWidth() - 1;
  int PieceY2 = FFigure->getHeight() - 1;
  int StartX = FFigureXPer/100;
  int StartY = FFigureYPer/100;
  int StartXPixel = (FFigureXPer - StartX*100) * FElementWidth / 100;
  int StartYPixel = (FFigureYPer - StartY*100) * FElementWidth / 100;
  int EndXPixel = (FFigureXPer - StartX*100 + XPercent) * FElementWidth / 100;
  int EndYPixel = (FFigureYPer - StartY*100 + YPercent) * FElementWidth / 100;
  int DiffXPixel = EndXPixel - StartXPixel;
  int DiffYPixel = EndYPixel - StartYPixel;
  if (DiffXPixel == 0 && DiffYPixel == 0)
  {
    FFigureXPer += XPercent;
    FFigureYPer += YPercent;
    return;
  }

  IReadMatrix * figure = FFigure;

  QPaintDevice * pd = new QPixmap(
    (PieceX2 - PieceX1 + 1) * FElementWidth +
    abs(DiffXPixel),
    (PieceY2 - PieceY1 + 1) * FElementHeight +
    abs(DiffYPixel)
  );

  int xprefix = (DiffXPixel < 0 ? - DiffXPixel : 0);
  int yprefix = (DiffYPixel < 0 ? - DiffYPixel : 0);
  bitBlt(pd, 0, 0, & FPixmap,
    getXCoord(StartX) + StartXPixel - xprefix,
    getYCoord(StartY) + StartYPixel - yprefix,
    (PieceX2 - PieceX1 + 1) * FElementWidth +
    abs(DiffXPixel),
    (PieceY2 - PieceY1 + 1) * FElementHeight +
    abs(DiffYPixel)
  );

  //QPaintDevice * pd = this;

  int w, h;
  int erasexdiff = (DiffXPixel == 0) ? 0 : (DiffXPixel > 0 ? -1 : 1);
  int eraseydiff = (DiffYPixel == 0) ? 0 : (DiffYPixel > 0 ? -1 : 1);


#ifdef WIN32
  QTime cl = QTime::currentTime();
  int Div;
  switch (FAnimationSpeed)
  {
  case 1: Div = 5; break;
  case 2: Div = 7; break;
  case 3: Div = 10; break;
  case 4: Div = 15; break;
  case 5: Div = 20; break;
  case 6: Div = 25; break;
  case 7: Div = 50; break;
  default: Div = 50; break;
  }
  if (StepNum >= Div)
    StepNum = StepNum / Div;
  else
    StepNum = 1;
#endif
  for (int Step = 1; Step <= StepNum; Step ++)
  {
    // Drawing pieces at new position
    for (w = PieceX1; w <= PieceX2; w ++)
    for (h = PieceY1; h <= PieceY2; h ++)
    {
      if (figure->getElementAt(w, h))
      {
        bitBlt(pd,
          (w - PieceX1) * FElementWidth + xprefix +
          DiffXPixel * Step / StepNum,
          (h - PieceY1) * FElementHeight + yprefix +
          DiffYPixel * Step / StepNum,
          &(FElementPixmap[figure->getElementAt(w, h)])
        );
      }
    }

    int ew = abs(DiffXPixel * Step / StepNum -
      DiffXPixel * (Step - 1) / StepNum);
    int erasewidth = erasexdiff == 0 ? FElementWidth : ew;
    int eh = abs(DiffYPixel * Step / StepNum -
      DiffYPixel * (Step - 1) / StepNum);
    int eraseheight = eraseydiff == 0 ? FElementHeight : eh;
    int erasepixelxdiff = erasexdiff == -1 ? FElementWidth - erasewidth : 0;
    int erasepixelydiff = eraseydiff == -1 ? FElementHeight - eraseheight : 0;

    // Erasing pieces at old position

    for (w = PieceX1; w <= PieceX2; w ++)
    for (h = PieceY1; h <= PieceY2; h ++)
    {
      int erasew = w + erasexdiff;
      int eraseh = h + eraseydiff;
      bool neederase =
        figure->getElementAt(w, h) &&
        (erasew < PieceX1 || erasew > PieceX2 ||
        eraseh < PieceY1 || eraseh > PieceY2 ||
        figure->getElementAt(erasew, eraseh) == 0);

      if (neederase)
      {
        int X0 = (erasew - PieceX1) * FElementWidth +
          DiffXPixel * Step / StepNum +
          erasepixelxdiff;
        int Y0 = (eraseh - PieceY1) * FElementHeight +
          DiffYPixel * Step / StepNum +
          erasepixelydiff;

        int X = getXCoord(StartX) +
          StartXPixel + X0;
        int Y = getYCoord(StartY) +
          StartYPixel + Y0;

        bitBlt(pd, X0 + xprefix, Y0 + yprefix, & FBackground, X - FIndentX, Y - FIndentY,
          erasewidth, eraseheight);
        //qDebug("Eraseheight %d YPercent %d", (int) eraseheight, (int) YPercent);
      }
    }

    bitBlt(this,
      getXCoord(StartX) + StartXPixel - xprefix,
      getYCoord(StartY) + StartYPixel - yprefix,
      pd);
    //qApp->flush();
    qApp->flushX();

    /*QTime ncl = QTime::currentTime();
    while (StepNum > 1 && ncl >= cl && cl.msecsTo(ncl) < 1)
    {
      ncl = QTime::currentTime();
    }

    cl = ncl;*/
    //if (StepNum >= 1) qApp->unlock();
    if (StepNum > 1)
    {
      //qDebug("Delay %d", (int) Delay);
#ifdef WIN32
      QTime ncl = QTime::currentTime();
      while (StepNum > 1 && ncl >= cl && cl.msecsTo(ncl) < 1)
      {
        ncl = QTime::currentTime();
      }

      cl = ncl;
    /*for (int i = 0; i < DelayNum; i ++)
        ButtonGridAnimateThread::pubUSleep(1);*/
      /*DWORD OldTime = GetTickCount();
      while (GetTickCount() - OldTime < 1) {}*/
      //ButtonGridAnimateThread::pubUSleep(1);
#else
      int DelayNum;
      switch (FAnimationSpeed)
      {
      case 1: DelayNum = 300; break;
      case 2: DelayNum = 200; break;
      case 3: DelayNum = 150; break;
      case 4: DelayNum = 75; break;
      case 5: DelayNum = 20; break;
      case 6: DelayNum = 5; break;
      case 7: DelayNum = 0; break;
      default: DelayNum = 0; break;
      }
      for (int i = 0; i < DelayNum; i ++)
        ButtonGridAnimateThread::pubUSleep(1);
#endif
    }
    //if (StepNum >= 1) qApp->lock();
  }

  bitBlt(& FPixmap,
    getXCoord(StartX) + StartXPixel - xprefix,
    getYCoord(StartY) + StartYPixel - yprefix,
    pd);
  delete pd;
  //if (StepNum >= 1) qApp->unlock();

  FFigureXPer += XPercent;
  FFigureYPer += YPercent;
}

void ButtonGrid::eraseFigure()
{
  //QMutexLocker locker(FMutex);
  if (FFigure)
  {
    for (int x = 0; x < FFigure->getWidth(); x ++)
    for (int y = 0; y < FFigure->getHeight(); y ++)
    {
      if (FFigure->getElementAt(x, y))
        bitBlt(& FPixmap,
          FIndentX + FFigureXPer * FElementWidth / 100 +
            (x - FElementIndentX) * FElementWidth,
          FIndentY + FFigureYPer * FElementHeight / 100 +
            (y - FElementIndentY) * FElementHeight,
          & FBackground,
          FFigureXPer * FElementWidth / 100 +
            (x - FElementIndentX) * FElementWidth,
          FFigureYPer * FElementHeight / 100 +
            (y - FElementIndentY) * FElementHeight,
          FElementWidth, FElementHeight);
    }
  }
}

void ButtonGrid::drawFigure()
{
  //QMutexLocker locker(FMutex);
  if (FFigure)
  {
    for (int x = 0; x < FFigure->getWidth(); x ++)
    for (int y = 0; y < FFigure->getHeight(); y ++)
    {
      if (FFigure->getElementAt(x, y))
        bitBlt(& FPixmap,
          FIndentX + FFigureXPer * FElementWidth / 100 +
            (x - FElementIndentX) * FElementWidth,
          FIndentY + FFigureYPer * FElementHeight / 100 +
            (y - FElementIndentY) * FElementHeight,
          &(FElementPixmap[FFigure->getElementAt(x, y)]));
    }
  }
}

void ButtonGrid::blinkLines(int LineNum, int * Lines)
{
  if (LineNum == 0) return;

  int RepeatNumber = 3;
  switch (FAnimationSpeed)
  {
  case 5: RepeatNumber = 3; break;
  case 6: RepeatNumber = 3; break;
  case 7: RepeatNumber = 0; break;
  default: RepeatNumber = 3; break;
  }

  int DelayNum;
  switch (FAnimationSpeed)
  {
  case 1: DelayNum = 200; break;
  case 2: DelayNum = 100; break;
  case 3: DelayNum = 70; break;
  case 4: DelayNum = 50; break;
  case 5: DelayNum = 30; break;
  case 6: DelayNum = 20; break;
  case 7: DelayNum = 0; break;
  default: DelayNum = 0; break;
  }

#ifdef WIN32
  QTime cl = QTime::currentTime();

#endif

  for (int i = 0; i < RepeatNumber; i ++)
  {
    int line, j;
    for (line = 0; line < LineNum; line ++)
    {
      int y = Lines[line];
      for (int x = FElementIndentX; x < FElements.getWidth(); x ++)
      {
        bitBlt(& FPixmap, getXCoord(x), getYCoord(y),
          &(FElementEmptyPixmap[FElements.getElementAt(x, y)]));
        /*if (FHasColor)
        {
          int Color = FElements.getElementAt(x, y) + 1;
          if (Color > FColorNumber)
            Color = 1;
        }
        else
          bitBlt(& FPixmap, getXCoord(x), getYCoord(y),
            & FBackground, (x - FElementIndentX) * FElementWidth,
            (y - FElementIndentY) * FElementHeight,
            FElementWidth, FElementHeight);*/
      }
    }

    bitBlt(this, 0, 0, & FPixmap);
    //qApp->flush();
    qApp->flushX();
    #ifdef WIN32
    {
      QTime ncl = QTime::currentTime();
      while (ncl >= cl && cl.msecsTo(ncl) < DelayNum)
      {
        ncl = QTime::currentTime();
      }

      cl = ncl;
    }
    #else
    for (j = 0; j < DelayNum*100; j ++)
      ButtonGridAnimateThread::pubUSleep(1);
    #endif

    for (line = 0; line < LineNum; line ++)
    {
      int y = Lines[line];
      for (int x = FElementIndentX; x < FElements.getWidth(); x ++)
      {
        bitBlt(& FPixmap, getXCoord(x), getYCoord(y),
          &(FElementPixmap[FElements.getElementAt(x, y)]));
      }
    }

    bitBlt(this, 0, 0, & FPixmap);
    //qApp->flush();
    qApp->flushX();
    #ifdef WIN32
    {
      QTime ncl = QTime::currentTime();
      while (ncl >= cl && cl.msecsTo(ncl) < DelayNum)
      {
        ncl = QTime::currentTime();
      }

      cl = ncl;
    }
    #else
    for (j = 0; j < DelayNum*100; j ++)
      ButtonGridAnimateThread::pubUSleep(1);
    #endif
  }
}
