/***************************************************************************
                          buttongrid.h  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef BUTTONGRID_H
#define BUTTONGRID_H

#include <qwidget.h>
#include <qpixmap.h>
#include "intmatrix.h"

const int TET_ColorNum = 26;

//class QMutex;
class ButtonGridAnimateThread;
/**
  *@author Dmitry Osinovsky
  *  ButtonGrid is a widget that draws a grid, in its cells
  *    can be "buttons" of different colors (or can be no button
  *    at all). I call them "buttons" cause they look like
  *    3D buttons, not because you can press at them (you cannot).
  */

class ButtonGrid : public QWidget, public virtual IMatrix  {
   Q_OBJECT
protected: // Protected methods
  /**  */
  void paintEvent( QPaintEvent * );
  void rebuildElementPixmap();
  void repaintPixmap();
public:
  ButtonGrid(QWidget *parent=0, const char *name=0);
  ~ButtonGrid();

  QSizePolicy sizePolicy() const
  { return QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed); }

  QSize sizeHint () const
  { return QSize(FIndentX * 2 + FElementWidth * (getWidth() - FElementIndentX),
      FIndentY * 2 + FElementHeight * (getHeight() - FElementIndentY)); }

  void setSizes(int ByWidth, int ByHeight);
  void resize ( int w, int h );
  void setElementAt(int X, int Y, int Value);
  int getElementAt(int X, int Y) const;
  int getWidth() const { return FElements.getWidth(); }
  int getHeight() const { return FElements.getHeight(); }
  void refresh() { if (! FUpdateCounter) repaint(false); }
  void setAppearance(QString backgroundFileName, QString elementsFileName,
    int ElementSize, int GridInterval, bool HasColor, int ColorNumber, int AnimationSpeed);

  bool getHasColor();
  void setHasColor(bool Value);
  void clear() { FElements.clear(); repaintPixmap(); }
  void beginUpdate() { ++ FUpdateCounter; }
  void endUpdate() { if (! -- FUpdateCounter) refresh(); }
  void setElementIndents(int ElementIndentX, int ElementIndentY);

  /* Sets an applied pixmap at X and Y
   *   pixmap became property of ButtonGrid
   *   and will be deleted by it.
   */
  void drawPixmap(QPixmap * AppliedPixmap, int X, int Y);

  IReadMatrix * getFigure() { return FFigure; }
  void setFigure(IReadMatrix * Figure, int FigureXPer, int FigureYPer);
  void doAnimateFigure(int XPercent, int YPercent, int StepNum);
  void animateFigure(int XPercent, int YPercent, int StepNum);
  void blinkLines(int LineNum, int * Lines); // Lines (line numbers) must be sorted
  void eraseFigure();
  void drawFigure();

  /* Animates moving the piece of figure
   * from (StartX + StartXPercent/100, StartY + StartYPercent/100) point
   * by (XPercent/100, YPercent/100) elements in StepNum cadres.
   * StartX, StartY in coordinates of ButtonGrid
   * must be the same point as PieceX1, PieceX2 in the coordinates
   * of the figure.
   * !!! Only one of XPercent and YPercent can be != 0, so
   * !!! it asserts XPercent == 0 || YPercent == 0
   */
  /*void animate(IReadMatrix * figure, int StartX, int StartY,
    int StartXPercent, int StartYPercent,
    int XPercent, int YPercent, int StepNum,
    int PieceX1, int PieceY1, int PieceX2, int PieceY2);*/

  int getXCoord(int XElementCoord)
    { return FIndentX + (XElementCoord - FElementIndentX) * FElementWidth; }
  int getYCoord(int YElementCoord)
    { return FIndentY + (YElementCoord - FElementIndentY) * FElementHeight; }

  int getAnimationSpeed() const { return FAnimationSpeed; }
  void setAnimationSpeed(int AnimationSpeed) { FAnimationSpeed = AnimationSpeed; }
private:
  QPixmap * FElementPixmap;
  QPixmap * FElementEmptyPixmap;
  QPixmap * FAppliedPixmap;
  QPixmap FPixmap;
  QPixmap FSourceBackground;
  QPixmap FSourceElements;
  QPixmap FBackground;
  IReadMatrix * FFigure;
  bool FHasColor;
  IntMatrix FElements;
  int FElementWidth, FElementHeight;
  int FIndentX, FIndentY;
  int FUpdateCounter;
  int FElementIndentX, FElementIndentY;
  int FGridInterval;
  int FColorNumber;
  int FAppliedX, FAppliedY;
  int FFigureXPer, FFigureYPer;
  int FAnimationSpeed;
  /*QMutex * FMutex;
  ButtonGridAnimateThread * FThread;*/
};

/*
 * Draws button-like 3D element.
 * x, y - left & top
 * w, h - width & height
 * d - depth of 3D effect
 * Main - main color
 * Ligth - color of light part
 * Shadow - color of shadow part
 */
void drawElement(QPainter & p, int x, int y, int w, int h, int d,
  const QColor & Main, const QColor & Light, const QColor & Shadow);

#endif

