/***************************************************************************
                          configfile.cpp  -  description
                             -------------------
    begin                : Thu Feb 28 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <qcstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qmessagebox.h>
#include "configfile.h"

Section * ConfigFile::getSection(const QString & SectionName)
{
  Section * res;
  SectionMap::iterator p = Sections.find(SectionName);
  if (p == Sections.end())
  {
    res = new Section();
    Sections[SectionName] = res;
  }
  else
    res = p -> second;

  return res;
}

void ConfigFile::bindIntValue(const QString & SectionName, const QString & Name, int * pValue)
{
  Section * sect = getSection(SectionName);
  (*sect)[Name] = new ConfigIntValue(pValue);
}

void ConfigFile::bindBoolValue(const QString & SectionName, const QString & Name, bool * pValue)
{
  Section * sect = getSection(SectionName);
  (*sect)[Name] = new ConfigBoolValue(pValue);
}

void ConfigFile::bindStringValue(const QString & SectionName, const QString & Name, QString * pValue)
{
  Section * sect = getSection(SectionName);
  (*sect)[Name] = new ConfigStringValue(pValue);
}

QCString ConfigFile::getConfigFileDirectory()
{
  QCString home = getenv("HOME");
  if (home.isEmpty())
  {
    home = QDir::currentDirPath();
  }
  //QMessageBox::information(0L, "Directory", home);

  return home;
}

void ConfigFile::writeConfigFile()
{
  QCString filename = getConfigFileDirectory() + QCString("/.quadros");
  FILE * cf = fopen(filename, "w");
  SectionMap::iterator it = Sections.begin();
  while (it != Sections.end())
  {
    fprintf(cf, "[%s]\n", it->first.latin1());

    ConfigValueMap::iterator its = it->second->begin();
    while (its != it->second->end())
    {
			QCString q = QCString(its->first.latin1()) + "=" +
				its->second->getStrValue().local8Bit() + "\n";
      fprintf(cf, (const char *) q);
      ++ its;
    }

    fprintf(cf, "\n");
    ++ it;
  }

  fclose(cf);
}

void ConfigFile::readConfigFile()
{
  QCString filename = getConfigFileDirectory() + QCString("/.quadros");
  QFile cf(filename);
  if (cf.open(IO_ReadOnly))
  {
    QTextStream cft(& cf);
    QString s;
    Section * sect = NULL;
    while (! cft.atEnd())
    {
      s = cft.readLine();
      if (s[0] == '[')
      {
        s.remove(0, 1);
        s.remove(s.length() - 1, 1);
        sect = getSection(s);
      }
      else if (s.length() > 0)
      {
        int pos = s.find('=');
        if (pos == -1)
        {
          qWarning("Bad file format, no [ or = in non empty line");
        }
        else if (sect == NULL)
        {
          qWarning("Bad file format, no [section] before values");
        }
        else
        {
          QString name = s.left(pos);
          QString value = s.mid(pos + 1);
          ConfigValueMap::iterator it = sect -> find(name);
          if (it == sect -> end())
          {
            qWarning("Bad file format, no [ or = in non empty line");
          }
          else
          {
            it -> second -> setStrValue(value);
          }
        }
      }
    }
  }
  cf.close();
}
