/***************************************************************************
                          configfile.h  -  description
                             -------------------
    begin                : Thu Feb 28 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include <map>
#include <qstring.h>

using namespace std;

class ConfigValue
{
public:
  virtual QString getStrValue() = 0;
  virtual void setStrValue(const QString & value) = 0;
  virtual ~ConfigValue() {}
};

class ConfigIntValue: public ConfigValue
{
private:
  int * pValue;
public:
  ConfigIntValue(int * ApValue): pValue(ApValue) {}
  QString getStrValue() { return QString::number(*pValue); }
  void setStrValue(const QString & Value) { * pValue = Value.toInt(); }
};

class ConfigBoolValue: public ConfigValue
{
private:
  bool * pValue;
public:
  ConfigBoolValue(bool * ApValue): pValue(ApValue) {}
  QString getStrValue() { return QString::number(*pValue); }
  void setStrValue(const QString & Value) { * pValue = Value.toInt() != 0; }
};

class ConfigStringValue: public ConfigValue
{
private:
  QString * pValue;
public:
  ConfigStringValue(QString * ApValue): pValue(ApValue) {}
  QString getStrValue() { return *pValue; }
  void setStrValue(const QString & Value) { * pValue = Value; }
};

typedef map <QString, ConfigValue *> ConfigValueMap;
class Section: public ConfigValueMap
{
public:
  ~Section()
  { for (iterator p = begin(); p != end(); ++ p) delete p->second; }
};

typedef map<QString, Section *> SectionMap;
class ConfigFile
{
private:
  SectionMap Sections;
public:
	~ConfigFile()
	{ for (SectionMap::iterator p = Sections.begin(); p != Sections.end(); ++ p) delete p->second; }

	Section * getSection(const QString & SectionName);
	void bindIntValue(const QString & SectionName, const QString & Name, int * pValue);
	void bindBoolValue(const QString & SectionName, const QString & Name, bool * pValue);
	void bindStringValue(const QString & SectionName, const QString & Name, QString * pValue);
	void writeConfigFile();
	void readConfigFile();
  static QCString getConfigFileDirectory();
};

#endif
