/***************************************************************************
                          designoptions.cpp  -  description
                             -------------------
    begin                : Wed Feb 27 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qwidget.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include "keyedit.h"
#include "fileedit.h"

#include "configfile.h"
#include "designoptions.h"
#include "designoptionsdlg.h"
#include "figurecollections.h"
#include "figureoptions.h"

const char * KeyNames[DesignOptions_KeyNum] =
{
"Left", "Right", "Drop", "Rotate",
"Rotate backward", "Accelerate", "Jump to bound (use with Left or Right keys)"
};

DesignOptionsDlg::DesignOptionsDlg(QWidget *parent, const char *name )
  : QDialog(parent, name, true)
{
  QGridLayout * Layout = new QGridLayout(this,
    DesignOptions_KeyNum + 3, 2);
  Layout -> setMargin(11);
  Layout -> setSpacing(6);
  // Layout -> setAutoAdd(true);

  int i;
  for (i = 0; i < DesignOptions_KeyNum; i ++)
  {
    QLabel * label = new QLabel(tr(KeyNames[i]), this);
    Layout -> addWidget(label, i, 0);
    FKeys[i] = new KeyEdit(this);
    Layout -> addWidget(FKeys[i], i, 1);
  }

  QLabel * label = new QLabel(tr("Interval before key repeats starts"), this);
  Layout -> addWidget(label, i, 0);

  FBeforeKeyRepeatInterval = new QSlider(100, 1100, 200, 240,
    QSlider::Horizontal, this);
  FBeforeKeyRepeatInterval -> setTickmarks(QSlider::Below);
  Layout -> addWidget(FBeforeKeyRepeatInterval, i ++, 1);

  label = new QLabel(tr("Interval between key repeats"), this);
  Layout -> addWidget(label, i, 0);

  FKeyRepeatRate = new QSlider(20, 220, 40, 20,
    QSlider::Horizontal, this);
  FKeyRepeatRate -> setTickmarks(QSlider::Below);
  Layout -> addWidget(FKeyRepeatRate, i ++, 1);

  FSmartRotate = new QCheckBox(tr("Rotate near walls"), this);
  Layout -> addMultiCellWidget(FSmartRotate, i, i, 0, 1);
  i ++;

  FColor = new QCheckBox(tr("Color figures"), this);
  Layout -> addMultiCellWidget(FColor, i, i, 0, 1);
  i ++;

  label = new QLabel(tr("Background"), this);
  Layout -> addWidget(label, i, 0);
  FBackgroundImage = new FileEdit(this);
  Layout -> addWidget(FBackgroundImage, i ++, 1);

  label = new QLabel(tr("Figure element skins"), this);
  Layout -> addWidget(label, i, 0);
  FElementsImage = new FileEdit(this);
  Layout -> addWidget(FElementsImage, i ++, 1);

  label = new QLabel(tr("Figure element size (if no skin)"), this);
  Layout -> addWidget(label, i, 0);
  FElementSize = new QSpinBox(4, 50, 1, this);
  Layout -> addWidget(FElementSize, i ++, 1);

  label = new QLabel(tr("Dot grid"), this);
  Layout -> addWidget(label, i, 0);
  FGridInterval = new QComboBox(this);
  FGridInterval -> insertItem( tr("No grid") );
  FGridInterval -> insertItem( tr("Dot at every element") );
  FGridInterval -> insertItem( tr("Dot at every second element") );
  FGridInterval -> insertItem( tr("Dot at every third element") );
  Layout -> addWidget(FGridInterval, i ++, 1);

  /*FAnimate = new QCheckBox(tr("Turn animation on"), this);
  Layout -> addMultiCellWidget(FAnimate, i, i, 0, 1);
  i ++;*/

  label = new QLabel(tr("Animation speed"), this);
  Layout -> addWidget(label, i, 0);

  FAnimationSpeed = new QSlider(1, DesignOptions_MaxAnimationSpeed, 1, 3,
    QSlider::Horizontal, this);
  FAnimationSpeed -> setTickmarks(QSlider::Below);
  Layout -> addWidget(FAnimationSpeed, i ++, 1);

  FSmoothMovement = new QCheckBox(tr("Figure smooth movement"), this);
  Layout -> addMultiCellWidget(FSmoothMovement, i, i, 0, 1);
  i ++;

  QPushButton * FigOptButton = new QPushButton(tr("Figure options..."), this);
  Layout -> addMultiCellWidget(FigOptButton, i, i, 0, 1);
  i ++;

  QPushButton * OkButton = new QPushButton(tr("OK"), this);
  Layout -> addWidget(OkButton, i, 0, AlignRight);
  QPushButton * CancelButton = new QPushButton(tr("Cancel"), this);
  Layout -> addWidget(CancelButton, i, 1, AlignLeft);

  connect(OkButton, SIGNAL(clicked()), this, SLOT(accept()));
  connect(CancelButton, SIGNAL(clicked()), this, SLOT(reject()));
  connect(FigOptButton, SIGNAL(clicked()), this, SLOT(figureOptionsClicked()));
  OkButton -> setDefault(true);
}

DesignOptionsDlg::~DesignOptionsDlg()
{
}

void DesignOptionsDlg::figureOptionsClicked()
{
  editFigureOptions(this, FOpt->FFigureOptions[FFigureCollectionNumber]);
}

void DesignOptionsDlg::editOptions(const QString & caption,
  DesignOptions & opt, int FigureCollectionNumber)
{
  FFigureCollectionNumber = FigureCollectionNumber;
  FOpt = & opt;
  setCaption(caption);

  for (int i = 0; i < DesignOptions_KeyNum; i ++)
    FKeys[i] -> setKey(opt.KeyCodes[i]);

  FBeforeKeyRepeatInterval -> setValue(opt.BeforeKeyRepeatInterval);
  FKeyRepeatRate -> setValue(opt.KeyRepeatRate);
  FSmartRotate -> setChecked(opt.SmartRotate);
  FColor -> setChecked(opt.Color);
  FBackgroundImage -> setFileName(opt.BackgroundFileName);
  FElementsImage -> setFileName(opt.ElementsFileName);
  FGridInterval -> setCurrentItem(opt.GridInterval);
  FElementSize -> setValue(opt.ElementSize);
  FAnimationSpeed -> setValue(opt.AnimationSpeed);
  FSmoothMovement -> setChecked(opt.SmoothMovement);

  if (exec() == QDialog::Accepted)
  {
    for (int i = 0; i < DesignOptions_KeyNum; i ++)
      opt.KeyCodes[i] = FKeys[i] -> key();

    opt.BeforeKeyRepeatInterval = FBeforeKeyRepeatInterval -> value();
    opt.KeyRepeatRate = FKeyRepeatRate -> value();
    opt.SmartRotate = FSmartRotate -> isChecked();
    opt.Color = FColor -> isChecked();
    opt.BackgroundFileName = FBackgroundImage -> fileName();
    opt.ElementsFileName = FElementsImage -> fileName();
    opt.GridInterval = FGridInterval -> currentItem();
    opt.ElementSize = FElementSize -> value();
    opt.AnimationSpeed = FAnimationSpeed -> value();
    opt.SmoothMovement = FSmoothMovement -> isChecked();
    //Animate = FAnimate -> isChecked();
  }
}

void bindDesignOptions(ConfigFile & file, const QString & sect, DesignOptions & opt)
{
  int i;
  for (i = 0; i < DesignOptions_KeyNum; i ++)
  {
    file.bindIntValue(sect, "Key" + QString::number(i), opt.KeyCodes + i);
  }

  file.bindIntValue(sect, "BeforeKeyRepeatInterval", & opt.BeforeKeyRepeatInterval);
  file.bindIntValue(sect, "KeyRepeatRate", & opt.KeyRepeatRate);
  file.bindBoolValue(sect, "SmartRotate", & opt.SmartRotate);
  file.bindBoolValue(sect, "Color", & opt.Color);
  file.bindStringValue(sect, "Background", & opt.BackgroundFileName);
  file.bindStringValue(sect, "ElementSkin", & opt.ElementsFileName);
  file.bindIntValue(sect, "ElementSize", & opt.ElementSize);
  file.bindIntValue(sect, "GridInterval", & opt.GridInterval);
  file.bindIntValue(sect, "AnimationSpeed", & opt.AnimationSpeed);
  file.bindBoolValue(sect, "SmoothMovement", & opt.SmoothMovement);

  for (unsigned j = 0; j < opt.FFigureOptions.size(); j ++)
  {
    bindFigureOptions(file, sect + FigureCollections::instance()->at(j)->code(), opt.FFigureOptions[j]);
  }
}

void editDesignOptions(QWidget * parent, const QString & caption,
  DesignOptions & opt, int FigureCollectionNumber)
{
  DesignOptionsDlg dlg(parent);
  dlg.editOptions(caption, opt, FigureCollectionNumber);
}

DesignOptions::DesignOptions()
{
  FFigureOptions.resize(FigureCollections::instance()->collectionNumber());
  for (int i = 0; i < FigureCollections::instance()->collectionNumber(); i ++)
  {
    FFigureOptions[i].initialize(FigureCollections::instance()->at(i));
  }
  qDebug("DesignOptions::DesignOptions end");
}
