/***************************************************************************
                          figure.cpp  -  description
                             -------------------
    begin                : Mon Mar 18 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <qstring.h>
#include <qtextstream.h>
#include "figure.h"
#include "profiler.h"

void Figure::debugPrint()
{
  int h, w;
  QString str;
  QTextOStream OStr(& str);

  for (h = 0; h < getHeight(); h ++)
  {
    for (w = 0; w < getWidth(); w ++)
    {
      OStr << at(w, h);
    }
    OStr << "\n";
  }

  qDebug(str);
}

void Figure::initialize(int Size, int * Contents, int ContentsSize)
{
  int w, h;

  assert(ContentsSize <= Size);
  setSizes(Size, Size);
  FElementNum = 0;

  // Minimum rectangle in to which figure can be inscribed.
  int x1 = Size, y1 = Size, x2 = 0, y2 = 0;

  for (h = 0; h < ContentsSize; h ++)
  {
    for (w = 0; w < ContentsSize; w ++)
    {
      if (Contents[w * ContentsSize + h])
      {
        if (w < x1) x1 = w;
        if (h < y1) y1 = h;
        if (w > x2) x2 = w;
        if (h > y2) y2 = h;
        FColor = Contents[w * ContentsSize + h];
        FElementNum ++;
      }

      setElementAt(w, h, Contents[w * ContentsSize + h]);
    }
  }

  //qDebug("x1=%d y1=%d x2=%d y2=%d", x1, y1, x2, y2);

  // Rotate square.
  FRotateCoord1 = x1 < y1 ? x1 : y1;
  FRotateCoord2 = x2 > y2 ? x2 : y2;

  // Determine if figure is symmetric.
  FIsSymmetric = true;
  for (w = x1; w <= x2; w ++)
  for (h = y1; h <= y2; h ++)
  {
    int oppositew = x2 - (w - x1),
        oppositeh = y2 - (h - y1);
    if (at(w, h) != at(oppositew, oppositeh))
    {
      FIsSymmetric = false;
      break;
    }
  }

  FNeedMoveAfterRotate = false;
  if (FIsSymmetric)
  {
    // This formulae can be derived from
    // applying rotate to (x2, y2) point twice
    // and computing difference between (x1, y2) and
    // obtained in this way point
    FXShift = (x1 - FRotateCoord1) - (FRotateCoord2 - FRotateCoord1) +
      (x2 - FRotateCoord1);
    FYShift = (y1 - FRotateCoord1) - (FRotateCoord2 - FRotateCoord1) +
      (y2 - FRotateCoord1);
  }
  else
  {
    FXShift = 0;
    FYShift = 0;
  }

  // Compute whether figure has flat side
  bool y1IsFlatSide = true, y2IsFlatSide = true,
    x1IsFlatSide = true, x2IsFlatSide = true;

  for (w = x1; w <= x2; w ++)
  {
    if (!at(w, y1))
      y1IsFlatSide = false;
    if (!at(w, y2))
      y2IsFlatSide = false;
  }

  for (h = y1; h <= y2; h ++)
  {
    if (!at(x1, h))
      x1IsFlatSide = false;
    if (!at(x2, h))
      x2IsFlatSide = false;
  }

  FHasFlatSide = y1IsFlatSide || y2IsFlatSide || x1IsFlatSide ||
    x2IsFlatSide;
}

void Figure::rotate()
{
  if (! FAllowRotate)
    return;

  Figure buf;
  int i, j,
    xshift = FNeedMoveAfterRotate ? FXShift : 0,
    yshift = FNeedMoveAfterRotate ? FYShift : 0;

  buf.setSizes(size(), size());

  for (j = FRotateCoord1; j <= FRotateCoord2; j ++)
  {
    for (i = FRotateCoord1; i <= FRotateCoord2; i ++)
    {
      if (at(i, j))
      {
        assert(j + xshift <= FRotateCoord2);
        assert(FRotateCoord2 - (i - FRotateCoord1) + yshift <= FRotateCoord2);
        buf.setElementAt(j + xshift, FRotateCoord2 - (i - FRotateCoord1) + yshift,
          at(i, j));
      }
    }
  }

  buf.copyTo(this);
  if (FIsSymmetric)
    FNeedMoveAfterRotate = ! FNeedMoveAfterRotate;
}

void Figure::draw(IMatrix * MainPainter, int X, int Y,
  IMatrix * HelpLinePainter) const
{
  for (int w = 0; w < size(); w ++)
  {
    int BottomColor = 0;
    for (int h = 0; h < size(); h ++)
    {
      /*if (at(w, h) > 26)
      {
        QString str;
        QTextOStream OStr(& str);

        for (h = 0; h < size(); h ++)
        {
          for (w = 0; w < size(); w ++)
          {
            OStr << at(w, h);
          }
          OStr << "\n";
        }

        //qDebug(str);
        assert(false);
      }*/

      if (at(w, h))
      {
        if (MainPainter)
          MainPainter -> setElementAt(X + w, Y + h, at(w, h));
        BottomColor = at(w, h);
      }
    }

    if (BottomColor && HelpLinePainter)
    {
      HelpLinePainter -> setElementAt(X + w, 0, BottomColor);
    }
  }
}

void Figure::erase(IMatrix * MainPainter, int X, int Y,
  IMatrix * HelpLinePainter) const
{
  for (int w = 0; w < size(); w ++)
  {
    int BottomColor = 0;
    for (int h = 0; h < size(); h ++)
    {
      if (at(w, h))
      {
        if (MainPainter)
          MainPainter -> setElementAt(X + w, Y + h, 0);
        BottomColor = at(w, h);
      }
    }

    if (BottomColor && HelpLinePainter)
    {
      HelpLinePainter -> setElementAt(X + w, 0, 0);
    }
  }
}

void Figure::changeColor(int newColor)
{
  assert(newColor != 0);
  for (int w = 0; w < size(); w ++)
  {
    for (int h = 0; h < size(); h ++)
    {
      if (at(w, h))
        setElementAt(w, h, newColor);
    }
  }

  FColor = newColor;
}

