/***************************************************************************
                          figure.h  -  description
                             -------------------
    begin                : Mon Mar 18 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef FIGURE_H
#define FIGURE_H

#include "intmatrix.h"

class Figure : protected IntMatrix, public virtual IReadMatrix {
public:
  Figure(): FAllowRotate(true) {}
  ~Figure() {}

  /* Here Size is determined by the figure collection
   *   and ContentsSize*ContentsSize is the size of
   *   Contents array.
   *   ContentsSize may be smaller than the Size.
   */
  void initialize(int Size, int * Contents, int ContentsSize);
  void debugPrint();
  void rotate();
  void draw(IMatrix * MainPainter, int X, int Y,
    IMatrix * HelpLinePainter = NULL) const;
  void erase(IMatrix * MainPainter, int X, int Y,
    IMatrix * HelpLinePainter = NULL) const;
  void changeColor(int newColor);

  int size() const { return getWidth(); }
  int at(int X, int Y) const { return getElementAt(X, Y); }
  int elementNum() const { return FElementNum; }
  void copyFigureTo(Figure * Clone) const
  {
    copyTo(Clone);
    Clone->FIsSymmetric = FIsSymmetric;
    Clone->FNeedMoveAfterRotate = FNeedMoveAfterRotate;
    Clone->FXShift = FXShift;
    Clone->FYShift = FYShift;
    Clone->FRotateCoord1 = FRotateCoord1;
    Clone->FRotateCoord2 = FRotateCoord2;
    Clone->FColor = FColor;
    Clone->FHasFlatSide = FHasFlatSide;
    Clone->FElementNum = FElementNum;
    Clone->FAllowRotate = FAllowRotate;
  }

  int color() const { return FColor; }
  bool hasFlatSide() const { return FHasFlatSide; }
  bool getAllowRotate() const { return FAllowRotate; }
  void setAllowRotate(bool AllowRotate) { FAllowRotate = AllowRotate; }
private:
  /* FIsSymmetric = true if after two rotates
   * figure returns to the original form
   * (but it may be shifted from original position).
   * Examples of symmetric figures: S-like, Z-like, Stick 4x1, block 2x2
   * If figure is symmetric, then after second rotate we may need
   * to shift it to the original position.
   * So before first rotate FNeedMoveAfterRotate = false,
   * before second rotate FNeedMoveAfterRotate = true.
   * For non-symmetric figures FNeedMoveAfterRotate always = false.
   * FXShift and FYShift is shift values.
   * For non-symmetric figures FXShift = 0, FYShift = 0.
   */
  bool FIsSymmetric;
  bool FNeedMoveAfterRotate;
  int FXShift, FYShift;

  /* Rotating figure inside square (0, 0) - (size() - 1, size() - 1)
   * may be bad idea, cause if size() is even (for tetris size() = 4)
   * and size of the figure is odd (for T- and L- like figures size = 3)
   * or vice versa then we cannot centre that figure in our square.
   *
   * So we need to compute rotate square. Here the following algorithm
   * is used:
   * We take minimum rectangle (x1, y1) - (x2, y2) into which figure
   * can be inscribed.
   * Lets compute FRotateCoord1 = min(x1, y1), FRotateCoord2 = max(x2, y2),
   * then (RotateCoord1, RotateCoord1) - (RotateCoord2, RotateCoord2)
   * will be our rotate square.
   */
  int FRotateCoord1, FRotateCoord2;

  int FColor;
  bool FHasFlatSide;
  int FElementNum;
  bool FAllowRotate;
};

#endif
