/***************************************************************************
                          figureoptions.cpp  -  description
                             -------------------
    begin                : Sun Mar 17 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qlayout.h>
#include "figureoptions.h"
#include "figureoptionsdlg.h"
#include "configfile.h"

FigureOptionsDlg::FigureOptionsDlg(QWidget *parent, const char *name ):
  QDialog(parent, name, true)
{
}

FigureOptionsDlg::~FigureOptionsDlg()
{
  qDebug("FigureOptionsDlg::~FigureOptionsDlg end");
}

void FigureOptionsDlg::editOptions(FigureOptions & opt)
{
  QVBoxLayout * Layout = new QVBoxLayout(this);
  Layout -> setMargin(11);
  Layout -> setSpacing(6);

  QGrid * Grid = new QGrid(4, this);
  Layout -> addWidget(Grid);
  FElements = new FigureElement *[opt.Figures->figureNumber()];
  for (int i = 0; i < opt.Figures->figureNumber(); i ++)
  {
    FElements[i] = new FigureElement(Grid);
    FElements[i]->initialize(opt.Options[i].Color,
      opt.Options[i].RotatePosition, opt.Figures->at(i));
  }

  QHBoxLayout * BottomLayout = new QHBoxLayout(this);
  Layout -> addLayout(BottomLayout);
  QPushButton * OkBtn = new QPushButton(tr("OK"), this);
  BottomLayout->addWidget(OkBtn, 0, AlignRight);
  QPushButton * CancelBtn = new QPushButton(tr("Cancel"), this);
  BottomLayout->addWidget(CancelBtn, 0, AlignLeft);

  connect(OkBtn, SIGNAL(clicked()), this, SLOT(accept()));
  connect(CancelBtn, SIGNAL(clicked()), this, SLOT(reject()));

  if (exec() == Accepted)
  {
    for (int i = 0; i < opt.Figures->figureNumber(); i ++)
    {
      opt.Options[i].Color = FElements[i]->color();
      opt.Options[i].RotatePosition = FElements[i]->rotate();
    }
  }

  delete[] FElements;
  qDebug("After FElements");
}

void editFigureOptions(QWidget * parent, FigureOptions & opt)
{
  FigureOptionsDlg dlg(parent);
  dlg.editOptions(opt);
}

void bindFigureOptions(ConfigFile & file, const QString & sect, FigureOptions & opt)
{
  for (int i = 0; i < opt.Figures->figureNumber(); i ++)
  {
    file.bindIntValue(sect, "Color" + QString::number(i), & opt.Options[i].Color);
    file.bindIntValue(sect, "Rotate" + QString::number(i), & opt.Options[i].RotatePosition);
  }
}

FigureElement::FigureElement(QWidget *parent):
  QHBox(parent)
{
  FGrid = new ButtonGrid(this);
  QVBox * Box = new QVBox(this);
  QPushButton * RotateBtn = new QPushButton(tr("Rotate"), Box);
  QPushButton * ColorBtn = new QPushButton(tr("Color"), Box);
  connect(RotateBtn, SIGNAL(clicked()), this, SLOT(rotateClicked()));
  connect(ColorBtn, SIGNAL(clicked()), this, SLOT(colorClicked()));
}

FigureElement::~FigureElement()
{
  qDebug("FigureElement::~FigureElement end");
}

void FigureElement::initialize(int Color, int Rotate,
  const Figure * Figure)
{
  Figure->copyFigureTo(& FFigure);
  FGrid->setAppearance("", "", 12, 0, true, TET_ColorNum, 1);
  FGrid->setSizes(FFigure.size(), FFigure.size());

  if (Color != -1)
    FFigure.changeColor(Color);

  for (int i = 0; i < Rotate; i ++)
    FFigure.rotate();
  FRotate = Rotate;
  FFigure.draw(FGrid, 0, 0);
  FGrid->refresh();
}

void FigureElement::rotateClicked()
{
  FFigure.erase(FGrid, 0, 0);
  FFigure.rotate();
  FFigure.draw(FGrid, 0, 0);
  FGrid->refresh();
  FRotate ++;
  if (FRotate == 4)
    FRotate = 0;
}

void FigureElement::colorClicked()
{
  int Color = FFigure.color() + 1;
  if (Color > TET_ColorNum)
    Color = 1;
  FFigure.changeColor(Color);
  FFigure.draw(FGrid, 0, 0);
  FGrid->refresh();
}
