/***************************************************************************
                          fileedit.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qfiledialog.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qurl.h>
#include <qapplication.h>
#include <qpainter.h>

#include "fileedit.h"

FileEdit::FileEdit(QWidget *parent, const char *name ) : QHBox(parent,name)
{
  FEdit = new QLineEdit(this);
  FEdit -> setReadOnly(true);
  FButton = new QPushButton("...", this);
  FButton -> setMaximumSize( QSize(20, 32767) );
  FClearButton = new QPushButton("X", this);
  FClearButton -> setMaximumSize( QSize(20, 32767) );
  connect(FButton, SIGNAL(clicked()), this, SLOT(buttonClicked()));
  connect(FClearButton, SIGNAL(clicked()), this, SLOT(clearButtonClicked()));
}

FileEdit::~FileEdit(){
}

class ImagePreview : public QWidget, public QFilePreview
{
private:
  QPixmap pm;
public:
  ImagePreview(QWidget * parent) : QWidget(parent), QFilePreview() {}
    // reimplementation from QFilePreview
  void previewUrl( const QUrl &url )
  {
    qDebug(url.toString(false, false));
    if (!pm.load(url.toString(false, false)))
    {
      pm.resize(0, 0);
    }
    paintEvent(0);
  }

  void paintEvent( QPaintEvent * )
  {
    QPainter p;
    p.begin(this);
    p.fillRect(0, 0, width(), height(), QBrush(black));
    p.end();
    bitBlt(this, 0, 0, & pm);
  }
};

void FileEdit::buttonClicked()
{
  QString workingDirectory;
  if ( !FEdit -> text().isEmpty() )
  {
	  QUrlOperator u( FEdit -> text() );
    if ( u.isLocalFile() && QFileInfo( u.path() ).isDir() ) {
	    workingDirectory = FEdit -> text();
    } else {
	    workingDirectory = u.toString();
  	}
  }

  if ( workingDirectory.isNull() )
  	workingDirectory = QDir::currentDirPath();
  	
  QFileDialog *dlg = new QFileDialog( workingDirectory, QString::null,
		this, 0, true );
		
  if ( qApp->mainWidget() && qApp->mainWidget()->icon() && !qApp->mainWidget()->icon()->isNull() )
  	dlg->setIcon( *qApp->mainWidget()->icon() );
	dlg->setCaption( QFileDialog::tr( "Open" ) );
  dlg->setMode( QFileDialog::ExistingFile );
  ImagePreview * pv = new ImagePreview(dlg);
  dlg->setContentsPreview(pv, pv);
  dlg->setContentsPreviewEnabled(true);
  dlg->setPreviewMode( QFileDialog::Contents );
  	
  if ( dlg->exec() == QDialog::Accepted )
  {
  	FEdit -> setText(dlg->selectedFile());
  }

  delete dlg;
}

void FileEdit::clearButtonClicked()
{
  FEdit -> setText("");
}

QString FileEdit::fileName()
{
  return FEdit -> text();
}

void FileEdit::setFileName(const QString & Value)
{
  FEdit -> setText(Value);
}
