/***************************************************************************
                          highscore.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qinputdialog.h>
#include <qmessagebox.h>

#include "highscore.h"
#include "skilloptions.h"
#include "quadrosprotocol.h"
#include "figurecollections.h"
#include "configfile.h"

const int TET_MaxHighScoreEntries = 10;
HighScore::HighScore(int FigureCollection, QWidget *parent, const char *name )
  : QDialog(parent, name, true)
{
  FFigureCollection = FigureCollection;

  setCaption(FigureCollections::instance()->at(FigureCollection)->name() + " high score");
  QVBoxLayout * Layout = new QVBoxLayout(this);
  Layout -> setMargin(11);
  Layout -> setSpacing(6);
  Layout -> setAutoAdd(true);
  FMainScore = new QListView(this);
  FMainScore -> addColumn("Score", -1);
  FMainScore -> setColumnAlignment(0, AlignRight);
  FMainScore -> addColumn("Lines", -1);
  FMainScore -> setColumnAlignment(1, AlignHCenter);
  FMainScore -> addColumn("Player", -1);
  FMainScore -> addColumn("Start level", -1);
  FMainScore -> setColumnAlignment(3, AlignHCenter);
  FMainScore -> addColumn("Trash lines", -1);
  FMainScore -> setColumnAlignment(4, AlignHCenter);
  FMainScore -> setSorting(-1);

  QHBox * Box = new QHBox(this);
  QPushButton * OKBtn = new QPushButton(tr("OK"), Box);
  QPushButton * ResetBtn = new QPushButton(tr("Reset"), Box);
  OKBtn -> setDefault(true);
  connect(OKBtn, SIGNAL(clicked()), this, SLOT(accept()));
  connect(ResetBtn, SIGNAL(clicked()), this, SLOT(slotResetHighScore()));
}

HighScore::~HighScore()
{
}

void HighScore::showHighScore()
{
  loadHighScore();
  exec();
}

void HighScore::gameEnd(long Score, int Lines,
  const SkillOptions & StartOptions)
{
  loadHighScore();

  QListViewItem * item = FMainScore -> firstChild();
  QListViewItem * pos = 0;
  int i;
  for (i = 0; i < FMainScore -> childCount(); i ++)
  {
    QString sScore = item -> text(0),
      Lines = item -> text(1),
      PlayerName = item -> text(2),
      StartLevel = item -> text(3),
      TrashLines = item -> text(4);

    if (sScore.toLong() < Score && pos == 0)
    {
      pos = item;
    }

    if (item -> nextSibling() != 0)
      item = item -> nextSibling();
  }

  if (pos || FMainScore -> childCount() < TET_MaxHighScoreEntries)
  {
    if (pos)
      pos = pos -> itemAbove();
    else
      pos = item;

    bool ok = false;
    QString PlayerName = QInputDialog::getText(tr("New high score!"), tr("Enter your name"),
      QLineEdit::Normal, FLastPlayerName, & ok, parentWidget());
    if (ok && !PlayerName.isEmpty())
    {
      FLastPlayerName = PlayerName;

       new QListViewItem(FMainScore, pos, QString::number(Score),
         QString::number(Lines), PlayerName,
        QString::number(StartOptions.StartLevel),
        StartOptions.TrashLines ? tr("Yes") : tr("No"));

      if (FMainScore -> childCount() + 1 > TET_MaxHighScoreEntries)
        delete item;

      saveHighScore();
    }
  }

  exec();
}

void HighScore::slotResetHighScore()
{
  if (QMessageBox::warning(this, tr("Confirmation"), tr("Reset the high scores?"),
    tr("Yes"), tr("No"), QString::null, 1, 1) == 0)
  {
    resetHighScore();
    saveHighScore();
  }
}

void HighScore::resetHighScore()
{
  FMainScore -> clear();
  for (int i = 0; i < TET_MaxHighScoreEntries; i ++)
  {
    QString Score = QString::number(100000 * (i + 1)), Lines = "",
      PlayerName = "CowboyNeal", StartLevel = "", TrashLines = "";
    new QListViewItem(FMainScore, Score, Lines, PlayerName,
      StartLevel, TrashLines);
  }
}

void HighScore::loadHighScore()
{
  QCString filename = ConfigFile::getConfigFileDirectory() + QCString("/.quadros.highscore.") +
    QCString(FigureCollections::instance()->at(FFigureCollection)->code());
  qDebug(filename);
  QFile hsf(filename);
  if (hsf.open(IO_ReadOnly))
  {
    QDataStream hsds(& hsf);
    int i;
    int Version[4];
    for (i = 0; i < 4; i ++)
    {
      hsds >> Version[i];
    }

    int count;
    FMainScore -> clear();
    hsds >> FLastPlayerName >> count;
    QListViewItem * item = 0;
    for (i = 0; i < count; i ++)
    {
      QString Score, Lines, PlayerName, StartLevel, TrashLines;
      hsds >> Score >> Lines >> PlayerName >> StartLevel >> TrashLines;
      item = new QListViewItem(FMainScore, item, Score, Lines, PlayerName,
        StartLevel, TrashLines);
    }
  }
  else
  {
    FLastPlayerName = "";
    resetHighScore();
  }
}

void HighScore::saveHighScore()
{
  QCString filename = ConfigFile::getConfigFileDirectory() + QCString("/.quadros.highscore.") +
    QCString(FigureCollections::instance()->at(FFigureCollection)->code());
  QFile hsf(filename);
  if (hsf.open(IO_WriteOnly))
  {
    QDataStream hsds(& hsf);
    int i;
    for (i = 0; i < 4; i ++)
    {
      hsds << TET_Version[i];
    }

    hsds << FLastPlayerName << FMainScore -> childCount();
    QListViewItem * item = FMainScore -> firstChild();
    for (i = 0; i < FMainScore -> childCount(); i ++)
    {
      QString Score = item -> text(0),
        Lines = item -> text(1),
        PlayerName = item -> text(2),
        StartLevel = item -> text(3),
        TrashLines = item -> text(4);
      hsds << Score << Lines << PlayerName << StartLevel << TrashLines;

      item = item -> nextSibling();
    }
  }
}

void showHighScore(QWidget * parent, int FigureCollection)
{
  HighScore dlg(FigureCollection, parent);
  dlg.showHighScore();
}

void gameEndHighScore(QWidget * parent,
  long Score, int Lines,
  const SkillOptions & StartOptions)
{
  HighScore dlg(StartOptions.FigureCollectionNumber, parent);
  dlg.gameEnd(Score, Lines, StartOptions);
}

