/***************************************************************************
                          intmatrix.h  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef INTMATRIX_H
#define INTMATRIX_H

#include <string.h>

class IReadMatrix
{
public:
  virtual int getElementAt(int X, int Y) const = 0;
  virtual int getWidth() const = 0;
  virtual int getHeight() const = 0;
  virtual ~IReadMatrix() {}
};

class IMatrix: public virtual IReadMatrix
{
public:
  virtual void setSizes(int Width, int Height) = 0;
  virtual void setElementAt(int X, int Y, int Value) = 0;
  virtual void refresh() {}
  virtual void beginUpdate() {}
  virtual void endUpdate() {}
  virtual void clear() = 0;

  virtual void setFigure(IReadMatrix * /*Figure*/, int /*FigureXPer*/,
    int /*FigureYPer*/) {}
  virtual void animateFigure(int /*XPercent*/, int /*YPercent*/,
    int /*StepNum*/) {}
  virtual void eraseFigure() {}
  virtual void drawFigure() {}
  virtual void blinkLines(int, int *) {}

  /*IReadMatrix * figure, int StartX, int StartY,
    int StartXPercent, int StartYPercent,
    int XPercent, int YPercent, int StepNum,
    int PieceX1, int PieceY1, int PieceX2, int PieceY2

    see buttongrid.h for details
    */
  /*virtual void animate(IReadMatrix *, int, int,
    int, int, int, int, int, int, int, int, int) {}*/
};

class IntMatrix: public virtual IMatrix {
private:
  int FWidth, FHeight;
  int * FElements;
public:
  IntMatrix(): FWidth(0), FHeight(0), FElements(NULL) {}
  ~IntMatrix() { if (FElements) delete[] FElements; }

  void setSizes(int Width, int Height)
  {
    FWidth = Width;
    FHeight = Height;
    if (FElements) delete[] FElements;
    FElements = new int[FWidth * FHeight];
    clear();
  }

  int getElementAt(int X, int Y) const
    { return FElements[Y * FWidth + X]; }

  void setElementAt(int X, int Y, int Value)
    { FElements[Y * FWidth + X] = Value; }

  int getWidth() const
    { return FWidth; }

  int getHeight() const
    { return FHeight; }

  void clear()
  {
    memset(FElements, 0, FWidth * FHeight * sizeof(int));
  }

  void copyTo(IntMatrix * copy) const
  {
    if (copy->FWidth != FWidth || copy->FHeight != FHeight)
      copy->setSizes(FWidth, FHeight);

    memmove(copy->FElements, FElements, FWidth * FHeight * sizeof(int));
  }
};

#endif
