/***************************************************************************
                          multioptions.cpp  -  description
                             -------------------
    begin                : Fri Mar 1 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qwidget.h>

#include "skilloptions.h"
#include "multiplayer.h"
#include "multioptions.h"
#include "multioptionsdlg.h"
#include "configfile.h"
#include "figurecollections.h"

MultiOptionsDlg::MultiOptionsDlg(QWidget *parent, const char *name ) : MultiplayerDialog(parent,name,true)
{
  for (int j = 0; j < FigureCollections::instance()->collectionNumber() - 1; j ++)
  {
    FiguresCombo->insertItem(FigureCollections::instance()->at(j)->name());
  }

  connect(LocalPlayBtn, SIGNAL(clicked()), this, SLOT(localPlayClicked()));
}

MultiOptionsDlg::~MultiOptionsDlg()
{
}

void MultiOptionsDlg::localPlayClicked()
{
  SecondPlayerChk -> setChecked(true);
}

bool MultiOptionsDlg::askMultiplayerOptions(NetOptions & opt1, NetDefaultSettings & opt2,
  SkillOptions & opt3)
{
  CountdownSpin -> setValue(opt1.SecBeforeStart);
  PlayTillSpin -> setValue(opt1.MaxScores);
  ThrowToTeamCombo -> setCurrentItem((int)opt1.TeamToThrow);
  ThrowToPlayerCombo -> setCurrentItem((int)opt1.PlayerToThrow);
  TraceChk -> setChecked(opt1.TraceMessages);
  FirstPlayerNameEdit -> setText(opt2.PlayerName1);
  FirstTeamEdit -> setText(opt2.TeamName1);
  SecondPlayerNameEdit -> setText(opt2.PlayerName2);
  SecondTeamEdit -> setText(opt2.TeamName2);
  ServerEdit -> setText(opt2.Host);
  PortEdit -> setText(opt2.Port);
  SecondPlayerChk -> setChecked(opt2.SecondPlayer);
  LocalPlayBtn -> setOn(opt2.GameType == NetDefaultSettings::gtLocalPlay);
  CreatePlayBtn -> setOn(opt2.GameType == NetDefaultSettings::gtCreateNetGame);
  JoinPlayBtn -> setOn(opt2.GameType == NetDefaultSettings::gtJoinNetGame);
  GameLevelSpin -> setValue(opt3.StartLevel);
  IncreaseLevelChk -> setChecked(opt3.IncreaseLevel);
  FiguresCombo -> setCurrentItem(opt3.FigureCollectionNumber);

  bool res = exec() == Accepted;
  if (res)
  {
    opt1.SecBeforeStart = CountdownSpin->value();
    opt1.MaxScores = PlayTillSpin->value();
    opt1.TeamToThrow = ThrowTo(ThrowToTeamCombo->currentItem());
    opt1.PlayerToThrow = ThrowTo(ThrowToPlayerCombo->currentItem());
    opt1.PlayerExitScores = 0;
    opt1.TeamExitScores = 1;
    opt1.TraceMessages = TraceChk -> isOn();

    opt2.PlayerName1 = FirstPlayerNameEdit -> text();
    opt2.TeamName1 = FirstTeamEdit -> text();
    opt2.PlayerName2 = SecondPlayerNameEdit -> text();
    opt2.TeamName2 = SecondTeamEdit -> text();
    opt2.Host = ServerEdit -> text();
    opt2.Port = PortEdit -> text();
    opt2.SecondPlayer = SecondPlayerChk -> isOn();
    if (LocalPlayBtn -> isOn())
      opt2.GameType = NetDefaultSettings::gtLocalPlay;
    else if (CreatePlayBtn -> isOn())
      opt2.GameType = NetDefaultSettings::gtCreateNetGame;
    else if (JoinPlayBtn -> isOn())
      opt2.GameType = NetDefaultSettings::gtJoinNetGame;
    else
      assert(false);

    opt3.StartLevel = GameLevelSpin->value();
    opt3.IncreaseLevel = IncreaseLevelChk->isOn();
    opt3.TrashLines = false;
    opt3.FieldHeight = 25;
    opt3.FigureCollectionNumber = FiguresCombo -> currentItem();
  }

  return res;
}

void bindNetOptions(ConfigFile & file, const QString & sect,
  NetOptions & opt1, NetDefaultSettings & opt2)
{
  file.bindIntValue(sect, "CountDown", & opt1.SecBeforeStart);
  file.bindIntValue(sect, "PlayTill", & opt1.MaxScores);
  file.bindIntValue(sect, "TeamToThrow", (int *) & opt1.TeamToThrow);
  file.bindIntValue(sect, "PlayerToThrow", (int *) & opt1.PlayerToThrow);
  file.bindBoolValue(sect, "TraceMessages", & opt1.TraceMessages);
  file.bindStringValue(sect, "PlayerName1", & opt2.PlayerName1);
  file.bindStringValue(sect, "TeamName1", & opt2.TeamName1);
  file.bindStringValue(sect, "PlayerName2", & opt2.PlayerName2);
  file.bindStringValue(sect, "TeamName2", & opt2.TeamName2);
  file.bindStringValue(sect, "Host", & opt2.Host);
  file.bindStringValue(sect, "Port", & opt2.Port);
  file.bindBoolValue(sect, "SecondPlayer", & opt2.SecondPlayer);
  file.bindIntValue(sect, "GameType", (int *) & opt2.GameType);
}

bool askMultiplayerOptions(QWidget * parent,
  NetOptions & opt1, NetDefaultSettings & opt2,
  SkillOptions & opt3)
{
  MultiOptionsDlg dlg(parent);
  return dlg.askMultiplayerOptions(opt1, opt2, opt3);
}

