/***************************************************************************
                          nettransport.cpp  -  description
                             -------------------
    begin                : Mon Feb 25 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qdatastream.h>

#include "nettransport.h"

void LocalTransport::setPeer(LocalTransport * Peer)
{
  FPeer = Peer;
  FIn = new QDataStream(FPeer -> FArray, IO_ReadOnly);
  FOut = new QDataStream(FArray, IO_WriteOnly);
}

LocalTransport::~LocalTransport()
{
  if (FPeer)
  {
    FPeer -> FPeer = NULL;
    delete FPeer -> FIn;
    FPeer -> FIn = NULL;
    delete FPeer -> FOut;
    FPeer -> FOut = NULL;
    FPeer = NULL;
    delete FIn;
    FIn = NULL;
    delete FOut;
    FOut = NULL;
  }
}

void LocalTransport::close()
{
  if (FPeer -> FReceiver)
  {
    FPeer -> FReceiver -> onClose();
  }
}

void LocalTransport::deleteSelf()
{
  FPeer = NULL;
  if (FFlushing)
    FDeleteMeAfterFlush = true;
  else
    delete this;
}

void LocalTransport::flush()
{
  FFlushing = true;
  if (FPeer -> FReceiver)
  {
    while (! FPeer -> FIn -> atEnd())
    {
      //qDebug("Before peer receiveOne()");
      if (! FPeer -> FReceiver -> receiveOne())
        break;
      //qDebug("After peer receiveOne()");
    }
  }

  delete FOut;
  if (FPeer)
    delete FPeer -> FIn;
  //qDebug("After streams delete");

  FArray.resize(0);
  FOut = new QDataStream(FArray, IO_WriteOnly);
  if (FPeer)
    FPeer -> FIn = new QDataStream(FArray, IO_ReadOnly);
  //qDebug("After streams create");
  FFlushing = false;
  if (FDeleteMeAfterFlush)
    delete this;
}

class ConnectionDlg: public QDialog
{
public:
  QLineEdit * FAction;
  QLineEdit * FStatus;
  ConnectionDlg(QWidget * parent=0, const char * name=0);
  void accept() { QDialog::accept(); }
};

ConnectionDlg::ConnectionDlg(QWidget * parent, const char * name):
  QDialog(parent, name, true)
{
  QVBox * box = new QVBox(this);
  FAction = new QLineEdit(box);
  FAction -> setReadOnly(true);
  FStatus = new QLineEdit(box);
  FStatus -> setReadOnly(true);
  QPushButton * btn = new QPushButton(box);
  btn -> setText(tr("Cancel"));
  connect(btn, SIGNAL(clicked()), this, SLOT(reject()));
  box -> resize(300, 100);
  resize(300, 100);
}

SocketTransport::SocketTransport(QWidget * DialogWidget):
  FConnectionDlg(NULL)
{
  connect(this, SIGNAL(readyRead()), this, SLOT(onReceive()));
  connect(this, SIGNAL(connectionClosed()), this, SLOT(onClose()));
  FIn = new QDataStream(this);
  FOut = new QDataStream(this);

  connect(this, SIGNAL(hostFound()), this, SLOT(onHostFound()));
  connect(this, SIGNAL(connected()), this, SLOT(onConnected()));
  connect(this, SIGNAL(error(int)), this, SLOT(onError(int)));
  FDialogWidget = DialogWidget;
}

SocketTransport::~SocketTransport()
{
  if (FConnectionDlg)
    delete FConnectionDlg;
  delete FIn;
  delete FOut;
}

void SocketTransport::flush()
{
  QSocket::flush();
}

void SocketTransport::close()
{
  QSocket::close();
}

void SocketTransport::onReceive()
{
  if (FReceiver)
  {
    while (bytesAvailable())
      FReceiver -> receiveOne();
  }
}

void SocketTransport::onClose()
{
  if (FReceiver)
  {
    FReceiver -> onClose();
  }
}

bool SocketTransport::connectTo(const QString & host, Q_UINT16 port)
{
  FConnectionDlg = new ConnectionDlg(FDialogWidget);
  FConnectionDlg -> FAction -> setText(QString(tr("Connecting to %1")).arg(host));
  connectToHost(host, port);

  bool res = FConnectionDlg -> exec() == QDialog::Accepted;
  delete FConnectionDlg;
  FConnectionDlg = NULL;
  return res;
}

void SocketTransport::onHostFound()
{
  FConnectionDlg -> FStatus -> setText(tr("Host found"));
}

void SocketTransport::onConnected()
{
  FConnectionDlg -> FStatus -> setText(tr("Connected"));
  FConnectionDlg -> accept();
  FConnectionDlg -> hide();
}

void SocketTransport::onError(int)
{
  if (FConnectionDlg -> isVisible())
  {
    FConnectionDlg -> FStatus -> setText(tr("ERROR: can not connect to host!!!"));
  }
}
