/***************************************************************************
                          quadros.cpp  -  description
                             -------------------
    begin                : Feb 24 00:04:59 MSK 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <time.h>
#include <qobjectlist.h>
#include <qstring.h>
#include "quadrosprotocol.h"
#include "quadros.h"
#include "configfile.h"
#include "designoptions.h"
#include "figurecollections.h"
#include "filenew.xpm"
#include "figureoptions.h"

int TET_Version[4] = {0, 1, 0, 0};

QuadrOsGameMode::QuadrOsGameMode(const QString & text, const QString & menuText, QObject * parent, TGameMode mode):
  QAction(parent)
{
  setText(text);
  setMenuText(menuText);
  setToggleAction(true);
  GameMode = mode;
}

QuadrOsApp::QuadrOsApp(): listener(NULL)
{
  setCaption(tr("QuadrOs " VERSION));
  setFocusPolicy(QWidget::StrongFocus);

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();

  initDoc();
  initView();

  FGameMode = SinglePlayer;
  FOptions.StartLevel = 1;
  FOptions.IncreaseLevel = true;
  FOptions.TrashLines = true;
  FOptions.FieldHeight = 25;
  FOptions.ComputerSpeed = 4;

  FOptions.RandomSeed = time(NULL)%0xFFFFl;
  FOptions.FigureCollectionNumber = 0;

  doc1->newGame(FOptions);
  doc1->endGame(false);
  doc2->newGame(FOptions);
  doc2->endGame(false);

  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyLeft] = Key_Left;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyRight] = Key_Right;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyDrop] = Key_Space;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyRotate] = Key_Up;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyRotateBackward] = 0;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyAccelerate] = Key_Down;
  FMulti2DesignOptions.KeyCodes[DesignOptions::KeyJumpBound] = Key_Control;
  FMulti2DesignOptions.BeforeKeyRepeatInterval = 240;
  FMulti2DesignOptions.KeyRepeatRate = 20;
  FMulti2DesignOptions.SmartRotate = true;
  FMulti2DesignOptions.Color = true;
  FMulti2DesignOptions.BackgroundFileName = "";
  FMulti2DesignOptions.ElementsFileName = "";
  FMulti2DesignOptions.GridInterval = 0;
  FMulti2DesignOptions.ElementSize = 18;
  FMulti2DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
  FMulti2DesignOptions.SmoothMovement = true;

  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyLeft] = Key_A;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyRight] = Key_D;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyDrop] = Key_Z;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyRotate] = Key_W;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyRotateBackward] = 0;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyAccelerate] = Key_S;
  FMulti1DesignOptions.KeyCodes[DesignOptions::KeyJumpBound] = Key_Shift;
  FMulti1DesignOptions.BeforeKeyRepeatInterval = 100;
  FMulti1DesignOptions.KeyRepeatRate = 100;
  FMulti1DesignOptions.SmartRotate = true;
  FMulti1DesignOptions.Color = true;
  FMulti1DesignOptions.BackgroundFileName = "";
  FMulti1DesignOptions.ElementsFileName = "";
  FMulti1DesignOptions.GridInterval = 0;
  FMulti1DesignOptions.ElementSize = 18;
  FMulti1DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
  FMulti1DesignOptions.SmoothMovement = true;

  FSingleDesignOptions = FMulti2DesignOptions;
  FSingleDesignOptions.AnimationSpeed = 3;

  sock1 = new QuadrOsClientSocket(stats, doc1);

  sock1 -> FNetOptions.SecBeforeStart = 1;
  sock1 -> FNetOptions.MaxScores = 10;
  sock1 -> FNetOptions.TeamToThrow = wtAll;
  sock1 -> FNetOptions.PlayerToThrow = wtAll;
  sock1 -> FNetOptions.PlayerExitScores = 0;
  sock1 -> FNetOptions.TeamExitScores = 1;
  sock1 -> FNetOptions.TraceMessages = true;

  FNetSettings.PlayerName1 = tr("Player One");
  FNetSettings.TeamName1 = tr("Team One");
  FNetSettings.PlayerName2 = tr("Player Two");
  FNetSettings.TeamName2 = tr("Team Two");
  FNetSettings.SecondPlayer = true;
  FNetSettings.Host = "localhost";
  FNetSettings.Port = "8000";
  FNetSettings.GameType = NetDefaultSettings::gtLocalPlay;

  sock1 -> FOptions = FOptions;

  sock2 = new QuadrOsClientSocket(NULL, doc2);
  connect(stats -> FSendBtn, SIGNAL(clicked()), this, SLOT(sendMessage()));
  connect(stats -> FStartBtn, SIGNAL(clicked()), this, SLOT(startMultiplayerGame()));
  connect(stats -> FExitBtn, SIGNAL(clicked()), this, SLOT(closeMultiplayerGame()));
  connect(stats, SIGNAL(onEndGame()), this, SLOT(closeMultiplayerGame()));
  connect(doc1, SIGNAL(gameFinished()), sock1, SLOT(gameFinished()));
  connect(doc1, SIGNAL(figureDropped(long, int, int, int, int)), sock1, SLOT(figureDropped(long, int, int, int, int)));
  connect(doc2, SIGNAL(gameFinished()), sock2, SLOT(gameFinished()));
  connect(doc2, SIGNAL(figureDropped(long, int, int, int, int)), sock2, SLOT(figureDropped(long, int, int, int, int)));

  connect(doc1, SIGNAL(gameFinished()), this, SLOT(gameFinished()));
  connect(doc1, SIGNAL(gameEnded()), this, SLOT(gameEnded()));

  readConfigFile();
  view1 -> setOptions(FSingleDesignOptions);
  view2 -> setOptions(FMulti2DesignOptions);
  const QObjectList * l = gameMode->children();
  QObjectListIt it(*l);
  for (; it.current(); ++it)
  {
    QuadrOsGameMode * mode = (QuadrOsGameMode *) *it;
    if (mode->GameMode == FGameMode)
    {
      mode->setOn(true);
      break;
    }
  }
  //delete l;

  startTimer(TET_TimerInterval);
  setKeyCompression(false);
}

QuadrOsApp::~QuadrOsApp()
{
  delete sock1;
  delete sock2;
}

/** initializes all QActions of the application */
void QuadrOsApp::initActions(){

  QPixmap newIcon;
  newIcon = QPixmap(filenew);

  gameNew = new QAction(tr("New Game"), newIcon, tr("&New"), QAccel::stringToKey(tr("F2")), this);
  gameNew->setStatusTip(tr("Starts a game"));
  gameNew->setWhatsThis(tr("New Game\n\nStarts a game"));
  connect(gameNew, SIGNAL(activated()), this, SLOT(slotGameNew()));

  gameNewMultiplayer = new QAction(tr("Multiplayer Game"), tr("&Multiplayer"), QAccel::stringToKey(tr("Ctrl-M")), this);
  gameNewMultiplayer->setStatusTip(tr("Starts a multiplayer game"));
  gameNewMultiplayer->setWhatsThis(tr("Multiplayer Game\n\nStarts a multiplayer game"));
  connect(gameNewMultiplayer, SIGNAL(activated()), this, SLOT(slotGameNewMultiplayer()));

  gameDemo = new QAction(tr("Demo"), newIcon, tr("&Demo"), QAccel::stringToKey(tr("Ctrl-D")), this);
  gameDemo->setStatusTip(tr("Makes computer play for you to see"));
  gameDemo->setWhatsThis(tr("Demo\n\nMakes computer play for you to see"));
  connect(gameDemo, SIGNAL(activated()), this, SLOT(slotGameDemo()));

  gamePauseOn = new QAction(tr("Pause"), tr("&Pause"), QAccel::stringToKey(tr("Esc")), this);
  gamePauseOn->setStatusTip(tr("Pause a game"));
  gamePauseOn->setWhatsThis(tr("Pause\n\nPause a game"));
  connect(gamePauseOn, SIGNAL(activated()), this, SLOT(slotGamePauseOn()));

  gamePauseOff = new QAction(tr("Pause off"), tr("Pause &off/next round"), QAccel::stringToKey(tr("Enter")), this);
  gamePauseOff->setStatusTip(tr("Pause off or next round in multiplayer"));
  gamePauseOff->setWhatsThis(tr("Pause off\n\nPause off"));
  connect(gamePauseOff, SIGNAL(activated()), this, SLOT(slotGamePauseOff()));

  gameShowHighScore = new QAction(tr("Show highscores"), tr("Show &highscores"), QAccel::stringToKey(tr("Ctrl+H")), this);
  gameShowHighScore->setStatusTip(tr("Show highscores"));
  connect(gameShowHighScore, SIGNAL(activated()), this, SLOT(slotGameHighScores()));

  gameQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  gameQuit->setStatusTip(tr("Quits the application"));
  gameQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(gameQuit, SIGNAL(activated()), this, SLOT(slotGameQuit()));

  optionsDesign = new QAction(tr("Design"), tr("&Design and controls..."), 0, this);
  optionsDesign->setStatusTip(tr("Allows to change design options and key bindings"));
  optionsDesign->setWhatsThis(tr("Design options\n\nAllows to change design options and key bindings"));
  connect(optionsDesign, SIGNAL(activated()), this, SLOT(slotOptionsDesign()));

  optionsDesign1 = new QAction(tr("Design player 1"), tr("Controls - player &one..."), 0, this);
  optionsDesign1->setStatusTip(tr("Allows to change key bindings for player one in two players mode"));
  optionsDesign1->setWhatsThis(tr("Design options\n\nAllows to change key bindings for player one in two players mode"));
  connect(optionsDesign1, SIGNAL(activated()), this, SLOT(slotOptionsDesign1()));

  optionsDesign2 = new QAction(tr("Design player 2"), tr("Controls - player &two..."), 0, this);
  optionsDesign2->setStatusTip(tr("Allows to change key bindings for player two in two players mode"));
  optionsDesign2->setWhatsThis(tr("Design options\n\nAllows to change key bindings for player two in two players mode"));
  connect(optionsDesign2, SIGNAL(activated()), this, SLOT(slotOptionsDesign2()));

  optionsSkill = new QAction(tr("Skill"), tr("&Skill..."), 0, this);
  optionsSkill->setStatusTip(tr("Allows to change start level and game options"));
  optionsSkill->setWhatsThis(tr("Skill\n\nAllows to change start level and game options"));
  connect(optionsSkill, SIGNAL(activated()), this, SLOT(slotOptionsSkill()));

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

  gameMode = new QActionGroup(this);
  gameMode->setMenuText(tr("&Mode"));
  gameMode->setText(tr("Game mode"));
  //gameMode->setUsesDropDown(true);
  connect(gameMode, SIGNAL(selected(QAction *)), this, SLOT(modeSelected(QAction *)));

  modeSinglePlayer = new QuadrOsGameMode(tr("Single player"), tr("&Single player"), gameMode, SinglePlayer);
  modeArcade = new QuadrOsGameMode(tr("Pentix arcade"), tr("&Pentix arcade"), gameMode, Arcade);
  modeHumanVsHuman = new QuadrOsGameMode(tr("Human vs human"), tr("&Human vs human"), gameMode, HumanVsHuman);
  modeHumanVsComputer = new QuadrOsGameMode(tr("Human vs computer"), tr("Human vs &computer"), gameMode, HumanVsComputer);
}

void QuadrOsApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry gameMenu
  gameMenu=new QPopupMenu();
  gameNew->addTo(gameMenu);
  gameNewMultiplayer->addTo(gameMenu);
  gameDemo->addTo(gameMenu);
  gamePauseOn->addTo(gameMenu);
  gamePauseOff->addTo(gameMenu);
  gameShowHighScore->addTo(gameMenu);
  gameQuit->addTo(gameMenu);

  modeMenu=new QPopupMenu();
  gameMode->addTo(modeMenu);

  optionsMenu=new QPopupMenu();
  optionsDesign->addTo(optionsMenu);
  optionsDesign1->addTo(optionsMenu);
  optionsDesign2->addTo(optionsMenu);
  optionsSkill->addTo(optionsMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(tr("&Game"), gameMenu);
  menuBar()->insertItem(tr("&Mode"), modeMenu);
  menuBar()->insertItem(tr("&Options"), optionsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}

void QuadrOsApp::initDoc()
{
  doc1 = new QuadrOsDoc();
  doc2 = new QuadrOsDoc();
}

void QuadrOsApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  box = new QHBox(this);
  view1 = new QuadrOsView(box, doc1);
  stats = new MultiPlayerStats(box);
  stats -> hide();
  view2 = new QuadrOsView(box, doc2);
  view2 -> hide();
  box -> setStretchFactor(view1, 100);
  box -> setStretchFactor(view2, 100);
  box -> setStretchFactor(stats, 1);
  setCentralWidget(box);
}

bool QuadrOsApp::queryExit()
{
  if (sock1 -> isConnected())
    closeMultiplayerGame();
  return 1;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void QuadrOsApp::slotGameNew()
{
  if (sock1->isConnected())
  {
    closeMultiplayerGame();
  }

  switch(FGameMode)
  {
  case SinglePlayer:
  case Arcade:
    SkillOptions opt = FOptions;
    opt.RandomSeed = time(NULL)%0xFFFFl;
    if (FGameMode == Arcade)
    {
      opt.IncreaseLevel = true;
      opt.TrashLines = true;
      opt.FigureCollectionNumber = 1;
      //opt.StartLevel = 1;
    }

    doc1->newGame(opt);
    setFocus();
    doc1->run();
    break;
  case HumanVsHuman:
  case HumanVsComputer:
    if (doc1->isPlaying())
      doc1->endGame();
    listener = new QuadrOsListener();
    qDebug("listener created");

    LocalTransport * transClient1 = new LocalTransport();
    LocalTransport * transServer1 = new LocalTransport();
    transClient1 -> setPeer(transServer1);
    transServer1 -> setPeer(transClient1);

    listener -> m_Socks.push_back(new QuadrOsServerSocket(listener, transServer1));

    QString PlayerName;
    if (FGameMode == HumanVsComputer)
      PlayerName = tr("Computer");
    else
      PlayerName = tr("PlayerOne");

    sock1 -> ConnectToServer(true, transClient1,
      PlayerName,
      PlayerName);

    LocalTransport * transClient2 = new LocalTransport();
    LocalTransport * transServer2 = new LocalTransport();

    transClient2 -> setPeer(transServer2);
    transServer2 -> setPeer(transClient2);

    listener -> m_Socks.push_back(new QuadrOsServerSocket(listener, transServer2));

    if (FGameMode == HumanVsComputer)
      PlayerName = tr("Human (you)");
    else
      PlayerName = tr("PlayerTwo");

    sock2 -> ConnectToServer(false, transClient2,
      PlayerName,
      PlayerName);
    if (!sock2->isConnected())
    {
      QMessageBox::information(stats,
        stats -> tr("Multiplayer"), sock2->m_MessageOnClose);
      closeMultiplayerGame();
      return;
    }

    view2 -> show();

    stats -> show();
    //gameNew -> setEnabled(false);
    gameNewMultiplayer -> setEnabled(false);
    gameDemo -> setEnabled(false);
    move(0, 0);

    if (FGameMode == HumanVsComputer)
    {
      view1->turnAIOn(true, FOptions.ComputerSpeed);
    }

    FMulti1DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
    view1 -> setOptions(FMulti1DesignOptions);
    FMulti2DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
    view2 -> setOptions(FMulti2DesignOptions);

    sock1 -> StartGame();
    setFocus();
    /*FOptions.RandomSeed = time(NULL)%0xFFFFl;
    doc1->newGame(FOptions);
    setFocus();
    doc1->run();*/
    break;
  }
}

void QuadrOsApp::slotGamePauseOn()
{
  if (doc1->isPlaying())
  {
    if (doc1->isRunning())
    {
      if (sock1->isConnected())
        sock1->pauseOn();
      else
        doc1->pause();
    }
    else
    {
      showMinimized();
    }
  }
  else if (doc2->isPlaying())
  {
    if (doc2->isRunning())
    {
      sock2->pauseOn();
    }
    else
    {
      showMinimized();
    }
  }
  else
  {
    showMinimized();
  }
}

void QuadrOsApp::slotGamePauseOff()
{
  if (doc1->isPlaying())
  {
    if (! doc1->isRunning())
    {
      setFocus();
      if (sock1->isConnected())
        sock1->pauseOff();
      else
        doc1->run();
    }
  }
  else if (doc2->isPlaying())
  {
    if (! doc1->isRunning())
    {
      setFocus();
      sock2->pauseOff();
    }
  }
  else if (! sock1->isConnected())
  {
    slotGameNew();
  }
  else
  {
    startMultiplayerGame();
  }
}

void QuadrOsApp::slotGameQuit()
{
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  qApp->quit();
}

void QuadrOsApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("QuadrOs\nVersion " VERSION "\n(c) 1995-2004 by Dmitry Osinovsky\n"
                         "This program is free software; you can use, redistribute and/or modify\n"
                         "it under the terms of under GNU Lesser General Public License as \n"
                         "published by the Free Software Foundation; use either 2.1 of the\n"
                         "License, or (at your option) any later version.\n"
                         "You should have received a copy of the GNU Lesser General Public\n"
                         "License along with this program; if not, write to\n"
                         "the Free Software Foundation, Inc.,\n"
                         "59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n"
                         "\n"
                         "Download updates and sources at http://quadros.sourceforge.net\n"
                         ) );
}

bool checkAI(const QString & PlayerName, QuadrOsView *View)
{
  QString Computer = View->tr("Computer");
  if (PlayerName.left(Computer.length()) == Computer)
  {
    QString SpeedStr = PlayerName.mid(Computer.length(), 2);
    bool ok;
    int Speed = SpeedStr.toInt(& ok);
    if (ok)
    {
      View->turnAIOn(true, Speed);
      return true;
    }
  }

  return false;
}

void QuadrOsApp::slotGameNewMultiplayer()
{
  if (sock1->isConnected())
    return;

  if (doc1->isPlaying())
    doc1->endGame();

  if (! askMultiplayerOptions(this, sock1->FNetOptions, FNetSettings, sock1->FOptions))
    return;

  writeConfigFile();
  qDebug("show() executed");
  if (FNetSettings.GameType == NetDefaultSettings::gtLocalPlay)
  {
    listener = new QuadrOsListener();
    qDebug("listener created");

    LocalTransport * transClient1 = new LocalTransport();
    LocalTransport * transServer1 = new LocalTransport();
    transClient1 -> setPeer(transServer1);
    transServer1 -> setPeer(transClient1);

    listener -> m_Socks.push_back(new QuadrOsServerSocket(listener, transServer1));

    sock1 -> ConnectToServer(true, transClient1,
      FNetSettings.PlayerName1,
      FNetSettings.TeamName1);

    LocalTransport * transClient2 = new LocalTransport();
    LocalTransport * transServer2 = new LocalTransport();

    transClient2 -> setPeer(transServer2);
    transServer2 -> setPeer(transClient2);

    listener -> m_Socks.push_back(new QuadrOsServerSocket(listener, transServer2));

    sock2 -> ConnectToServer(false, transClient2,
      FNetSettings.PlayerName2,
      FNetSettings.TeamName2);
    if (!sock2->isConnected())
    {
      QMessageBox::information(stats,
        stats -> tr("Multiplayer"), sock2->m_MessageOnClose);
      closeMultiplayerGame();
      return;
    }

    view2 -> show();
    qDebug("Local play initialized");
  }
  else if (FNetSettings.GameType == NetDefaultSettings::gtCreateNetGame)
  {
    unsigned int Port = FNetSettings.Port.toInt();
    listener = new QuadrOsListener(Port);
    SocketTransport * transClient1 = new SocketTransport(this);

    if (! transClient1 -> connectTo("127.0.0.1", Port))
      return;

    sock1 -> ConnectToServer(true, transClient1,
      FNetSettings.PlayerName1,
      FNetSettings.TeamName1);

    if (FNetSettings.SecondPlayer)
    {
      SocketTransport * transClient2 = new SocketTransport(this);
      if (!transClient2 -> connectTo("127.0.0.1", Port))
        return;

      sock2 -> ConnectToServer(false, transClient2,
        FNetSettings.PlayerName2,
        FNetSettings.TeamName2);
      view2 -> show();
    }
    qDebug("Game created");
  }
  else if (FNetSettings.GameType == NetDefaultSettings::gtJoinNetGame)
  {
    unsigned int Port = FNetSettings.Port.toInt();
    QString Host = FNetSettings.Host;
    SocketTransport * transClient1 = new SocketTransport(this);
    if (!transClient1 -> connectTo(Host, Port))
      return;

    sock1 -> ConnectToServer(false, transClient1,
      FNetSettings.PlayerName1,
      FNetSettings.TeamName1);

    if (FNetSettings.SecondPlayer)
    {
      SocketTransport * transClient2 = new SocketTransport(this);
      if (!transClient2 -> connectTo(Host, Port))
        return;
      sock2 -> ConnectToServer(false, transClient2,
        FNetSettings.PlayerName2,
        FNetSettings.TeamName2);
      view2 -> show();
    }
    qDebug("Joined to game");
  }
  stats -> show();
  gameNew -> setEnabled(false);
  gameNewMultiplayer -> setEnabled(false);
  gameDemo -> setEnabled(false);
  move(0, 0);

  checkAI(FNetSettings.PlayerName1, view1);
  if (FNetSettings.SecondPlayer)
  {
    FMulti1DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
    view1 -> setOptions(FMulti1DesignOptions);
    FMulti2DesignOptions.AnimationSpeed = DesignOptions_MaxAnimationSpeed;
    view2 -> setOptions(FMulti2DesignOptions);
    checkAI(FNetSettings.PlayerName2, view2);
  }
}

void QuadrOsApp::startMultiplayerGame()
{
  sock1 -> StartGame();
  setFocus();
}

void QuadrOsApp::closeMultiplayerGame()
{
  qDebug("QuadrOsApp::closeMultiplayerGame begin");
  sock1->close();
  sock2->close();
  stats->hide();
  view2->hide();
  view1->turnAIOff();
  view2->turnAIOff();

  if (listener)
  {
    listener -> Close();
    delete listener;
    listener = NULL;
  }

  gameNew -> setEnabled(true);
  gameNewMultiplayer -> setEnabled(true);
  gameDemo -> setEnabled(true);

  // Return controls to single player

  view1 -> setOptions(FSingleDesignOptions);

  // Resizing back to normal width. Don't touch! It's magic!
  box -> layout() -> activate();
  box -> resize(box->sizeHint().width(), box->sizeHint().height());
  layout() -> activate();
  resize(0, 0);
  /*All this didn't work.
  box -> updateGeometry();
  setCentralWidget(NULL);
  setCentralWidget(box);
  updateGeometry();
  resize(0, 0);
  setUpLayout();*/
  qDebug("QuadrOsApp::closeMultiplayerGame end");
}

void QuadrOsApp::sendMessage()
{
  sock1 -> sendMessage(stats -> FMessage -> text());
  stats -> FMessage -> clear();
}

void QuadrOsApp::timerEvent (QTimerEvent *)
{
  if (listener)
    listener -> OnTimer();

  doc1->onTimer();
  doc2->onTimer();
  view1->onTimer();
  view2->onTimer();

}

void QuadrOsApp::keyReleaseEvent(QKeyEvent * e)
{
  view1 -> parentKeyReleaseEvent(e);
  view2 -> parentKeyReleaseEvent(e);
}

void QuadrOsApp::keyPressEvent(QKeyEvent * e)
{
  if (e->key() == Key_P)
  {
    if (doc1->isPlaying() || doc2->isPlaying())
    {
      if (doc1->isRunning() || doc2->isRunning())
        slotGamePauseOn();
      else
        slotGamePauseOff();
    }
    return;
  }
  else if (e->key() == Key_Return)
  {
    slotGamePauseOff();
  }

  view1 -> parentKeyPressEvent(e);
  view2 -> parentKeyPressEvent(e);
}

void QuadrOsApp::slotOptionsDesign()
{
  designDialog(FSingleDesignOptions, tr("Single player options"));
  if (! sock2 -> isConnected() /*|| view2 -> isAIOn()*/)
    view1 -> setOptions(FSingleDesignOptions);
}

void QuadrOsApp::slotOptionsDesign1()
{
  designDialog(FMulti1DesignOptions, tr("Options for two players mode, player one"));
  if (sock2 -> isConnected() /*&& ! view2 -> isAIOn()*/)
    view1 -> setOptions(FMulti1DesignOptions);
}

void QuadrOsApp::slotOptionsDesign2()
{
  designDialog(FMulti2DesignOptions, tr("Options for two players mode, player two"));
  view2 -> setOptions(FMulti2DesignOptions);
}

void QuadrOsApp::designDialog(DesignOptions & opt, const QString & caption)
{
  if (sock1->isConnected())
    editDesignOptions(this, caption, opt, doc1->options().FigureCollectionNumber);
  else
    editDesignOptions(this, caption, opt, FOptions.FigureCollectionNumber);
  writeConfigFile();
}

void QuadrOsApp::slotOptionsSkill()
{
  editSkillOptions(this, FOptions);
  writeConfigFile();
}

void QuadrOsApp::bindConfigFile(ConfigFile & cf)
{
  bindDesignOptions(cf, "SingleDesignOptions", FSingleDesignOptions);
  bindDesignOptions(cf, "Multi1DesignOptions", FMulti1DesignOptions);
  bindDesignOptions(cf, "Multi2DesignOptions", FMulti2DesignOptions);
  bindSkillOptions(cf, "SkillOptions", FOptions);
  bindNetOptions(cf, "NetOptions", sock1 -> FNetOptions, FNetSettings);
  bindSkillOptions(cf, "NetSkillOptions", sock1 -> FOptions);
  cf.bindIntValue("GlobalOptions", "GameMode", &FGameMode);
}

void QuadrOsApp::writeConfigFile()
{
  ConfigFile cf;
  bindConfigFile(cf);
  cf.writeConfigFile();
}

void QuadrOsApp::readConfigFile()
{
  ConfigFile cf;
  bindConfigFile(cf);
  cf.readConfigFile();
}

void QuadrOsApp::focusOutEvent(QFocusEvent *)
{
  if (doc1->isPlaying())
  {
    if (doc1->isRunning())
    {
      if (sock1->isConnected())
        sock1->pauseOn();
      else
      {
        if (! view1->isAIOn())
          doc1->pause();
      }
    }
  }
  view1 -> parentFocusOut();
  view2 -> parentFocusOut();
}

void QuadrOsApp::gameFinished()
{
  if (! sock1->isConnected() && ! view1->isAIOn() &&
    doc1->options().IncreaseLevel)
  {
    gameEndHighScore(this,
      doc1->score(), doc1->lines(), doc1->options());
  }
}

void QuadrOsApp::slotGameDemo()
{
  if (sock1->isConnected())
    return;
  slotGamePauseOff();
  view1->turnAIOn(false, 30);
}

void QuadrOsApp::gameEnded()
{
  if (! sock1->isConnected())
  {
    view1->turnAIOff();
  }
}

void QuadrOsApp::modeSelected(QAction * action)
{
  if (FGameMode != ((QuadrOsGameMode *) action)->GameMode)
  {
    FGameMode = ((QuadrOsGameMode *) action)->GameMode;
    writeConfigFile();
    slotGameNew();
  }
}

void QuadrOsApp::slotGameHighScores()
{
  showHighScore(this, FGameMode==Arcade?1:FOptions.FigureCollectionNumber);
}
