/***************************************************************************
                          quadros.h  -  description
                             -------------------
    begin                : Feb 24 00:04:59 MSK 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef QUADROS_H
#define QUADROS_H

// include files for QT
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qhbox.h>
#include <qabstractlayout.h>

// application specific includes
#include "quadrosview.h"
#include "quadrosdoc.h"
#include "multiplayerstats.h"
#include "quadrosclientsocket.h"
#include "quadroslistener.h"
#include "multioptions.h"

enum TGameMode {SinglePlayer, Arcade, HumanVsHuman, HumanVsComputer, Custom};

class QuadrOsGameMode: public QAction
{
  Q_OBJECT

public:
  QuadrOsGameMode(const QString & text, const QString & menuText, QObject * parent, TGameMode mode);
  TGameMode GameMode;
};

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class QuadrOsView is
  * created which creates your view.
  */
class QuadrOsApp : public QMainWindow
{
  Q_OBJECT

  public:
    /** construtor */
    QuadrOsApp();
    /** destructor */
    ~QuadrOsApp();
    /** initializes all QActions of the application */
    void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /** setup the document*/
    void initDoc();
    /** setup the mainview*/
    void initView();

    /** overloaded for Message box on last window exit */
    bool queryExit();

  public slots:

    void slotGameNew();
    void slotGameNewMultiplayer();
    void slotGamePauseOn();
    void slotGamePauseOff();
    void slotGameHighScores();

    void slotGameDemo();
    /** exits the application*/
    void slotGameQuit();

    /** shows an about dlg*/
    void slotHelpAbout();

    void slotOptionsDesign();
    void slotOptionsDesign1();
    void slotOptionsDesign2();
    void slotOptionsSkill();

    void startMultiplayerGame();
    void closeMultiplayerGame();
    void sendMessage();
    void gameFinished();
    void gameEnded();
    void modeSelected(QAction * action);

  protected:
    void designDialog(DesignOptions & opt, const QString & caption);
    void bindConfigFile(ConfigFile & cf);
    void writeConfigFile();
    void readConfigFile();

    void timerEvent (QTimerEvent *);
    void keyReleaseEvent(QKeyEvent * e);
    void keyPressEvent(QKeyEvent * e);
    void focusOutEvent(QFocusEvent *);

  private:
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    QuadrOsView *view1;
    QuadrOsView *view2;
    MultiPlayerStats *stats;
    QHBox *box;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    QuadrOsDoc *doc1;
    QuadrOsDoc *doc2;
    QuadrOsClientSocket *sock1;
    QuadrOsClientSocket *sock2;
    QuadrOsListener *listener;

    SkillOptions FOptions;
    DesignOptions FSingleDesignOptions; // single player or multiplayer without second local player
    DesignOptions FMulti1DesignOptions; // multiplayer player, player 1
    DesignOptions FMulti2DesignOptions; // multiplayer player, player 2
    NetDefaultSettings FNetSettings;

    /** game_menu contains all items of the menubar entry "Game" */
    QPopupMenu *gameMenu;
    QPopupMenu *modeMenu;
    QPopupMenu *optionsMenu;
    /** view_menu contains all items of the menubar entry "Help" */
    QPopupMenu *helpMenu;
    /** actions for the application initialized in initActions() and used to en/disable them
      * according to your needs during the program */
    QAction *gameNew;
    QAction *gameNewMultiplayer;
    QAction *gamePauseOn;
    QAction *gamePauseOff;
    QAction *gameShowHighScore;
    QAction *gameDemo;
    QAction *gameQuit;
    QAction *optionsDesign;
    QAction *optionsDesign1;
    QAction *optionsDesign2;
    QAction *optionsSkill;

    QActionGroup *gameMode;
    QuadrOsGameMode *modeSinglePlayer;
    QuadrOsGameMode *modeArcade;
    QuadrOsGameMode *modeHumanVsHuman;
    QuadrOsGameMode *modeHumanVsComputer;
    //QAction *modeHumanVsComputer;

    QAction *helpAboutApp;

    int FGameMode;
};

// from highscore.cpp
void showHighScore(QWidget * parent, int FigureCollection);

void gameEndHighScore(QWidget * parent, long Score, int Lines,
  const SkillOptions & StartOptions);

#endif

