/***************************************************************************
                          quadrosai.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include "quadrosai.h"
#include "intmatrix.h"
#include "quadrosdoc.h"

QuadrOsAI::QuadrOsAI(QuadrOsDoc * Doc)
  : FDoc(Doc)
{
  // Y-coords (from top) of surface for each X-coord
  surface = new int[FDoc->getFieldWidth() + 2];
  bestrotate = -1;
  bestmove = -1;
  FIdle = 0;

  connect(FDoc, SIGNAL(gameEnded()), this, SLOT(gameEnded()));
}

QuadrOsAI::~QuadrOsAI()
{
  delete[] surface;
}

int SmartSqr(int a)
{
  a = abs(a);
  return (a > 3) ? 9 + a : a * a;
}

long QuadrOsAI::evaluateField(QuadrOsField & Field, int lines, TDemoMatrix & DEMO_Matrix, int DEMO_CaseNum)
{
  //qDebug("Entering evaluateField");
  int i, j, h, w;
  long s;
  for (i = 0; i < 30; i ++)
    DEMO_Matrix[i][DEMO_CaseNum] = 0;

  // Compute surface, lowest point (max coord),
  //  highest point (min coord) & holes
  int max, min, minw, maxw;
  processSurfaceAndHoles(DEMO_Matrix, DEMO_CaseNum, max, min, minw, maxw, Field);

  // FOUR means four lines filled without holes but just
  //   waiting long straight figure to be completed
  bool HasFour = false, // do we have FOUR anywhere
    ThisHasFour = true, // do we have FOUR in the current 4 lines
    ThisIsIncomplete = true, // do we have incomplete FOUR anywhere
    IsIncomplete = false; // do we have incomplete FOUR in the current 4 lines

  int StartFour = max - 1, Count = 0, ThisCount = 0;

  for (h = max - 1; h >= min; h --) {
    bool HasHole = false, Incomplete = false;
    for (w = 0; w < FDoc->getFieldWidth(); w ++) {
      if (w != maxw && ! Field.getAt(w, h)) {
        Incomplete = true;
        if (surface[1 + w] < h) {
          HasHole = true;
        }
      }

      if (Field.getAt(w, h))
        ThisCount ++;
    }

    if (Incomplete)
      ThisHasFour = false;

    if (HasHole)
      ThisIsIncomplete = false;

    // Last line of the FOUR
    if (StartFour - 3 == h)  {
      if (ThisHasFour)
        HasFour = true;

      if (ThisIsIncomplete)
      {
        if (ThisCount > Count)
          Count = ThisCount;

        IsIncomplete = true;
      }

      if (h != min)
      {
        StartFour -= 4;
        ThisHasFour = true;
        ThisIsIncomplete = false;
        ThisCount = 0;
      }
    }
    else if (h == min)
    {
      if (ThisIsIncomplete)
      {
        if (ThisCount > Count)
          Count = ThisCount;

        IsIncomplete = true;
      }
    }
  }

  //if (lines > 1 && lines < 4)
  //  DEMO_Matrix[CollectFourLines][DEMO_CaseNum] += (lines - 1) * 5;

  if (HasFour)
    DEMO_Matrix[CollectFourLines][DEMO_CaseNum] += FDoc->getFieldWidth() * 6;

  if (IsIncomplete)
    DEMO_Matrix[CollectFourLines][DEMO_CaseNum] += Count;

  if (! IsIncomplete && ! HasFour)
    maxw = -10;

  // This was supposed to penalize collecting FOUR if we go too high
  //  but I just don't remember what this strange formula means
  // ??? Stop collecting four if we get higher than 11/18 of the field height
  if (Count <
    9 * 4 * FDoc->getFieldWidth() *
    (double(FDoc->getFieldHeight())/2 - min) /
    FDoc->getFieldHeight())
  {
    DEMO_Matrix[CollectFourLines][DEMO_CaseNum] = 0;
    maxw = -10;
  }

  if (DEMO_Params[MakeLongThinHoles] == 0 && DEMO_Params[CollectFourLines] == 0)
  {
    maxw = -10;
  }

  //DEMO_Matrix[2][DEMO_CaseNum]+=max;

  //if (maxw == 0 || maxw == FDoc->getFieldWidth() - 1)
  //  DEMO_Matrix[7][DEMO_CaseNum] ++;

  while (true)
  {
    DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] = 0;
    DEMO_Matrix[NoLongThickHoles][DEMO_CaseNum] = 0;
    DEMO_Matrix[MakeLongThinHoles][DEMO_CaseNum] = 0;

    for (i = 0; i < FDoc->getFieldWidth(); i ++)
    {
      if (i == maxw)
      {
        int LeftDiff = surface[1 + i] - surface[i];
        int RightDiff = surface[1 + i] - surface[2 + i];

        DEMO_Matrix[MakeLongThinHoles][DEMO_CaseNum] += LeftDiff > 4 ? 4 : LeftDiff;
        DEMO_Matrix[MakeLongThinHoles][DEMO_CaseNum] += RightDiff > 4 ? 4 : RightDiff;
      }
      else if ((surface[1 + i] - surface[i] > 1) &&
        (surface[1 + i] - surface[2 + i] > 1))
      {
        DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] -= surface[1 + i] - surface[i] - 1;
        DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] -= surface[1 + i] - surface[2 + i] - 1;
      }

      if (i<FDoc->getFieldWidth()-1) {
        if ((surface[1+i]-surface[i]>2)&&
          (abs(surface[1+i]-surface[2+i])<=1)&&
          (surface[1+i]-surface[3+i]>2)) {
          DEMO_Matrix[NoLongThickHoles][DEMO_CaseNum]--;
        }
      }

      if (i == maxw - 1)
      {
        if (surface[i] < surface[1 + i])
        {
          if (i > 0)
            DEMO_Matrix[MakeSmoothSurface][DEMO_CaseNum] -= SmartSqr(surface[1+i]-surface[i]);
          if (surface[i] + 1 < surface[1 + i] &&
            max - surface[1 + i] < 4)
            DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] -= surface[1 + i] - surface[i] - 1;
        }
      }
      else if (i == maxw + 2)
      {
        if (surface[i] > surface[1 + i])
        {
          if (i > 0)
            DEMO_Matrix[MakeSmoothSurface][DEMO_CaseNum] -= SmartSqr(surface[1+i]-surface[i]);

          if (surface[i] > surface[1 + i] + 1 &&
            max - surface[i] < 4)
            DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] -= surface[i] - surface[1 + i] - 1;
        }
      }
      else if (i <= 0)
      {
        // what is this supposed for???
      }
      else if (i == maxw + 1 || i == maxw)
      {
        int MaxAllow = int(double(min) / FDoc->getFieldHeight() * 8);

        if (min < FDoc->getFieldHeight()/2)
          MaxAllow = 4;

        int Diff = abs(surface[1+i]-surface[i]) - MaxAllow;
        if (Diff > 0)
        {
          DEMO_Matrix[MakeSmoothSurface][DEMO_CaseNum] -= SmartSqr(Diff);
        }
      }
      else
      {
        DEMO_Matrix[MakeSmoothSurface][DEMO_CaseNum] -= SmartSqr(surface[1+i]-surface[i]);
      }
    }

    if (maxw == FDoc->getFieldWidth() - 2 || maxw == 1)
      DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] -= 3;

    if (maxw == -10)
      break;

    if (min < FDoc->getFieldHeight() / 2 && DEMO_Matrix[NoLongThinHoles][DEMO_CaseNum] < 0)
    {
      maxw = -10;
      DEMO_Matrix[CollectFourLines][DEMO_CaseNum] = 0;
      continue;
    }

    break;
  }

  if (lines >= 4)
  {
    DEMO_Matrix[CollectFourLines][DEMO_CaseNum] += 1000;
  }

  if (lines >= 8)
  {
    DEMO_Matrix[CollectFourLines][DEMO_CaseNum] += 1000;
  }

  h=0; // y-coord of mass center
  for (i = 0; i < Field.getFigureSize(); i ++)
    for (j = 0; j < Field.getFigureSize(); j ++)
      if (Field.getFigureAt(i, j))
        h += j;
  if (h % Field.getFigureSize() > 1) h = h/Field.getFigureSize() + 1;
  else h = h/Field.getFigureSize();
  DEMO_Matrix[PutFigureLower][DEMO_CaseNum] += Field.getFigureY()+h+lines;
  s=0;
  for (i=0;i<30;i++) s += (int) (DEMO_Matrix[i][DEMO_CaseNum]*DEMO_Params[i]);
  return s;
}

long QuadrOsAI::iteratePossibleMoves(QuadrOsField & SourceField, int lines,
                                     bool SetBestMoves, bool SubIterateNextFigure)
{
  //qDebug("Entering iteratePossibleMoves");
  long bests;
  int rotate, move, i, j, OldFigureX;
  long s;
  //long s1;
  // Values of criteria for different moves
  TDemoMatrix DEMO_Matrix;

  // Number of the current move
  int DEMO_CaseNum;

  // CaseNumbers contains numbering of each possible move
  //   and -1 for impossible moves.
  //   It has two dimensions - first is number of rotates,
  //   second is figure shift + field width.
  IntMatrix CaseNumbers;
  CaseNumbers.setSizes(4, 2 * FDoc->getFieldWidth());
  for (i = 0; i < 4; i ++)
  for (j = 0; j < 2 * FDoc->getFieldWidth(); j ++)
    CaseNumbers.setElementAt(i, j, -1);

  // Fields on which we are experimenting with possible moves.
  QuadrOsFieldTest AfterRotateField, Field;

  DEMO_CaseNum = 0;

  bests = -10000000l;
  if (SetBestMoves)
  {
    bestrotate = 0;
    bestmove = 0;
  }
  OldFigureX = SourceField.getFigureX();

  SourceField.copyTo(& AfterRotateField);

  for (rotate = 0; rotate < 4; rotate ++) // rotate cycle
  {
    //AfterRotateField.debugPrintFigure();
    for (move = - FDoc->getFieldWidth(); move <= FDoc->getFieldWidth();
      move ++) // moving cycle
    {
      if (AfterRotateField.canPlaceCurFigureAt(OldFigureX + move,
        AfterRotateField.getFigureY()))
      {
        //qDebug("Move is possible");
        AfterRotateField.copyTo(& Field);
        CaseNumbers.setElementAt(rotate, move + FDoc->getFieldWidth(), DEMO_CaseNum);

        Field.setFigureX(OldFigureX + move);
        //qDebug("Position set, dropping figure");
        //qDebug("Figure x %d figure y %d", Field.getFigureX(), Field.getFigureY());

        Field.dropFigure();

        int lines1 = Field.collapseFullLines();
        if (lines1 >= 4 && lines >= 4)
          lines1 += lines;
        else
          lines1 = lines1 > lines ? lines1 : lines;

        if (SubIterateNextFigure)
        {
          Field.nextFigure();
          //s1 = evaluateField(Field, lines1, DEMO_Matrix, DEMO_CaseNum);
          s = iteratePossibleMoves(Field, lines1, false, false);
          //s = 0.9*s + 0.1*s1;
        }
        else
          s = evaluateField(Field, lines1, DEMO_Matrix, DEMO_CaseNum);

        //qDebug("Result %ld", s);
        DEMO_CaseNum++;
        if (s > bests) {
          bests=s;
          if (SetBestMoves)
          {
            //qDebug("Setting best moves");
            //qDebug("Rotate %d", rotate);
            //qDebug("Move %d", move);
            bestrotate = rotate;
            bestmove = move;
          }
        }
      }
    }

    if (!  AfterRotateField.rotateFigure(false, false))
    {
      break;
    }
  }

  //qDebug("Best result %ld", bests);
  return bests;
}

void QuadrOsAI::getBestMove()
{
  iteratePossibleMoves(*FDoc, 0, true, true);
  //qDebug("Best rotate %d", bestrotate);
  //qDebug("Best move %d", bestmove);
  /*long bests;
  int rotate, move, i, j, OldFigureX;
  long s;
  // Values of criteria for different moves
  TDemoMatrix DEMO_Matrix;

  // Number of the current move
  int DEMO_CaseNum;

  // CaseNumbers contains numbering of each possible move
  //   and -1 for impossible moves.
  //   It has two dimensions - first is number of rotates,
  //   second is figure shift + field width.
  IntMatrix CaseNumbers;
  CaseNumbers.setSizes(4, 2 * FDoc->getFieldWidth());
  for (i = 0; i < 4; i ++)
  for (j = 0; j < 2 * FDoc->getFieldWidth(); j ++)
    CaseNumbers.setElementAt(i, j, -1);

  // Fields on which we are experimenting with possible moves.
  QuadrOsFieldTest AfterRotateField, Field;

  DEMO_CaseNum = 0;

  bests = -10000000l;
  bestrotate = 0;
  bestmove = 0;
  OldFigureX = FDoc->getFigureX();

  FDoc->copyTo(& AfterRotateField);

  for (rotate = 0; rotate < 4; rotate ++) // rotate cycle
  {
    for (move = - FDoc->getFieldWidth(); move <= FDoc->getFieldWidth();
      move ++) // moving cycle
    {
      if (AfterRotateField.canPlaceCurFigureAt(OldFigureX + move,
        AfterRotateField.getFigureY()))
      {
        AfterRotateField.copyTo(& Field);
        CaseNumbers.setElementAt(rotate, move + FDoc->getFieldWidth(), DEMO_CaseNum);

        Field.setFigureX(OldFigureX + move);

        Field.dropFigure();

        int lines = Field.collapseFullLines();

        s = evaluateField(Field, lines, DEMO_Matrix, DEMO_CaseNum);
        DEMO_CaseNum++;
        if (s > bests) {
          bests=s;
          bestrotate=rotate;
          bestmove=move;
        }
      }
    }

    if (!  AfterRotateField.rotateFigure(false, false))
    {
      break;
    }
  }*/
}

void QuadrOsAI::setAggressive(bool Value)
{
  if (Value)
  {
    /*DEMO_Params[NoBuriedHoles] = 70;
    DEMO_Params[ExcavateLastBuriedHole] = 3;
    // DEMO_Params[MaxCohesion] = 0;
    DEMO_Params[NoLongThinHoles] = 20;
    DEMO_Params[NoLongThickHoles] = 1;
    DEMO_Params[MakeSmoothSurface] = 3;
    DEMO_Params[PutFigureLower] = 2;
    DEMO_Params[MakeLongThinHoles] = 2;
    DEMO_Params[CollectFourLines] = 2;*/

    DEMO_Params[NoBuriedHoles] = 100;
    DEMO_Params[ExcavateLastBuriedHole] = 25;
    // DEMO_Params[MaxCohesion] = 0;
    DEMO_Params[NoLongThinHoles] = 2;
    DEMO_Params[NoLongThickHoles] = 1;
    DEMO_Params[MakeSmoothSurface] = 6;
    DEMO_Params[PutFigureLower] = 2;
    DEMO_Params[MakeLongThinHoles] = 1;
    DEMO_Params[CollectFourLines] = 5;
  }
  else
  {
    DEMO_Params[NoBuriedHoles] = 100;
    DEMO_Params[ExcavateLastBuriedHole] = 50;
    // DEMO_Params[MaxCohesion] = 0;
    DEMO_Params[NoLongThinHoles] = 2;
    DEMO_Params[NoLongThickHoles] = 1;
    DEMO_Params[MakeSmoothSurface] = 6;
    DEMO_Params[PutFigureLower] = 2;
    DEMO_Params[MakeLongThinHoles] = 0;
    DEMO_Params[CollectFourLines] = 0;
  }
}

/*// This procedure tries to find a highest hole and lowest (max) point of the surface
void QuadrOsAI::findHighestHole(int & minholew, int & minholeh, int & prevmax)
{
  int w, h;
  minholew = FDoc->getFieldWidth();
  minholeh = FDoc->getFieldHeight();
  prevmax = 0;
  for (w = 0; w < FDoc->getFieldWidth(); w ++)
  {
    for (h = 0; h < FDoc->getFieldHeight(); h ++)
    {
      if (FDoc->getAt(w, h)) break;
    }

    if (h > prevmax)
      prevmax = h;

    if (h == FDoc->getFieldHeight()) continue;

    for (; h < FDoc->getFieldHeight(); h ++)
    {
      if (! FDoc->getAt(w, h)) break;
    }

    if (h == FDoc->getFieldHeight()) break;

    if (h < minholeh)
    {
      minholeh = h;
      minholew = w;
    }
  }
}*/

// Compute surface, lowest point (max coord),
//  highest point (min coord) & holes
void QuadrOsAI::processSurfaceAndHoles(
  TDemoMatrix & DEMO_Matrix, int DEMO_CaseNum,
  int & max, int & min,
  int & minw, int & maxw, QuadrOsField & Field)
{
  //qDebug("Entering processSurfaceAndHoles");
  int w, h, hh;
   surface[0] = 0;
   surface[FDoc->getFieldWidth() + 1] = 0;
   max = 0;
   maxw = - 10;
  min = FDoc->getFieldHeight();

   for (w = 0; w < FDoc->getFieldWidth(); w ++) {
     // Surface
     for (h = 0; h < FDoc->getFieldHeight(); h ++) {
       if (Field.getAt(w, h)) break;
     }

     surface[1 + w] = h;

     // highest point
     if (h <= min) {
       min = h;
       minw = w;
     }

     // lowest point
     if (h > max) {
       max = h;
       maxw = w;
     }

     if (h == max && w == FDoc->getFieldWidth() - 1 && maxw != 0)
     {
       max = h;
       maxw = w;
     }
   }

  // holes
   for (w = 0; w < FDoc->getFieldWidth(); w ++) {
     h = surface[1 + w];
     if (h == FDoc->getFieldHeight()) continue;
     hh = h;
     while (1) {
       for (; h < FDoc->getFieldHeight(); h++) {
         if (!Field.getAt(w, h)) break;
       }

       if (h == FDoc->getFieldHeight())
         break; // no more holes - break

       DEMO_Matrix[NoBuriedHoles][DEMO_CaseNum]--;

      DEMO_Matrix[ExcavateLastBuriedHole][DEMO_CaseNum] -=
        (h - hh) / (h);
       h++;
     }
   }
}

void QuadrOsAI::gameEnded()
{
  bestrotate = -1;
  bestmove = -1;
  FIdle = 0;
}

void QuadrOsAI::onTimer()
{
  if (! FDoc->isRunning())
    return;

  FIdle ++;
  if (FIdle * TET_TimerInterval >= 500 / FSpeed)
  {
    FIdle = 0;

    if (bestrotate > -1)
    {
      if (bestrotate > 0)
      {
        FDoc->rotateFigure(false, false);
        bestrotate --;
        return;
      }

      if (bestmove > 0)
      {
        FDoc->moveFigureRight();
        bestmove --;
        return;
      }

      if (bestmove < 0)
      {
        FDoc->moveFigureLeft();
        bestmove ++;
        return;
      }

      while (FDoc->lowerFigure(true));
      bestrotate = -1;
    }
    else
    {
      getBestMove();
    }
  }
}

void QuadrOsAI::setSpeed(int Speed)
{
  FSpeed = Speed;
  FIdle = 0;
}

