/***************************************************************************
                          quadrosai.h  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef QUADROSAI_H
#define QUADROSAI_H

#include <qobject.h>

class QuadrOsField;
class QuadrOsDoc;

typedef long TDemoMatrix[30][100];
/**
  *@author Dmitry Osinovsky
  */

class QuadrOsAI : public QObject  {
  Q_OBJECT
public:
  QuadrOsAI(QuadrOsDoc * Doc);
  ~QuadrOsAI();

  void onTimer();
  void getBestMove();
  void setAggressive(bool Value);
  void setSpeed(int Speed);

protected slots:
  void gameEnded();

private:
  long evaluateField(QuadrOsField & Field, int line, TDemoMatrix & DEMO_Matrix, int DEMO_CaseNum);
  long iteratePossibleMoves(QuadrOsField & Field,
                            int lines, bool SetBestMoves, bool SubIterateNextFigure);

  /*// This procedure tries to find a highest hole and lowest (max) point of the surface
  void findHighestHole(int & minholew, int & minholeh, int & prevmax);*/

  // Compute surface, lowest point (max coord),
  //  highest point (min coord) & holes
  void processSurfaceAndHoles(TDemoMatrix & DEMO_Matrix, int DEMO_CaseNum, int & max, int & min, int & minw,
    int & maxw, QuadrOsField & Field);

  // Criteria of play field goodness
  enum {NoBuriedHoles = 0, ExcavateLastBuriedHole = 1,
    NoLongThinHoles = 2, NoLongThickHoles = 3, MakeSmoothSurface = 4,
    PutFigureLower = 5, MakeLongThinHoles = 6, CollectFourLines = 7};

  bool FAggressive;
  QuadrOsDoc * FDoc;

  // Coefficients on which multiplies values of above criteria
  double DEMO_Params[30];

  /*// Values of criteria for different moves
  TDemoMatrix DEMO_Matrix;

  // Number of the current move
  int DEMO_CaseNum;*/

  // Number of rotates and moves for the best move
  //  Are decreased as this rotates and moves are performed
  int bestrotate, bestmove;

  // Y-coords (from top) of surface for each X-coord
  int * surface;

  int FSpeed;
  int FIdle;
};

#endif
