/***************************************************************************
                          quadrosclientsocket.cpp  -  description
                             -------------------
    begin                : Mon Feb 25 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

#include "multiplayerstats.h"
#include "quadrosdoc.h"
#include "quadrosclientsocket.h"

bool QuadrOsClientSocket::receiveOne()
{
  int MessageType;

  qDebug("QuadrOsClientSocket::receiveOne()");
  (*FTransport->getIn()) >> MessageType;
  qDebug("MessageType %d received", MessageType);

  switch(MessageType)
  {
  case NET_CloseConnection:
    (*FTransport->getIn()) >> m_MessageOnClose;
    break;

  case NET_Connected:
    break;

  case NET_RefreshPlayersInfo:
    {
      int PlayerNum, TeamNum;
      long Score, TotalScore;
      QString PlayerName, Team, Height;

      if (FStats)
        FStats -> FTeams -> clear();

      QListViewItem * item = 0;
      (*FTransport->getIn()) >> TeamNum;
      for (int i = 0; i < TeamNum; i ++)
      {
        (*FTransport->getIn()) >> Team >> Height >> Score >> TotalScore;
        if (FStats)
          item = new QListViewItem(FStats -> FTeams,  Team,
            QString::null, QString::number(Score), QString::number(TotalScore));

        (*FTransport->getIn()) >> PlayerNum;
        for (int i = 0; i < PlayerNum; i ++)
        {
          (*FTransport->getIn()) >> PlayerName >> Height >> Score >> TotalScore;

          if (FStats)
            new QListViewItem(item, PlayerName,
              Height, QString::number(Score), QString::number(TotalScore));
        }

        if (item)
          item -> setOpen(true);
      }
    }
    break;

  case NET_Chat:
    {
      QString Message, ChatText;
      (*FTransport->getIn()) >> Message;
      if (Message.at(0) == '&')
      {
        Message.remove(0, 1);
        if (FStats)
        {
          FStats -> FMsgDlg -> setText(Message);
          FStats -> FMsgDlg -> show();
        }
      }

      int Line, Col;
      if (FStats)
      {
        FStats -> FLog -> getCursorPosition(& Line, & Col);
        bool atEnd = Line >= FStats -> FLog -> numLines() - 1;
        FStats -> FLog -> append(Message);
        if (atEnd)
          FStats -> FLog -> setCursorPosition(
            FStats -> FLog -> numLines(), 0);
      }
    }
    break;

  case NET_StartGame:
    {
      (*FTransport->getIn()) >> FOptions.StartLevel >>
        FOptions.RandomSeed;
      FDoc->newGame(FOptions, true);
      FDoc->run();
      if (FStats)
        FStats -> FMsgDlg -> hide();
    }
    break;

  case NET_EndGame:
    if (FDoc->isPlaying())
    {
      FDoc->endGame();
    }

    if (FStats)
      FStats -> FStartBtn -> setEnabled(m_bServer);
    break;

  case NET_PauseOn:
    if (FDoc->isRunning())
      FDoc->pause();
    break;

  case NET_PauseOff:
    if (FDoc->isPlaying() && !FDoc->isRunning())
      FDoc->run();
    if (FStats)
    {
      FStats -> FMsgDlg -> hide();
      FStats -> parentWidget() -> parentWidget() -> setFocus();
    }
    break;

  case NET_AddLines:
    {
      int LineNum;
      (*FTransport->getIn()) >> LineNum;

      for (int i = 0; i < LineNum; i ++)
      {
        FDoc->addTrashLineWithCheck();
      }
    }
    break;

  case NET_Options:
    {
      int Trace, IncreaseLevel;
      (*FTransport->getIn()) >>
        FNetOptions.MaxScores >>
        (int &) FNetOptions.PlayerToThrow >>
        FNetOptions.PlayerExitScores >>
        (int &) FNetOptions.TeamToThrow >>
        FNetOptions.TeamExitScores >>
        FOptions.StartLevel >>
        IncreaseLevel >>
        FNetOptions.SecBeforeStart >>
        FOptions.FieldHeight >>
        Trace >>
        FOptions.FigureCollectionNumber;

      FNetOptions.TraceMessages = Trace;
      FOptions.IncreaseLevel = IncreaseLevel;
      FOptions.TrashLines = false;
      qDebug("Options received");
    }
    break;
  }

  return true;
}

void QuadrOsClientSocket::ConnectToServer(bool bServer,
  NetTransport * Transport, QString PlayerName, QString Team)
{
  m_MessageOnClose = tr("Server closed connection");
  m_bServer = bServer;
  FTransport = Transport;
  FTransport -> setReceiver(this);
  qDebug("Client sockets fields initialized");

  (*FTransport->getOut()) << int(NET_Connected) <<
    PlayerName <<
    Team <<
    TET_Version[0] <<
    TET_Version[1] <<
    TET_Version[2] <<
    TET_Version[3]
    ;

  qDebug("NET_Connected sent");

  FTransport->flush();

  qDebug("NET_Connected flush");

  if (bServer)
    SendOptions();
  qDebug("Options sent");

  if (FStats)
  {
    FStats -> FStartBtn -> setEnabled(m_bServer);
  }
}

void QuadrOsClientSocket::onClose()
{
  qDebug("QuadrOsClientSocket::onClose begin");
  if (FTransport)
  {
    FTransport->deleteSelf();
    FTransport = NULL;
  }

  if (FStats)
  {
    QMessageBox::information(FStats,
      FStats -> tr("Multiplayer"), m_MessageOnClose);
    FStats -> endGame();
  }

  qDebug("QuadrOsClientSocket::onClose end");
}

void QuadrOsClientSocket::SendOptions()
{
  (*FTransport->getOut()) << int(NET_Options) <<
    FNetOptions.MaxScores <<
    int(FNetOptions.PlayerToThrow) <<
    FNetOptions.PlayerExitScores <<
    int(FNetOptions.TeamToThrow) <<
    FNetOptions.TeamExitScores <<
    FOptions.StartLevel <<
    (int)FOptions.IncreaseLevel <<
    FNetOptions.SecBeforeStart <<
    FOptions.FieldHeight <<
    (int)FNetOptions.TraceMessages <<
    FOptions.FigureCollectionNumber;
  FTransport->flush();
}

void QuadrOsClientSocket::StartGame()
{
  if (FStats)
  {
    if (FStats -> FTeams -> childCount() <= 1)
    {
      QMessageBox::information(FStats,
        FStats -> tr("Multiplayer"),
        FStats -> tr("You have no competing side"));
      return;
    }
  }

  (*FTransport->getOut()) << (int) (NET_StartGame) <<
    FOptions.StartLevel;
  FTransport->flush();
  if (FStats)
  {
    FStats -> FStartBtn -> setEnabled(false);
  }
}

void QuadrOsClientSocket::close()
{
  qDebug("QuadrOsClientSocket::close begin");
  if (FTransport)
  {
    FTransport -> close();
    delete FTransport;
    FTransport = NULL;
  }

  if (FDoc -> isPlaying())
    FDoc -> endGame();
  qDebug("QuadrOsClientSocket::close end");
}

void QuadrOsClientSocket::pauseOn()
{
  (*FTransport->getOut()) << (int) (NET_PauseOn);
  FTransport->flush();
}

void QuadrOsClientSocket::pauseOff()
{
  (*FTransport->getOut()) << (int) (NET_PauseOff);
  FTransport->flush();
}

void QuadrOsClientSocket::gameFinished()
{
  if (! isConnected())
    return;

  (*FTransport->getOut()) << (int) (NET_EndGame);
  FTransport->flush();
}

void QuadrOsClientSocket::figureDropped(long Score, int, int, int Height, int CollapsedLines)
{
  if (! isConnected())
    return;

  (*FTransport->getOut()) << (int) (NET_FigureDropped)
    << (int) Height
    << (long) Score
    << CollapsedLines;

  qDebug("FigureDropped sent");
  FTransport->flush();
}

bool QuadrOsClientSocket::isConnected()
{
  return FTransport != NULL;
}

QuadrOsClientSocket::~QuadrOsClientSocket()
{
  if (FTransport) delete FTransport;
}

QuadrOsClientSocket::QuadrOsClientSocket(MultiPlayerStats * Stats, QuadrOsDoc * Doc):
  FStats(Stats), FDoc(Doc), m_bServer(false), FTransport(NULL)
{
  FOptions.StartLevel = 1;
  FOptions.IncreaseLevel = false;
  FOptions.TrashLines = false;
  FOptions.FieldHeight = 25;
}

void QuadrOsClientSocket::sendMessage(QString str)
{
  (*FTransport->getOut()) << (int) (NET_Chat) << str;
  FTransport->flush();
}
