/***************************************************************************
                          quadrosdoc.cpp  -  description
                             -------------------
    begin                : Feb 24 00:04:59 MSK 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include "quadrosdoc.h"
#include "profiler.h"

/* Wow! Dozer! */
const int DOZ_Width = 40;
const int DOZ_Height = 25;
const int DOZ_Frame = 2;
extern int DOZ_Dozer[DOZ_Frame][DOZ_Height][DOZ_Width];

QuadrOsDoc::QuadrOsDoc(): FIsRunning(false), FIsPlaying(false),
  FIdleTime(0), FDozerCoord(-1), FDozerIdleTime(-1), FStartGameAfterDozer(false),
  FCurrentFrequency(0),
  FCurLevel(5),
  FNextLevel(false)
{
  qDebug("QuadrOsDoc created");
  FAllFigOptions.resize(FigureCollections::instance()->collectionNumber());
}

QuadrOsDoc::~QuadrOsDoc()
{
  if (FCurrentFrequency)
    delete[] FCurrentFrequency;
}

void QuadrOsDoc::newGame(const SkillOptions Options,
  bool Immediate)
{
  if (FIsPlaying)
    endGame();

  FOptions = Options;
  FFigures = FigureCollections::instance()->at(FOptions.FigureCollectionNumber);
  FFigOptions = FAllFigOptions[FOptions.FigureCollectionNumber];

  if (FDozerIdleTime != -1)
  {
    if (Immediate)
    {
      FDozerIdleTime = -1;
      FDozerCoord = -1;
      FPainter->refresh();
    }
    else
    {
      FStartGameAfterDozer = true;
      return;
    }
  }

  // FField.clear();
  FNextFigurePainter->setSizes(FFigures->figureSize(), FFigures->figureSize());
  FFigureRandom.SetSeed(FOptions.RandomSeed);
  FTrashLineRandom.SetSeed(FOptions.RandomSeed);
  FCurLevel = FOptions.StartLevel;
  FScores = 0;
  FLines = 0;
  FFigureTime = 0;
  FCurHeight = 0;
  FAfterTrashLineCounter = 0;

  if (FCurrentFrequency)
    delete[] FCurrentFrequency;
  FCurrentFrequency = new int[FFigures->figureNumber()];

  if (FFigures->figureSize() == 4)
  {
    for (int i = 0; i < FFigures->figureNumber(); i ++)
    {
      FCurrentFrequency[i] = OO_RAND_MAX/FFigures->figureNumber();
    }
  }
  else
  {
    for (int j = 1; j <= FFigures->figureSize(); j ++)
    {
      int FigureCount = FFigures->figureNumber(j);
      for (int i = 0; i < FFigures->figureNumber(); i ++)
      {
        FCurrentFrequency[i] = OO_RAND_MAX/FigureCount;
      }
    }
  }

  // + TET_HeightIndent cause we have indent at the top of the field
  setFieldSizes(TET_Width, FOptions.FieldHeight + TET_HeightIndent);

  int NextFigureNumber = -1;

  // skip S- and Z- sized figures at the beginning of a game
  do
  {
    NextFigureNumber = getRandomFigure();//FFigureRandom() % FFigures->figureNumber();
  }
  while (! FFigures->at(NextFigureNumber)->hasFlatSide());

  nextFigure(NextFigureNumber);
  /*const FigureCollection * ArcadeLevels = FigureCollections::instance()->at(FigureCollections::instance() ->collectionNumber() - 1);
  ArcadeLevels->at(0)->copyFigureTo(& FNextFigure);
  nextFigure(-1);*/

  FIsRunning = false;
  FIsPlaying = true;
  FIdleTime = 0;
  FPainter -> refresh();
  updateStats(FScores, FCurLevel, FLines, FCurHeight);
  qDebug("QuadrOsDoc::newGame() executed");
}

void QuadrOsDoc::endGame(bool Animate)
{
  // PrintTimes();
  if (! FIsPlaying)
    return;

  gameEnded();

  FIsPlaying = false;
  FIsRunning = false;

  FCurFigure.erase(NULL, FFigureX, FFigureY, FHelperLinePainter);
  FPainter->eraseFigure();
  FPainter->setFigure(NULL, 0, 0);

  if (Animate)
  {
     // Run dozer
    FDozerCoord = 0;
    FDozerIdleTime = 20;
  }

  FHelperLinePainter -> refresh();
  FNextFigurePainter -> clear();
  FNextFigurePainter -> refresh();
  qDebug("QuadrOsDoc::endGame() executed");
}

void QuadrOsDoc::run()
{
  if (! FIsPlaying)
    return;
  pauseOff();
  FPainter -> drawFigure();
  FPainter -> refresh();
  FHelperLinePainter -> refresh();
  FIsRunning = true;
  qDebug("QuadrOsDoc::run() executed");
}

void QuadrOsDoc::pause()
{
  FIsRunning = false;
  pauseOn();
}

void QuadrOsDoc::addScores(int ScoreNumber)
{
  FScores += ScoreNumber * FFigures->scoreNum();
}


int QuadrOsDoc::getRandomFigure()
{
  int r, i, s, NextFigure;
  int FigureSize;

  if (FFigures->figureSize() == 4)
    FigureSize = 4;
  else
  {
    int rand[5];
    rand[0] = (FCurLevel - 1) * 1;
    rand[1] = (FCurLevel - 1) * 1;
    rand[2] = (FCurLevel - 1) * 2;
    rand[3] = 110 + (FCurLevel - 1) * 6;
    rand[4] = (FCurLevel - 1) * 6;
    int sum = rand[0] + rand[1] + rand[2] + rand[3] + rand[4];
    r = FFigureRandom() % sum;
    FigureSize = 4;
    for (int i = 0; i <= 4; i ++)
    {
      if (r < rand[i])
      {
        FigureSize = i + 1;
        break;
      }
      r -= rand[i];
    }
  }

  do
  {
    s = 0;
    r = FFigureRandom();
    NextFigure = FFigures->figureNumber();
    for (i = 0; i < FFigures->figureNumber(); i ++)
    {
      if (FFigures->at(i)->elementNum() == FigureSize)
      {
        s += FCurrentFrequency[i];
        if (r < s)
        {
          NextFigure = i;
          break;
        }
      }
    }
  } while (NextFigure == FFigures->figureNumber());

  int FigureCount = FFigures->figureNumber(FigureSize);

  r = OO_RAND_MAX / FigureCount;
  r = FCurrentFrequency[NextFigure] < r ?
    FCurrentFrequency[NextFigure] : r;

  for (i = 0; i < FFigures->figureNumber(); i ++)
  {
    if (FFigures->at(i)->elementNum() == FigureSize)
      FCurrentFrequency[i] += r / FigureCount;
  }

  FCurrentFrequency[NextFigure] -= r;

  return NextFigure;
//  return int((FFigureRandom() * double(TET_FigureNumber)) / (OO_RAND_MAX + 1.0));
}

void QuadrOsDoc::nextFigure(int NextFigureNumber)
{
  if (NextFigureNumber == -1)
  {
    FNextFigure.copyFigureTo(& FCurFigure);
  }
  else
  {
    FFigures->at(NextFigureNumber)->copyFigureTo(& FCurFigure);
    if (FFigOptions.Figures)
    {
      if (FFigOptions.Options[NextFigureNumber].Color != -1)
        FCurFigure.changeColor(FFigOptions.Options[NextFigureNumber].Color);
      for (int i = 0; i < FFigOptions.Options[NextFigureNumber].RotatePosition; i ++)
        FCurFigure.rotate();
    }
  }

  int FigNum = getRandomFigure();
  qDebug("Next figure %d", FigNum);
  FFigures->at(FigNum)->copyFigureTo(& FNextFigure);

  if (FFigOptions.Figures)
  {
    if (FFigOptions.Options[FigNum].Color != -1)
      FNextFigure.changeColor(FFigOptions.Options[FigNum].Color);
    for (int i = 0; i < FFigOptions.Options[FigNum].RotatePosition; i ++)
      FNextFigure.rotate();
  }

  FFigureX = (FFieldWidth - FCurFigure.size()) / 2;
  FFigureY = 0;
  FFigureYPercent = 0;
  FFallHeight = 0;
  FIdleTime = 0;

  FNextFigurePainter -> clear();
  FNextFigure.draw(FNextFigurePainter, 0, 0);
  FNextFigurePainter -> refresh();
  FPainter -> setFigure(& FCurFigure, FFigureX * 100, FFigureY * 100);
  drawFigure();
}

void QuadrOsDoc::drawFigure()
{
  FCurFigure.draw(NULL, FFigureX, FFigureY, FHelperLinePainter);
}

void QuadrOsDoc::eraseFigure()
{
  FCurFigure.erase(NULL, FFigureX, FFigureY, FHelperLinePainter);
}

int QuadrOsDoc::ticksBeforeFigureLowering()
{
  return 2000/TET_TimerInterval/((FCurLevel < 3 ? 3 : (FCurLevel > 25 ? 25 : FCurLevel)) * 2 - 1);
}

void QuadrOsDoc::onTimer()
{
  if (FFigureTime < 2000)
    FFigureTime += TET_TimerInterval;

  if (FDozerIdleTime > -1)
  {
    FDozerIdleTime += TET_TimerInterval;
    if (FDozerIdleTime >= 20)
      moveDozer();
  }

  bool DoLower = false;
  if (FIsRunning)
  {
    FIdleTime ++;
    if (FSmoothMovement && canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
    {
      int Movement = 100*FIdleTime/ticksBeforeFigureLowering() - 100*(FIdleTime-1)/ticksBeforeFigureLowering();
      FFigureYPercent += Movement;
      FPainter->animateFigure(0, Movement, 1);
    }

    DoLower = (FIdleTime >= ticksBeforeFigureLowering());
    if (DoLower)
    {
      FIdleTime = 0;
      lowerFigure(! FSmoothMovement);
      if (FCurLevel < 14)
        FFallHeight = 0;
    }
  }
}

bool QuadrOsDoc::accelerateFigure(bool Animate)
{
  FIdleTime = 0;
  if (FCurLevel < 14)
    FFallHeight = 0;
  return moveFigureDown(Animate);
}

void QuadrOsDoc::landFigure()
{
  //qDebug("Fall height %d", FFallHeight);
  if (FFallHeight > 3)
  {
    FPainter -> animateFigure(0, -25, 50);
    FPainter -> animateFigure(0, 25, 50);
    /*FPainter -> animate(& FCurFigure, FFigureX, FFigureY,
      0, 0,
      0, -25, 40,
      0, 0, FCurFigure.size() - 1, FCurFigure.size() - 1);

    FPainter -> animate(& FCurFigure, FFigureX, FFigureY,
      0, -25,
      0, 25, 40,
      0, 0, FCurFigure.size() - 1, FCurFigure.size() - 1);*/
  }

  addScores(2000 / (FFigureTime + 2));
  FFigureTime = 0;

  QuadrOsField::landFigure();

  FNextLevel = false;
  checkForFullLines();
  /*if (FNextLevel && (FCurHeight < FFieldHeight / 3))
  {
    const FigureCollection * ArcadeLevels = FigureCollections::instance()->at(FigureCollections::instance() ->collectionNumber() - 1);
    ArcadeLevels->at((FCurLevel - 2) % ArcadeLevels->figureNumber())->copyFigureTo(& FCurFigure);
    addScores(50000 * FCurLevel);
    FFigureX = (FFieldWidth - FCurFigure.size()) / 2;
    FFigureY = 0;
    FFigureYPercent = 0;
    FFallHeight = 0;
    FIdleTime = 0;

    FPainter -> setFigure(& FCurFigure, FFigureX * 100, FFigureY * 100);
  }
  else*/
  nextFigure();

  if (! canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
  {
    FIsRunning = false;
    gameFinished();
    endGame();
  }
  else
  {
    FPainter -> drawFigure();
    FPainter -> refresh();
  }

  FHelperLinePainter -> refresh();
}

void QuadrOsDoc::checkForFullLines()
{
  int lines = collapseFullLines();
  FLines += lines;

  for (int lt = lines; lt > 0; lt --)
    addScores(lt * lt * 1000);

  while (FOptions.IncreaseLevel && FLines >= 15 * FCurLevel)
  {
    FCurLevel ++;

    FIdleTime = 0;
    FNextLevel = true;
  }

  if (FOptions.TrashLines && (++ FAfterTrashLineCounter) >= 17 + 100 / FCurLevel)
  {
    FAfterTrashLineCounter = 0;
    addScores(150 * FCurLevel);
    addTrashLine();
  }

  if (lines > 1)
  {
    qDebug("Dropped %d lines", lines);
  }

  figureDropped(FScores, FCurLevel, FLines, FCurHeight, lines);
  updateStats(FScores, FCurLevel, FLines, FCurHeight);
}

void QuadrOsDoc::addTrashLine()
{
  for (int hh = 0; hh < FFieldHeight; hh ++)
  for (int w = 0; w < FFieldWidth; w ++)
  {
    int Element = FField.getElementAt(1 + w, 2 + hh);
    FField.setElementAt(1 + w, 1 + hh, Element);
    FPainter -> setElementAt(w, hh, Element);
  }

  int ElemNum = 0;
  for(int w = 0; w < FFieldWidth; w ++)
  {
    int PutElem;
    if (FTrashLineRandom() % 4)
    {
      PutElem = 1;
      ElemNum ++;
    }
    else
    {
      PutElem = 0;
    }

     FField.setElementAt(1 + w, FFieldHeight, PutElem);
     FPainter -> setElementAt(w, FFieldHeight - 1, PutElem);
  }

  // If all line is full, then we need to make at least
  //   one hole in it
  if (ElemNum == FFieldWidth)
  {
    int ClearElementAt = (FTrashLineRandom() % FFieldWidth);
    FField.setElementAt(1 + ClearElementAt, FFieldHeight, 0);
     FPainter -> setElementAt(ClearElementAt, FFieldHeight - 1, 0);
  }

  FCurHeight ++;
}

void QuadrOsDoc::addTrashLineWithCheck()
{
  if (! isPlaying())
    return;

  if (FFigureY > 0)
  {
    //FPainter -> eraseFigure();
    FFigureY --;
    FFigureYPercent -= 100;
    FPainter -> animateFigure(0, -100, 1);
  }
  else
  {
    while (! canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
    {
      landFigure();

      if (! isPlaying())
      {
        return;
      }
    }
  }

  addTrashLine();
  //FPainter -> drawFigure();
  FPainter -> refresh();
}

void QuadrOsDoc::moveDozer()
{
  int i = FDozerCoord ++;
  if (FDozerCoord == DOZ_Width)
  {
    FDozerCoord = -1;
    FDozerIdleTime = -1;
    if (FStartGameAfterDozer)
    {
      qDebug("Starting game after dozer");
      FStartGameAfterDozer = false;
      newGame(FOptions);
      run();
    }

    return;
  }

   for (int j = 0; j < FFieldHeight; j ++)
   {
     for (int k = 0; k < FFieldWidth - 1; k ++)
     {
       FPainter -> setElementAt(k, j,
         FPainter -> getElementAt(1 + k, j));
     }

     if (i < FFieldWidth)
     {
       for (int k = 0; k < i + 1; k ++)
       {
         int Element = (j + DOZ_Height - FFieldHeight >= 0) ?
           DOZ_Dozer[i%DOZ_Frame][j + DOZ_Height - FFieldHeight][k] : 0;
         FPainter -> setElementAt(FFieldWidth - i + k - 1, j,
           Element);
       }
     } else {
       for (int k = i - FFieldWidth + 1; k < i + 1; k ++)
       {
         int Element = (j + DOZ_Height - FFieldHeight >= 0) ?
           DOZ_Dozer[i%DOZ_Frame][j + DOZ_Height - FFieldHeight][k] : 0;
         FPainter -> setElementAt(FFieldWidth - i + k - 1, j,
           Element);
       }
     }
   }

   FPainter -> refresh();
}

int DOZ_Dozer[DOZ_Frame][DOZ_Height][DOZ_Width]=
{{{
//0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,0,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,1,1,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,1,0,1,1,1,1,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,0,0,0,1,0,1,0,0,0,0,1,1,1,0,0,0,0,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,0,0,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,0,0,0,1,1,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,0,1,0,0,0,1,1,0,0,0,1,1,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}},
{{
//0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,0,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,0,1,1,0,0,0,1,1,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,0,0,0,1,0,1,1,1,1,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,0,0,0,1,0,1,0,0,0,0,1,1,1,0,0,0,0,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,1,1,0,0,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,1,0,0,0,0,1,0,0,0,1,0,1,0,0,1,0,1,0,0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
1,0,0,0,0,0,0,1,0,0,1,1,0,0,0,1,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},{
0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}}};


