/***************************************************************************
                          quadrosdoc.h  -  description
                             -------------------
    begin                : Feb 24 00:04:59 MSK 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/
#ifndef QUADROSDOC_H
#define QUADROSDOC_H

// include files for QT
#include <qobject.h>

// application specific includes
#include "ooprandom.h"
#include "skilloptions.h"
#include "quadrosfield.h"
#include "figureoptions.h"

const int TET_Width = 10;
const int TET_HeightIndent = 0;
const int TET_TimerInterval = 20;

/**
  * the Document Class
  */

class QuadrOsDoc : public QuadrOsField
{
  Q_OBJECT

public:
  QuadrOsDoc();
  ~QuadrOsDoc();

  void run();
  void pause();
  // Running - means Playing and not paused
  bool isRunning() { return FIsRunning; }

  void newGame(const SkillOptions Options, bool Immediate = false);
  void endGame(bool Animate = true);
  // Playing - means that game started and not ended yet,
  //   although it can be paused
  bool isPlaying() { return FIsPlaying; }

  void addTrashLineWithCheck();
  void onTimer();
  long score() { return FScores; }
  int lines() { return FLines; }
  SkillOptions options() { return FOptions; }
  bool accelerateFigure(bool Animate = true);
  void setFigureOptions(const vector<FigureOptions> & AllFigOptions)
    { FAllFigOptions = AllFigOptions; }

signals:
  void figureDropped(long Score, int Level, int Lines, int Height, int CollapsedLines);
  void updateStats(long Score, int Level, int Lines, int Height);

  // Called if player is defeated
  void gameFinished();

  // Called if game ended (either player is defeated or press "New Game" or ...)
  void gameEnded();

  void pauseOn();
  void pauseOff();

protected:
  /* Functions drawFigure and eraseFigure are needed to draw figure's "shadow" on helper line */
  void drawFigure();
  void eraseFigure();

  void landFigure();
  void checkForFullLines();
  void nextFigure(int NextFigureNumber = -1);
  int getRandomFigure();
  void addScores(int ScoreNumber); // can adjust scores according to different factors
  void addTrashLine();
  void moveDozer();
  int ticksBeforeFigureLowering(); // how many ticks of timer must pass for figure to lower on one element

private:
  // Running - means Playing and not paused
  bool FIsRunning;

  // Playing - means that game started and not ended yet,
  //   although it can be paused
  bool FIsPlaying;

  // Number of times the timer event was called after last lowering
  //   of a figure
  int FIdleTime;

  // Coordinate of the dozer, -1 if no dozer running
  int FDozerCoord;

  // Number of times the timer event was called after last dozer moving
  int FDozerIdleTime;

  bool FStartGameAfterDozer;

  // Random generators
  RandomGenerator FFigureRandom, FTrashLineRandom;

  int * FCurrentFrequency;

  const FigureCollection * FFigures;

  /*********************************************
   * Level & scores & skill
   ********************************************/
  int FCurLevel,
    FScores,
    FLines, // total number of lines collapsed since beginning of the game
    FFigureTime, // time which player spent at current figure
    FAfterTrashLineCounter; // how many figures have been dropped after last trash line

  bool FNextLevel;

  SkillOptions FOptions;
  FigureOptions FFigOptions;
  vector<FigureOptions> FAllFigOptions;
};

#endif
