/***************************************************************************
                          quadrosfield.cpp  -  description
                             -------------------
    begin                : Wed Mar 13 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "quadrosfield.h"
#include "profiler.h"

void QuadrOsFieldTest::nextFigure()
{
  FNextFigure.copyFigureTo(& FCurFigure);
  FFigureX = (FFieldWidth - FCurFigure.size()) / 2;
  FFigureY = 0;
  FFigureYPercent = 0;
  FFallHeight = 0;
}

QuadrOsField::QuadrOsField(QObject * parent):
  QObject(parent),
  FOwnPainters(false), FPainter(NULL),
  FHelperLinePainter(NULL), FNextFigurePainter(NULL),
  FFieldWidth(0), FFieldHeight(0),
  FSmoothMovement(true), FFallHeight(0)
{
  setPainters(NULL, NULL, NULL);
}

QuadrOsField::~QuadrOsField()
{
  if (FOwnPainters)
  {
    delete FPainter;
    delete FHelperLinePainter;
    delete FNextFigurePainter;
  }
}

void QuadrOsField::setPainters(IMatrix * Painter,
  IMatrix * HelperLinePainter, IMatrix * NextFigurePainter)
{
  if (FOwnPainters)
  {
    delete FPainter;
    delete FHelperLinePainter;
    delete FNextFigurePainter;
  }

  if (Painter == NULL)
  {
    FOwnPainters = true;
    FPainter = new IntMatrix();
    FHelperLinePainter = new IntMatrix();
    FNextFigurePainter = new IntMatrix();
  }
  else
  {
    FOwnPainters = false;
    FPainter = Painter;
    FHelperLinePainter = HelperLinePainter;
    FNextFigurePainter = NextFigurePainter;
  }

  FPainter -> setSizes(FFieldWidth, FFieldHeight);
  FHelperLinePainter -> setSizes(FFieldWidth, 1);
}

void QuadrOsField::setFieldSizes(int FieldWidth, int FieldHeight)
{
  FFieldWidth = FieldWidth;
  FFieldHeight = FieldHeight;
  FPainter -> setSizes(FFieldWidth, FFieldHeight);
  FHelperLinePainter -> setSizes(FFieldWidth, 1);

  // Field size is increased cause the borders
  FField.setSizes(FFieldWidth + 2, FFieldHeight + 2);

  // Set borders at bottom, left & right
  for (int x = 0; x < FFieldWidth; x ++)
  {
    FField.setElementAt(x + 1, FFieldHeight + 1, 1);
  }

  for (int y = 0; y < FFieldHeight; y ++)
  {
    FField.setElementAt(0, y + 1, 1);
    FField.setElementAt(FFieldWidth + 1, y + 1, 1);
  }
}

bool QuadrOsField::canPlaceFigureAt(int X, int Y, const Figure & AFigure)
{
  int w, h;

  for (w = 0; w < AFigure.size(); w ++)
  for (h = 0; h < AFigure.size(); h ++)
  {
    if ((X + w >= 0) && (X + w < FFieldWidth))
    {
      if (AFigure.at(w, h) && FField.getElementAt(1 + X + w, 1 + Y + h))
      {
        return false;
      }
    }
    else
    {
      if (AFigure.at(w, h))
      {
        return false;
      }
    }
  }

  return true;
}

void QuadrOsField::moveFigureRight()
{
  if (canPlaceFigureAt(FFigureX + 1, FFigureY, FCurFigure))
  {
    FFallHeight = 0;
    /*FPainter->animate(& FCurFigure, FFigureX, FFigureY, 0, 0,
      100, 0, 50, 0, 0, FCurFigure.size() - 1, FCurFigure.size() - 1);*/

    if (FSmoothMovement && !canPlaceFigureAt(FFigureX + 1, FFigureY + 1, FCurFigure))
    {
      FPainter -> animateFigure(0, FFigureY * 100 - FFigureYPercent, 10);
      FFigureYPercent = FFigureY * 100;
    }

    FPainter -> animateFigure(100, 0, 50);
    eraseFigure();
    FFigureX ++;
    drawFigure();
    //FPainter -> refresh();
    FHelperLinePainter -> refresh();
  }
}

void QuadrOsField::moveFigureLeft()
{
  if (canPlaceFigureAt(FFigureX - 1, FFigureY, FCurFigure))
  {
    FFallHeight = 0;
    /*FPainter->animate(& FCurFigure, FFigureX, FFigureY, 0, 0,
      -100, 0, 50, 0, 0, FCurFigure.size() - 1, FCurFigure.size() - 1);*/

    if (FSmoothMovement && !canPlaceFigureAt(FFigureX - 1, FFigureY + 1, FCurFigure))
    {
      FPainter -> animateFigure(0, FFigureY * 100 - FFigureYPercent, 10);
      FFigureYPercent = FFigureY * 100;
    }

    FPainter -> animateFigure(-100, 0, 50);
    eraseFigure();
    FFigureX --;
    drawFigure();
    //FPainter -> refresh();
    FHelperLinePainter -> refresh();
  }
}

bool QuadrOsField::rotateFigure(bool Backward, bool SmartRotate)
{
  Figure buf;
  // rotates figure,tryes to place it
  // if it is not possible rotates it backward

  FCurFigure.copyFigureTo(& buf);

  FPainter -> eraseFigure();
  eraseFigure();
  FCurFigure.rotate();
  if (Backward)
  {
    FCurFigure.rotate();
    FCurFigure.rotate();
  }

  bool CanMovePlus = true, CanMoveMinus = true, Placed = false;
  int i;

  // Checks if rotated figure can be placed,
  //   if not && SmartRotate then tries to move it left or
  //   right before rotate
  for (i = 0; i <= (SmartRotate ? 3 : 0); i ++)
  {
    // Can we move source (not rotated) figure right?
    CanMovePlus = CanMovePlus &&
      canPlaceFigureAt(FFigureX + i, FFigureY, buf);

    // If we can and we can place here rotated figure also
    if (CanMovePlus && canPlaceFigureAt(FFigureX + i, FFigureY,
      FCurFigure))
    {
      FFigureX += i;
      if (! canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
      {
        FFigureYPercent = FFigureY * 100;
      }

      FPainter -> setFigure(& FCurFigure, FFigureX * 100, FFigureYPercent);
      FPainter -> drawFigure();
      FPainter -> refresh();
      drawFigure();
      FHelperLinePainter -> refresh();
      Placed = true;
      break;
    }

    // the same to the left
    CanMoveMinus = CanMoveMinus &&
      canPlaceFigureAt(FFigureX - i, FFigureY, buf);

    if (CanMoveMinus && canPlaceFigureAt(FFigureX - i, FFigureY,
      FCurFigure))
    {
      FFigureX -= i;
      if (! canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
      {
        FFigureYPercent = FFigureY * 100;
      }

      FPainter -> setFigure(& FCurFigure, FFigureX * 100, FFigureYPercent);
      FPainter -> drawFigure();
      FPainter -> refresh();
      drawFigure();
      FHelperLinePainter -> refresh();
      Placed = true;
      i = -i;
      break;
    }
  }

  // If we cannot place rotated figure, then return everything
  //   to the state it was before rotate
  if (! Placed)
  {
    buf.copyFigureTo(& FCurFigure);
    FPainter -> drawFigure();
    drawFigure();
  }
  else
    FFallHeight = 0;

  return Placed;
}

bool QuadrOsField::moveFigureDown(bool Animate)
{
  if (canPlaceFigureAt(FFigureX, FFigureY + 1, FCurFigure))
  {
    FPainter->animateFigure(0, FFigureY * 100 + 100 - FFigureYPercent, Animate ? 30 : 1);

      /*FPainter->animate(& FCurFigure, FFigureX, FFigureY, 0, 0,
        0, 100, 30, 0, 0, FCurFigure.size() - 1, FCurFigure.size() - 1);*/
    /*if (Animate)
      eraseFigure();*/
    FFigureY ++;
    FFigureYPercent = FFigureY * 100;
    /*if (Animate)
      drawFigure();
    if (Animate)
      FPainter -> refresh();*/

    FFallHeight ++;

    return true;
  }
  else
  {
    FPainter->animateFigure(0, FFigureY * 100 - FFigureYPercent, Animate ? 30 : 1);
    return false;
  }
}

bool QuadrOsField::lowerFigure(bool Animate)
{
  bool res = moveFigureDown(Animate);
  if (! res)
  {
    if (!Animate)
      FPainter -> drawFigure();
    landFigure();
  }

  return res;
}

void QuadrOsField::landFigure()
{
  int w, h;
  for (w = 0; w < FCurFigure.size(); w ++)
  for (h = 0; h < FCurFigure.size(); h ++)
  {
    if (FCurFigure.at(w, h))
    {
      FField.setElementAt(1 + FFigureX + w, 1 + FFigureY + h,
        FCurFigure.at(w, h));
      FPainter->setElementAt(FFigureX + w, FFigureY + h,
        FCurFigure.at(w, h));
    }
  }

  FHelperLinePainter -> clear();
  //eraseFigureAt(NULL, FFigureX, FFigureY, FCurFigure, FHelperLinePainter);
}

int QuadrOsField::collapseFullLines()
{
  // we will check lines from FFigureY upto FFigureY + 5, or
  //   upto end of the field
  int upto = ((FFieldHeight > FFigureY + FCurFigure.size()) ? FFigureY + FCurFigure.size() : FFieldHeight);

  int linesArr[4];
  int lines = 0;
  for (int h = FFigureY; h < upto; h ++)
  {
    int HasSpaces = 0, HasElems = 0;
    for (int w = 0; w < FFieldWidth; w ++)
    {
      if (FField.getElementAt(1 + w, 1 + h))
        HasElems = 1;
      else
        HasSpaces = 1;
    }

    if (! HasSpaces)
    {
      // we've found full line without spaces
      linesArr[lines] = h;
      lines ++;
    }
    else if (HasElems && FCurHeight < FFieldHeight - h)
    {
      FCurHeight = FFieldHeight - h;
    }
  }
  FPainter->blinkLines(lines, linesArr);

  for (int i = 0; i < lines; i ++)
  {
    int h = linesArr[i];
    for (int hh = h; hh > 0; hh --)
    for (int w = 0; w < FFieldWidth; w ++)
    {
      int Element = FField.getElementAt(1 + w, hh);
      FField.setElementAt(1 + w, 1 + hh, Element);
      FPainter -> setElementAt(w, hh, Element);
    }

    for (int w = 0; w < FFieldWidth; w ++)
    {
      FField.setElementAt(1 + w, 1, 0);
      FPainter -> setElementAt(w, 0, 0);
    }

    FPainter -> refresh();

    FCurHeight --;
  }

  return lines;
}

void QuadrOsField::dropFigure()
{
  /*eraseFigure();
  while (lowerFigure(false));*/
  while (lowerFigure(true));
}

void QuadrOsField::copyTo(QuadrOsField * copy)
{
  if (FFieldWidth != copy->FFieldWidth || FFieldHeight != copy->FFieldHeight)
  {
    copy->setFieldSizes(FFieldWidth, FFieldHeight);
  }

  FField.copyTo(& copy->FField);
  FCurFigure.copyFigureTo(& copy->FCurFigure);
  FNextFigure.copyFigureTo(& copy->FNextFigure);
  copy->FFigureX = FFigureX;
  copy->FFigureY = FFigureY;
  copy->FCurHeight = FCurHeight;
}
