/***************************************************************************
                          quadrosfield.h  -  description
                             -------------------
    begin                : Wed Mar 13 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef QUADROSFIELD_H
#define QUADROSFIELD_H

#include <qobject.h>
#include "intmatrix.h"
#include "figure.h"

/**
  *@author Dmitry Osinovsky
  */

class QuadrOsField : public QObject
{
  Q_OBJECT
public:
  QuadrOsField(QObject * parent = 0);
  ~QuadrOsField();

  void setPainters(IMatrix * Painter,
    IMatrix * HelperLinePainter, IMatrix * NextFigurePainter);

  void moveFigureRight();
  void moveFigureLeft();
  bool rotateFigure(bool Backward, bool SmartRotate);
  void dropFigure();

  // Move figure down, if possible. Returns if it is possible.
  //   If not possible, land figure
  bool lowerFigure(bool Animate = true);

  void copyTo(QuadrOsField * copy);
  bool getAt(int X, int Y) { return FField.getElementAt(1+X, 1+Y) != 0; }
  int getFieldWidth() { return FFieldWidth; }
  int getFieldHeight() { return FFieldHeight; }
  int getFigureX() { return FFigureX; }
  int getFigureY() { return FFigureY; }
  bool canPlaceCurFigureAt(int X, int Y)
    { return canPlaceFigureAt(X, Y, FCurFigure); }

  bool getFigureAt(int X, int Y) const { return FCurFigure.at(X, Y); }
  int getFigureSize() const { return FCurFigure.size(); }
  void setFieldSizes(int FieldWidth, int FieldHeight);
  bool getSmoothMovement() const { return FSmoothMovement; }
  void setSmoothMovement(bool SmoothMovement) { FSmoothMovement = SmoothMovement; }
  void debugPrintFigure() { FCurFigure.debugPrint(); }

protected:
  bool canPlaceFigureAt(int X, int Y, const Figure & AFigure);
  int collapseFullLines();

  // Move figure down, if possible. Returns if it is possible
  //  Do not land figure
  bool moveFigureDown(bool Animate = true);

  virtual void landFigure();

  /* Functions drawFigure and eraseFigure are needed to draw figure's "shadow" on helper line */
  virtual void drawFigure() {}
  virtual void eraseFigure() {}

protected:
  // FOwnPainter == true if no painter was supplied and we had to
  //   create our own fake painter
  bool FOwnPainters;

  // FPainter - painter of the main field
  IMatrix * FPainter;
  IMatrix * FHelperLinePainter;
  IMatrix * FNextFigurePainter;

  // Main field, except for falling figure
  IntMatrix FField;

  Figure FCurFigure, FNextFigure;

  int FFieldWidth, FFieldHeight;

  // Coordinates of figure, truncated to elements
  int FFigureX, FFigureY;

  // Y coord of figure in percents of elements
  int FFigureYPercent;

  bool FSmoothMovement;

  int FCurHeight; // current height of the "buildings" in the field

  // Height from which figure fell from
  //  either as a result of dropping or (on high levels) by itself
  int FFallHeight;
};

class QuadrOsFieldTest : public QuadrOsField
{
  Q_OBJECT
public:
  void setFigureX(int FigureX) { FFigureX = FigureX; }
  int collapseFullLines() { return QuadrOsField::collapseFullLines(); }
  void nextFigure();
};

#endif
