/***************************************************************************
                          quadroslistener.cpp  -  description
                             -------------------
    begin                : Mon Feb 25 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <algorithm>
#include <qtextstream.h>
#include "nettransport.h"
#include "quadroslistener.h"
#include "quadrosdoc.h"
#include "figurecollections.h"

void QuadrOsListener::newConnection(int socket)
{
	SocketTransport * pSock = new SocketTransport(NULL);
	pSock -> setSocket(socket);
	
	if (m_bPlaying)
	{
		int MessageType;
		QString Tmp;

		(*pSock -> getIn()) >> MessageType >> Tmp >> Tmp;
			// really this reading is just to wait for socket to initialize

		(*pSock -> getOut()) << (int)(NET_CloseConnection) <<
			tr("Network game has already started");
		pSock -> flush();
		pSock -> close();
		delete pSock;
		return;
	}

	QuadrOsServerSocket * pQSock = new QuadrOsServerSocket(this, pSock);
	m_Socks.push_back(pQSock);
}

void QuadrOsListener::init()
{
  m_nRefreshPlayerTicks = 0;
  m_TimerTicks = 0;
  m_SecBeforeStart = -1;
	m_AddedLines = 0;
	m_bPlaying = false;
	m_bInited = false;
	m_bClosing = false;
	FFigureCollectionNumber = 0;
	FTimerCounter = 0;
}
	
void QuadrOsListener::Close()
{
  qDebug("QuadrOsListener::Close begin");
  m_bPlaying = false;
	m_bClosing = true;
	while(! m_Socks.empty())
	{
		QuadrOsServerSocket * sock = m_Socks.front();
    qDebug(sock->m_PlayerName.local8Bit());
    qDebug(sock->m_Team.local8Bit());
		sock->FTransport->close();
    qDebug("Transport closed");
    if (m_Socks.front() == sock)
  		sock->onClose();
    qDebug("Socket closed");
	}
	
	m_bInited = false;
  qDebug("QuadrOsListener::Close end");
}

void QuadrOsListener::RefreshPlayersInfo()
{
	QString Height;
	for (unsigned int iTo = 0; iTo < m_Socks.size(); iTo ++)
	{
		QuadrOsServerSocket * sock = m_Socks[iTo];
		(*sock->FTransport->getOut()) << (int)(NET_RefreshPlayersInfo);

		(*sock->FTransport->getOut()) << m_Teams.size();
		
		for (TeamMap::iterator p = m_Teams.begin();
		  p != m_Teams.end(); ++ p)
		{
			CTeam * Team = p->second;
			QString TeamName = p->first;

			if (Team->m_Number <= 0)
				Height = tr("-/%1").arg(Team->m_Players.size());
			else
				Height = tr("%1/%2").arg(Team->m_Number).arg(Team->m_Players.size());

			(*sock->FTransport->getOut()) << Team->m_Name << Height
				<< Team->m_Score << Team->m_TotalScore << Team->m_Players.size();
				
  		for (unsigned int iPT = 0; iPT < Team->m_Players.size(); iPT ++)
  		{
  			QuadrOsServerSocket * sock1 = Team->m_Players[iPT];

  			if (sock1->Height == -1)
  				Height = "-";
  			else
  				Height = QString::number(sock1->Height);

  			(*sock->FTransport->getOut())
  			  << sock1->m_PlayerName << Height
  				<< sock1->Score << sock1->TotalScore;
  		}
		}

		sock->FTransport->flush();
	}
}

void QuadrOsListener::StartGame(int StartLevel)
{
	m_RanGen.SetSeed(time(NULL)%0xFFFFl);
	InitialPlayerNumber = 0;
	m_bPlaying = true;
	m_AddedLines = 0;
	
	for (unsigned int pos = 0; pos < m_Socks.size(); pos ++)
	{
		QuadrOsServerSocket * sock = m_Socks[pos];
		(*sock->FTransport->getOut()) << (int)(NET_StartGame)
			<< int(StartLevel) << (unsigned int)(time(NULL)%0xFFFFl);

		sock->m_bPlaying = true;
		sock->Height = 0;
		sock->PendingLines = 0;
		sock->Score = 0;
		sock->m_AddedLines = 0;

		for (int i = 0; i < 4; i ++)
			sock->m_Stats[i] = 0;

		sock->FTransport->flush();
		InitialPlayerNumber ++;
	}

	InitialTeamNumber = 0;
	
	for (TeamMap::iterator p = m_Teams.begin();
	  p != m_Teams.end(); ++ p)
	{
		QString TeamName = p->first;
		CTeam * Team = p->second;

		Team->m_bPlaying = true;
		Team->m_Score = 0;
		Team->m_Number = Team->m_Players.size();
		Team->m_AddedLines = 0;
		Team->m_SentLines = 0;
		InitialTeamNumber ++;
	}

	RefreshPlayersInfo();
}

typedef QuadrOsServerSocket * PTetrisServerSocket;
typedef CTeam * PTeam;

int CompareSocks(const void * arg1, const void * arg2)
{
	return (* (QuadrOsServerSocket * *) arg2)->TotalScore -
		(* (QuadrOsServerSocket * *) arg1)->TotalScore;
}

int CompareTeams(const void * arg1, const void * arg2)
{
	return (* (CTeam * *) arg2)->m_TotalScore -
		(* (CTeam * *) arg1)->m_TotalScore;
}

void QuadrOsListener::EndGame()
{
	unsigned i;
	bool ShowWinner = false;

	QuadrOsServerSocket * * SockList = new PTetrisServerSocket[m_Socks.size()];
	i = 0;

	unsigned int pos;
	for (pos = 0; pos < m_Socks.size(); pos ++)
	{
		QuadrOsServerSocket * sock = m_Socks[pos];
		(*sock->FTransport->getOut()) << (int)(NET_EndGame);

		sock->FTransport->flush();
		if (sock->m_bPlaying)
		{
			sock->Score += (InitialPlayerNumber - 1);
			sock->m_bPlaying = false;
		}

		for (int j = 0; j < 4; j ++)
			sock->m_TotalStats[j] += sock->m_Stats[j];

		sock->TotalScore += sock->Score;
		sock->m_pTeam->m_Score += sock->Score * NET_PlayerExitScores;
		SockList[i ++] = sock;
	}

	qsort(SockList, m_Socks.size(), sizeof(QuadrOsServerSocket *), CompareSocks);

	QString MsgP;
  QTextOStream MsgPS(&MsgP);
  MsgPS << tr("Player results:\n");
	for (i = 0; i < m_Socks.size(); i ++)
	{
		QuadrOsServerSocket * sock = SockList[i];
		MsgPS << "  " << sock->m_PlayerName << ": " << sock->TotalScore <<
		  "\n";
	}

	delete SockList;

	CTeam * * TeamList = new PTeam[m_Teams.size()];
	i = 0;

	for (TeamMap::iterator p = m_Teams.begin();
	  p != m_Teams.end(); ++ p)
	{
		QString TeamName = p->first;
		CTeam * Team = p->second;

		if (Team->m_bPlaying)
		{
			Team->m_Score += (InitialTeamNumber - 1)*(NET_TeamExitScores);
			Team->m_bPlaying = false;
		}

		Team->m_TotalScore += Team->m_Score;

		if (uint(Team->m_TotalScore) >= NET_MaxScores)
		{
			ShowWinner = true;
		}

		TeamList[i ++] = Team;
	}

	qsort(TeamList, m_Teams.size(), sizeof(CTeam *), CompareTeams);
	QString Msg;
  QTextOStream MsgS(&Msg);
  MsgS << tr("Team results:\n");
	for (i = 0; i < m_Teams.size(); i ++)
	{
		CTeam * Team = TeamList[i];
		MsgS << "  " << Team->m_Name << ": " << Team->m_TotalScore << "\n";
	}

	delete TeamList;

	m_bPlaying = false;

	QString Msg1;
	QTextOStream Msg1S(&Msg1);
	Msg1S << tr("Player statistics:\n");
	for (pos = 0; pos < m_Socks.size(); pos ++)
	{
		QuadrOsServerSocket * sock = m_Socks[pos];
		Msg1S << sock->m_PlayerName << ":";
		for (int i = 0; i < 4; i ++)
			Msg1S << QString("\t") << sock->m_Stats[i] <<
				"/" << sock->m_TotalStats[i];

		Msg1S << "\n";
	}

	Broadcast(Msg1);

	if (ShowWinner)
	{
		Msg += MsgP + tr("Game over\n");
		Broadcast(QString("&") + Msg);
  	for (TeamMap::iterator p = m_Teams.begin();
  	  p != m_Teams.end(); ++ p)
  	{
  		QString TeamName = p->first;
  		CTeam * Team = p->second;
			Team->m_TotalScore = 0;
		}

   	for (pos = 0; pos < m_Socks.size(); pos ++)
   	{
   		QuadrOsServerSocket * sock = m_Socks[pos];
			sock->TotalScore = 0;
			for (int i = 0; i < 4; i ++)
				sock->m_TotalStats[i] = 0;
		}
	}

	RefreshPlayersInfo();
}

void QuadrOsListener::Broadcast(const QString Message)
{
	for (unsigned pos = 0; pos < m_Socks.size(); pos ++)
	{
		QuadrOsServerSocket * sock = m_Socks[pos];
		(*sock->FTransport->getOut()) << (int)(NET_Chat) << Message;

		sock->FTransport->flush();
	}
}

void QuadrOsListener::OnTimer()
{
  FTimerCounter ++;
  if (FTimerCounter > 1000/TET_TimerInterval)
  {
    FTimerCounter = 0;
  	if (m_SecBeforeStart >= 0)
  	{
  		m_SecBeforeStart --;
  		if (m_SecBeforeStart >= 0)
  		{
  			QString Msg;
  			Msg = QString("&") + tr("Starting in %1 seconds.\nPress ESC to pause.\n").arg(m_SecBeforeStart);
  			Broadcast(Msg);
  		}
  		else
  		{
  			if (m_bPlaying)
  			{
        	for (unsigned pos = 0; pos < m_Socks.size(); pos ++)
        	{
        		QuadrOsServerSocket * sock = m_Socks[pos];
  					(*sock->FTransport->getOut()) << (int)(NET_PauseOff);
  					sock->FTransport->flush();
  				}
  			}
  			else
  			{
  				StartGame(NET_StartLevel);
  			}
  		}
  	}

  	if ((! NET_TraceMessages) && m_bPlaying)
  	{
  		m_nRefreshPlayerTicks ++;
  		if (m_nRefreshPlayerTicks >= 20/TET_TimerInterval)
  		{
  			m_nRefreshPlayerTicks = 0;
  			RefreshPlayersInfo();
  		}
  	}
  }
}

QString QuadrOsListener::ThrowLines(int Lines, CTeam * Except)
{
	qDebug("QuadrOsListener::ThrowLines");

  TeamMap::iterator p;
	switch(NET_TeamToThrow)
	{
	case wtAll:
		{
			StringListBuilder List("", "", ", ", false);
     	for (p = m_Teams.begin();
     	  p != m_Teams.end(); ++ p)
     	{
     		QString TeamName = p->first;
     		CTeam * Team = p->second;

				if (Team->m_bPlaying && Team != Except)
				{
					List.add(Team->ThrowLines(Lines));
				}

			}

			return List.getResult();
		}
		break;

	case wtStrongest:
		{
			int MinHeight = 25000;
			CTeam * Strongest = NULL;
     	for (p = m_Teams.begin();
     	  p != m_Teams.end(); ++ p)
     	{
     		QString TeamName = p->first;
     		CTeam * Team = p->second;

				if (Team->m_bPlaying && Team != Except)
				{
					int Height = Team->GetStrength();
					if (Height < MinHeight)
					{
						MinHeight = Height;
						Strongest = Team;
					}
				}
			}

			if (Strongest)
			{
				QString Result = Strongest->ThrowLines(Lines);
				if (Result.isEmpty())
					return Strongest->m_Name;
				else
					return Strongest->m_Name + " (" + Result + ")";
			}
			else
				return QString("");
		}
		break;

	case wtWeakest:
		{
			int MaxHeight = -1;
			CTeam * Weakest = NULL;
     	for (p = m_Teams.begin();
     	  p != m_Teams.end(); ++ p)
     	{
     		QString TeamName = p->first;
     		CTeam * Team = p->second;

				if (Team->m_bPlaying && Team != Except)
				{
					int Height = Team->GetStrength();
					if (Height > MaxHeight)
					{
						MaxHeight = Height;
						Weakest = Team;
					}
				}
			}

			if (Weakest)
			{
				QString Result = Weakest->ThrowLines(Lines);
				if (Result.isEmpty())
					return Weakest->m_Name;
				else
					return Weakest->m_Name + " (" + Result + ")";
			}
			else
				return QString("");
		}
		break;

	case wtRandom:
		{
			double Num = 0;
     	for (p = m_Teams.begin();
     	  p != m_Teams.end(); ++ p)
     	{
     		QString TeamName = p->first;
     		CTeam * Team = p->second;

				if (Team->m_bPlaying && Team != Except)
				{
					double Score = double(m_AddedLines - Team->m_SentLines) / Team->m_AddedLines;
					Num += Score;
				}
			}

			double RandomNum = double(m_RanGen()) / OO_RAND_MAX * Num - 0.001;
			double i = 0;
			CTeam * Team = NULL;

     	for (p = m_Teams.begin();
     	  p != m_Teams.end(); ++ p)
     	{
     		QString TeamName = p->first;
     		Team = p->second;

				if (Team->m_bPlaying && Team != Except)
				{
					double Score = double(m_AddedLines - Team->m_SentLines) / Team->m_AddedLines;
					i += Score;
					if (RandomNum <= i)
					{
						break;
					}
				}
			}

			assert(Team);

			QString Result = Team->ThrowLines(Lines);

			if (Result.isEmpty())
				return Team->m_Name;
			else
				return Team->m_Name + " (" + Result + ")";
		}
		break;

	case wtRandomDisperse:
		{
			StringListBuilder List("", "", ", ", false);

			for (int j = 0; j < Lines; j ++)
			{
				double Num = 0;
      	for (p = m_Teams.begin();
      	  p != m_Teams.end(); ++ p)
      	{
      		QString TeamName = p->first;
      		CTeam * Team = p->second;

					if (Team->m_bPlaying && Team != Except)
					{
						double Score = double(m_AddedLines - Team->m_SentLines) / Team->m_AddedLines;
						Num += Score;
					}
				}

				double RandomNum = double(m_RanGen()) / OO_RAND_MAX * Num - 0.001;

				double i = 0;
				CTeam * Team = NULL;

      	for (p = m_Teams.begin();
      	  p != m_Teams.end(); ++ p)
      	{
      		QString TeamName = p->first;
      		Team = p->second;

					if (Team->m_bPlaying && Team != Except)
					{
						double Score = double(m_AddedLines - Team->m_SentLines) / Team->m_AddedLines;
						i += Score;
						if (RandomNum <= i)
						{
							break;
						}
					}
				}

				assert(Team);

				QString Result = Team->ThrowLines(1);
				if (Result.isEmpty())
					List.add(Team->m_Name);
				else
					List.add(Team->m_Name + " (" + Result + ")");
			}

			return List.getResult();
		}
		break;

	default:
		assert(false);
		return QString("");
	}
}

QuadrOsServerSocket::QuadrOsServerSocket(QuadrOsListener * pListener, NetTransport * Transport):
	m_pListener(pListener), m_pTeam(NULL),
	m_bPlaying(FALSE), Height(-1),
	PendingLines(0), m_AddedLines(0),
	Score(0), TotalScore(0),
	FTransport(Transport)
{
	int i;
	for (i = 0; i < 4; i ++)
	{
		m_TotalStats[i] = 0;
		m_Stats[i] = 0;
	}
	FTransport -> setReceiver(this);
}
	
void QuadrOsServerSocket::onClose()
{
	m_pListener->m_Socks.erase(find(m_pListener->m_Socks.begin(),
	  m_pListener->m_Socks.end(), this));

	if (! m_pListener->m_bClosing &&m_pListener->m_bPlaying)
		PlayerStopGame();

	if (m_pTeam)
		m_pTeam->DeletePlayer(this);

	if (! m_pListener->m_bClosing)
  	m_pListener->RefreshPlayersInfo();

	delete this;
}

void QuadrOsServerSocket::PlayerStopGame()
{
	unsigned pos;
	m_bPlaying = false;
	int TeamPlayerNum = 0;
	for (pos = 0; pos < m_pTeam->m_Players.size(); pos ++)
	{
		QuadrOsServerSocket * sock = m_pTeam->m_Players[pos];
		if (sock->m_bPlaying)
		{
			TeamPlayerNum ++;
		}
	}

	m_pTeam->m_Number = TeamPlayerNum;

	if (TeamPlayerNum == 0)
	{
		m_pTeam->m_bPlaying = false;
		int TeamNum = 0;
		CTeam * Vict = NULL;
   	for (TeamMap::iterator p = m_pListener->m_Teams.begin();
   	  p != m_pListener->m_Teams.end(); ++ p)
   	{
   		QString TeamName = p->first;
   		CTeam * Team = p->second;

			if (Team->m_bPlaying)
			{
				TeamNum ++;
				Vict = Team;
			}
		}

		m_pTeam->m_Score += (m_pListener->InitialTeamNumber - TeamNum - 1)*m_pListener->NET_TeamExitScores;

		if (TeamNum <= 1)
		{
			if (Vict)
			{
				QString Msg = m_pListener->tr("%1 won this round, team status: %2 sent, %3 of %4 received.\n").
				  arg(Vict->m_Name).arg(Vict->m_SentLines).arg(Vict->m_AddedLines).arg(m_pListener->m_AddedLines);

				m_pListener->Broadcast(Msg);
			}
			m_pListener->EndGame();
		}
	}

	m_pListener->RefreshPlayersInfo();
}

QString QuadrOsListener::ThrowTo2Str(ThrowTo TT, bool ToTeam)
{
	switch (TT)
	{
	case wtAll:
		return ToTeam ? tr("all teams") : tr("all players");

	case wtStrongest:
		return ToTeam ? tr("the strongest team") : tr("the strongest player");

	case wtWeakest:
		return ToTeam ? tr("the weakest team") : tr("the weakest player");

	case wtRandom:
		return ToTeam ? tr("the random team") : tr("the random player");

	case wtRandomDisperse:
		return ToTeam ? tr("the random team dispatching lines") : tr("the random player dispatching lines");

	default:
		return QString("error");
	}
}

void QuadrOsServerSocket::SendOptions()
{
	(*FTransport->getOut()) << int(NET_Options) <<
		m_pListener->NET_MaxScores <<
		int(m_pListener->NET_PlayerToThrow) <<
		m_pListener->NET_PlayerExitScores <<
		int(m_pListener->NET_TeamToThrow) <<
		m_pListener->NET_TeamExitScores <<
		m_pListener->NET_StartLevel <<
		(int) m_pListener->IncreaseLevel <<
		m_pListener->NET_SecBeforeStart <<
		m_pListener->SKILL_Height <<
		(int) m_pListener->NET_TraceMessages <<
		m_pListener->FFigureCollectionNumber;

	QString Opt = m_pListener->tr(
	  "Options:\n"
		"  Start level: %1\n"
		"  Game type: %2\n"
		"  Throw to %3 of %4\n"
		"  Play till %5 scores\n"
		"  Height %6\n").
		arg(10 - m_pListener->NET_StartLevel).
		arg(FigureCollections::instance()->at(m_pListener->FFigureCollectionNumber)->name()).
		arg(m_pListener->ThrowTo2Str(m_pListener->NET_PlayerToThrow, false)).
		arg(m_pListener->ThrowTo2Str(m_pListener->NET_TeamToThrow, true)).
		arg(m_pListener->NET_MaxScores).
		arg(m_pListener->SKILL_Height - 1);

	(*FTransport->getOut()) << int(NET_Chat) << Opt;
	FTransport->flush();
}

void QuadrOsServerSocket::ThrowLines(int Lines)
{
	if (m_pListener->NET_PendLines)
		PendingLines += Lines;
	else
		Height += Lines;

	(*FTransport->getOut()) << (int)(NET_AddLines) << (int)(Lines);
	FTransport->flush();
	qDebug("QuadrOsServerSocket::ThrowLines flush");

	m_AddedLines += Lines;
	m_pTeam->m_AddedLines += Lines;
	m_pListener->m_AddedLines += Lines;
	qDebug("QuadrOsServerSocket::ThrowLines end");
}

bool QuadrOsServerSocket::receiveOne()
{
	int MessageType;
	unsigned pos;

  qDebug("QuadrOsServerSocket::receiveOne()");
	(*FTransport->getIn()) >> MessageType;
  qDebug("MessageType %d received", MessageType);

	switch(MessageType)
	{
	case NET_Connected:
		{
			int Version[4];
			(*FTransport->getIn()) >> m_PlayerName >> m_Team;
		  qDebug("PlayerName %s Team %s", (const char *)m_PlayerName,
		    (const char *)m_Team);
      (*FTransport->getIn()) >>
				Version[0] >> Version[1] >>
				Version[2] >> Version[3];
		  qDebug("Version %d %d %d %d", Version[0], Version[1],
				Version[2], Version[3]);


			for (int i = 0; i < 3; i ++)
			{
				if (Version[i] != TET_Version[i])
				{
					(*FTransport->getOut()) << (int)(NET_CloseConnection) <<
						QString(m_pListener->tr("Incompatible tetris versions"));
					FTransport->flush();
					FTransport->close();
					onClose();
					return false;
				}
			}

    	for (pos = 0; pos < m_pListener->m_Socks.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_pListener->m_Socks[pos];
  		  qDebug("In cycle %s", (const char *)sock->m_PlayerName);
				if (sock == this)
					continue;

				if (sock->m_PlayerName == m_PlayerName)
				{
					(*FTransport->getOut()) << (int)(NET_CloseConnection) <<
						QString(m_pListener->tr("Player with the same name already exists"));
					FTransport->flush();
					FTransport->close();
					onClose();
					return false;
				}
			}

 		  qDebug("Unique names checked");
			(*FTransport->getOut()) << (int)(NET_Connected);
 		  qDebug("NET_Connected sent");
			FTransport->flush();
 		  qDebug("NET_Connected flush");
			TeamMap::iterator it = m_pListener->m_Teams.find(m_Team);
			if (it != m_pListener->m_Teams.end())
			  m_pTeam = it->second;
			else
			{
   		  qDebug("No team - create");
				m_pTeam = new CTeam;
				m_pTeam->m_pListener = m_pListener;
				m_pTeam->m_Name = m_Team;
				m_pListener->m_Teams[m_Team] = m_pTeam;
   		  qDebug("Team created");
			}

			m_pTeam->AddPlayer(this);
   		qDebug("Player added");

			if (m_pListener->m_bInited)
				SendOptions();
   		qDebug("Options sent");

			m_pListener->RefreshPlayersInfo();
   		qDebug("Refresh player info");
		}
		break;

	case NET_Chat:
		{
			QString Message;
			(*FTransport->getIn()) >> Message;

			m_pListener->Broadcast(m_PlayerName + ": " + Message);
		}
		break;

	case NET_StartGame:
		{
			int StartLevel;
			(*FTransport->getIn()) >> StartLevel;
			m_pListener->m_SecBeforeStart = m_pListener->NET_SecBeforeStart;
			if (! m_pListener->NET_SecBeforeStart)
				m_pListener->OnTimer();
			//m_pListener->StartGame(StartLevel);
		}
		break;

	case NET_EndGame:
		{
			Height = -1;

			QString Msg =
			  m_pListener->tr("%1 was filled up with presents, team status: %2 sent, %3 of %4 received.\n").
			  arg(m_PlayerName).arg(m_pTeam->m_SentLines).arg(m_pTeam->m_AddedLines).
			  arg(m_pListener->m_AddedLines);
			m_pListener->Broadcast(Msg);
			m_bPlaying = false;

			if (! m_pListener->m_bPlaying)
				break;

			int PlayerNum = 0;
    	for (pos = 0; pos < m_pListener->m_Socks.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_pListener->m_Socks[pos];
				if (sock->m_bPlaying)
				{
					PlayerNum ++;
				}
			}

			Score += (m_pListener->InitialPlayerNumber - PlayerNum - 1);
			PlayerStopGame();
		}
		break;

	case NET_PauseOn:
		m_pListener->m_SecBeforeStart = -1;

		if (m_pListener->m_bPlaying)
		{
    	for (pos = 0; pos < m_pListener->m_Socks.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_pListener->m_Socks[pos];
				(*sock->FTransport->getOut()) << (int)(NET_PauseOn);
				sock->FTransport->flush();
			}
		}

		m_pListener->Broadcast(QString("&") + m_pListener->tr("%1 paused the game.\n").arg(m_PlayerName));
		break;

	case NET_PauseOff:
		m_pListener->Broadcast(m_pListener->tr("%1 resumed the game.\n").arg(m_PlayerName));
		m_pListener->m_SecBeforeStart = m_pListener->NET_SecBeforeStart;
		if (! m_pListener->NET_SecBeforeStart)
			m_pListener->OnTimer();

		break;

	case NET_FigureDropped:
		{
			int Lines;
			long Score1;
			(*FTransport->getIn()) >> Height >> Score1 >> Lines;
      qDebug("FigureDropped received %d %ld %d", Height, Score1, Lines);
			PendingLines = 0;

			if (Lines)
				m_Stats[Lines > 4 ? 3 : Lines - 1] ++;

			if (Lines > 1 && m_bPlaying)
			{
				if (Lines < 4)
					Lines --;

				QString Msg;
				Msg = m_pListener->tr("%1 has sent a %2-line present%3.\n").arg(m_PlayerName).arg(Lines);
				qDebug(Msg);

				QString To = m_pListener->ThrowLines(Lines, m_pTeam);
				if (! To.isEmpty())
				{
					Msg = Msg.arg(m_pListener->tr(" to %1").arg(To));
				}
				else
				{
					Msg = Msg.arg("");
				}

				m_pTeam->m_SentLines += Lines;

				if (m_pListener->NET_TraceMessages)
				{
					m_pListener->Broadcast(Msg);
				}
			}

			if (m_pListener->NET_TraceMessages)
				m_pListener->RefreshPlayersInfo();
		}

		break;

	case NET_Options:
		{
		  int IncreaseLevel, Trace;
			(*FTransport->getIn()) >>
				m_pListener->NET_MaxScores >>
				(int &) m_pListener->NET_PlayerToThrow >>
				m_pListener->NET_PlayerExitScores >>
				(int &) m_pListener->NET_TeamToThrow >>
				m_pListener->NET_TeamExitScores >>
				m_pListener->NET_StartLevel >>
				IncreaseLevel >>
				m_pListener->NET_SecBeforeStart >>
				m_pListener->SKILL_Height >>
				Trace >>
				m_pListener->FFigureCollectionNumber;
				
			m_pListener->NET_TraceMessages = Trace;
			m_pListener->IncreaseLevel = IncreaseLevel;			

			m_pListener->m_bInited = true;

    	for (pos = 0; pos < m_pListener->m_Socks.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_pListener->m_Socks[pos];
				sock->SendOptions();
			}
		}
		break;
	}

	return true;
}

void CTeam::AddPlayer(QuadrOsServerSocket * pPlayer)
{
	m_Players.push_back(pPlayer);
}

void CTeam::DeletePlayer(QuadrOsServerSocket * pPlayer)
{
	m_Players.erase(find(m_Players.begin(), m_Players.end(), pPlayer));

	if (m_Players.size() == 0)
	{
		m_pListener->m_Teams.erase(m_pListener->m_Teams.find(m_Name));
		delete this;
	}
}

QString CTeam::ThrowLines(int Lines)
{
	unsigned pos;
	switch(m_pListener->NET_PlayerToThrow)
	{
	case wtAll:
		{
    	for (pos = 0; pos < m_Players.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_Players[pos];
				if (sock->m_bPlaying)
				{
					sock->ThrowLines(Lines);
				}
			}
			qDebug("After ThrowLines");

			return QString("");
		}
		break;

	case wtStrongest:
		{
			int MinHeight = 25000;
			QuadrOsServerSocket * Strongest = NULL;
    	for (pos = 0; pos < m_Players.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_Players[pos];
				if (sock->m_bPlaying)
				{
					if (sock->Height < MinHeight)
					{
						MinHeight = sock->Height;
						Strongest = sock;
					}
				}
			}

			if (Strongest)
			{
				Strongest->ThrowLines(Lines);
				return Strongest->m_PlayerName;
			}
			else
				return QString("");
		}
		break;

	case wtWeakest:
		{
			int MaxHeight = -1;
			QuadrOsServerSocket * Weakest = NULL;
    	for (pos = 0; pos < m_Players.size(); pos ++)
    	{
    		QuadrOsServerSocket * sock = m_Players[pos];
				if (sock->m_bPlaying)
				{
					if (sock->Height > MaxHeight)
					{
						MaxHeight = sock->Height;
						Weakest = sock;
					}
				}
			}

			if (Weakest)
			{
				Weakest->ThrowLines(Lines);
				return Weakest->m_PlayerName;
			}
			else
				return QString("");
		}
		break;

	case wtRandom:
		{
			int RandomNum = m_pListener->m_RanGen() % m_Number;
			QuadrOsServerSocket * sock = m_Players[RandomNum];

			assert(sock);

			sock->ThrowLines(Lines);
			return sock->m_PlayerName;
		}
		break;

	case wtRandomDisperse:
		{
			StringListBuilder List("", "", ", ", false);

			for (int j = 0; j < Lines; j ++)
			{
				int RandomNum = m_pListener->m_RanGen() % m_Number;
  			QuadrOsServerSocket * sock = m_Players[RandomNum];

				assert(sock);

				sock->ThrowLines(1);
				List.add(sock->m_PlayerName);
			}

			return List.getResult();
		}
		break;

	default:
		assert(false);
		return QString("");
	}
}

int CTeam::GetStrength()
{
	int Result = - m_Number * m_pListener->SKILL_Height;
	unsigned pos;
 	for (pos = 0; pos < m_Players.size(); pos ++)
 	{
 		QuadrOsServerSocket * sock = m_Players[pos];
		if (sock->m_bPlaying)
		{
			Result += sock->Height;
		}
	}

	return Result;
}

CTeam::~CTeam()
{
}
