/***************************************************************************
                          quadrosview.cpp  -  description
                             -------------------
    begin                : Feb 24 00:04:59 MSK 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <qvbox.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpen.h>

#include "quadrosdoc.h"
#include "buttongrid.h"
#include "quadrosview.h"
#include "quadrosai.h"

const int TET_KeyboardTimerInterval = TET_TimerInterval;

QuadrOsView::QuadrOsView(QWidget *parent, QuadrOsDoc *doc) :
  QHBox(parent)
{
  QVBox * MainVBox = new QVBox(this);
  MainVBox -> setMargin(2);
  FMainField = new ButtonGrid(MainVBox);
  FHelperLine = new ButtonGrid(MainVBox);

  QVBox * StatsVBox = new QVBox(this);
  QLabel * Label;
  StatsVBox -> setMargin(2);
  StatsVBox -> setSpacing(10);
  StatsVBox -> setStretchFactor(new QFrame(StatsVBox), 100);

  Label = new QLabel(tr("Next figure"), StatsVBox);
  FNextFigure = new ButtonGrid(StatsVBox);
  Label = new QLabel(tr("Scores"), StatsVBox);
  FScoreLCD = new QLCDNumber(8, StatsVBox);
  FScoreLCD -> setFrameStyle(QFrame::Panel | QFrame::Sunken);
  FScoreLCD -> setLineWidth(2);
  FScoreLCD -> setBackgroundColor(black);
  StatsVBox -> setStretchFactor(FScoreLCD, 150);

  Label = new QLabel(tr("Level"), StatsVBox);
  FLevelLCD = new QLCDNumber(8, StatsVBox);
  FLevelLCD -> setFrameStyle(QFrame::Panel | QFrame::Sunken);
  FLevelLCD -> setLineWidth(2);
  FLevelLCD -> setBackgroundColor(black);
  StatsVBox -> setStretchFactor(FLevelLCD, 150);

  Label = new QLabel(tr("Lines"), StatsVBox);
  FLinesLCD = new QLCDNumber(8, StatsVBox);
  FLinesLCD -> setFrameStyle(QFrame::Panel | QFrame::Sunken);
  FLinesLCD -> setLineWidth(2);
  FLinesLCD -> setBackgroundColor(black);
  StatsVBox -> setStretchFactor(FLinesLCD, 150);

  StatsVBox -> setStretchFactor(new QFrame(StatsVBox), 500);
  /*(new QFrame(StatsVBox)) -> setFrameStyle(0);
  (new QFrame(StatsVBox)) -> setFrameStyle(0);*/

  FMainField -> setElementIndents(0, TET_HeightIndent);
  // setOptions(FOptions);
  doc -> setPainters(FMainField, FHelperLine, FNextFigure);
  FDoc = doc;
  connect(FDoc, SIGNAL(updateStats(long, int, int, int)), this, SLOT(updateStats(long, int, int, int)));
  connect(FDoc, SIGNAL(pauseOn()), this, SLOT(pauseOn()));
  connect(FDoc, SIGNAL(pauseOff()), this, SLOT(pauseOff()));
  FScore = 0;
  FAnimatedScore = 0;
  FAI = 0;
}

QuadrOsView::~QuadrOsView()
{
}

void QuadrOsView::resize ( int w, int h )
{
  QWidget::resize(w, h);
  // FButtonGrid.resize(w, h);
  fputs("QuadrOsView::resize executed \n", stderr);
}

void QuadrOsView::updateStats(long Score, int Level, int Lines, int)
{
  FLevelLCD -> display(Level);
  FLinesLCD -> display(Lines);
  if (Score > FAnimatedScore)
  {
    FScore = Score;
  }
  else
  {
    FScoreLCD -> display((int) Score);
    FScore = Score;
    FAnimatedScore = Score;
  }

  FLeftKeyState = -1;
  FRightKeyState = -1;
  FKeyAccelerateState = -1;
}

void QuadrOsView::parentKeyPressEvent(QKeyEvent * e)
{
  e-> accept();
  if (e -> isAutoRepeat() || FAI != 0)
  {
    e -> ignore();
    return;
  }

  if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyLeft])
    FLeftKeyState = 0;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyRight])
    FRightKeyState = 0;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyJumpBound])
    FKeyBoundState = true;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyAccelerate])
    FKeyAccelerateState = FOptions.KeyRepeatRate;

  if (! FDoc -> isRunning())
  {
    e -> ignore();
    return;
  }

  int RepeatCount = 1;
  if (FKeyBoundState)
    RepeatCount = 10;

  if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyLeft])
  {
    for (int i = 0; i < RepeatCount; i ++)
      FDoc -> moveFigureLeft();
  }
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyRight])
  {
    for (int i = 0; i < RepeatCount; i ++)
      FDoc -> moveFigureRight();
  }
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyRotate])
  {
    FDoc -> rotateFigure(false, FOptions.SmartRotate);
  }
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyRotateBackward])
  {
    FDoc -> rotateFigure(true, FOptions.SmartRotate);
  }
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyDrop])
  {
    FDoc -> dropFigure();
  }
  else
  {
    e -> ignore();
  }
}

void QuadrOsView::parentKeyReleaseEvent(QKeyEvent * e)
{
  if (e -> isAutoRepeat() || FAI != 0)
    return;

  if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyLeft])
    FLeftKeyState = -1;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyRight])
    FRightKeyState = -1;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyJumpBound])
    FKeyBoundState = false;
  else if (e -> key() == FOptions.KeyCodes[DesignOptions::KeyAccelerate])
    FKeyAccelerateState = -1;

  e -> ignore();
}

void QuadrOsView::onTimer()
{
  if (FScore > FAnimatedScore)
  {
    if (FScore - FAnimatedScore < 10)
      FAnimatedScore ++;
    else
      FAnimatedScore += (FScore - FAnimatedScore) / 10;

    FScoreLCD -> display((int) FAnimatedScore);
  }

  if (! FDoc -> isRunning())
    return;

  if (FAI != 0)
  {
    FAI->onTimer();
    return;
  }

  if (FLeftKeyState >= 0)
  {
    FLeftKeyState ++;
    if (FLeftKeyState * TET_KeyboardTimerInterval >= FOptions.BeforeKeyRepeatInterval)
    {
      FDoc -> moveFigureLeft();
      FLeftKeyState = (FOptions.BeforeKeyRepeatInterval - FOptions.KeyRepeatRate) / TET_KeyboardTimerInterval;
    }
  }

  if (FRightKeyState >= 0)
  {
    FRightKeyState ++;
    if (FRightKeyState * TET_KeyboardTimerInterval >= FOptions.BeforeKeyRepeatInterval)
    {
      FDoc -> moveFigureRight();
      FRightKeyState = (FOptions.BeforeKeyRepeatInterval - FOptions.KeyRepeatRate) / TET_KeyboardTimerInterval;
    }
  }

  if (FKeyAccelerateState >= 0)
  {
    FKeyAccelerateState ++;
    if (FKeyAccelerateState * TET_KeyboardTimerInterval >= FOptions.KeyRepeatRate)
    {
      if (FDoc -> accelerateFigure())
        FKeyAccelerateState = 0;
    }
  }
}

void QuadrOsView::parentFocusOut()
{
  clearKeyboardState();
}

void QuadrOsView::setOptions(const DesignOptions Options)
{
  FOptions = Options;
  FMainField -> setAppearance(FOptions.BackgroundFileName,
    FOptions.ElementsFileName, FOptions.ElementSize,
    FOptions.GridInterval, FOptions.Color, TET_ColorNum, Options.AnimationSpeed);
  FHelperLine -> setAppearance(FOptions.BackgroundFileName,
    FOptions.ElementsFileName, FOptions.ElementSize,
    FOptions.GridInterval, FOptions.Color, TET_ColorNum, Options.AnimationSpeed);
  FNextFigure -> setAppearance(FOptions.BackgroundFileName,
    FOptions.ElementsFileName, FOptions.ElementSize,
    0, FOptions.Color, TET_ColorNum, Options.AnimationSpeed);
  FDoc->setSmoothMovement(Options.SmoothMovement);

  clearKeyboardState();
  FDoc->setFigureOptions(Options.FFigureOptions);
}

void QuadrOsView::turnAIOn(bool Aggressive, int AISpeed)
{
  FAI = new QuadrOsAI(FDoc);
  FAI->setAggressive(Aggressive);
  FAI->setSpeed(AISpeed);
  clearKeyboardState();
}

void QuadrOsView::turnAIOff()
{
  if (FAI != 0)
  {
    delete FAI;
    FAI = 0;
  }
}

void QuadrOsView::clearKeyboardState()
{
  FLeftKeyState = -1;
  FRightKeyState = -1;
  FKeyBoundState = false;
  FKeyAccelerateState = -1;
}

void QuadrOsView::pauseOn()
{
  QPixmap * pausePic = new QPixmap(FMainField->width()*3/4, FMainField->height()/4);
  pausePic->fill(Qt::black);
  QPainter p;
  p.begin(pausePic, FMainField);
  p.setPen(QPen(white, 5));
  p.drawRoundRect(pausePic->rect());
  QFont f("arial", 35);
  //f.setPointSize(40);
  p.setFont(f);
  p.drawText(pausePic->rect(), Qt::AlignHCenter | Qt::AlignVCenter,
    tr("PAUSE"));
  p.end();
  FMainField->drawPixmap(pausePic, FMainField->width()/8, FMainField->height()*3/8);
}

void QuadrOsView::pauseOff()
{
  FMainField->drawPixmap(0, 0, 0);
}

