/***************************************************************************
                          skilloptions.cpp  -  description
                             -------------------
    begin                : Fri Mar 1 2002
    copyright            : (C) 2002 by Dmitry Osinovsky
    email                : osdm@mail.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <qwidget.h>
#include <qdialog.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include "configfile.h"
#include "skilloptions.h"
#include "skilloptionsdlg.h"
#include "figurecollections.h"

SkillOptionsDlg::SkillOptionsDlg(QWidget *parent, const char *name )
  : QDialog(parent, name, true)
{
  QGridLayout * Layout = new QGridLayout(this,
    5, 2);
  Layout -> setMargin(11);
  Layout -> setSpacing(6);

  int i = 0;
  QLabel * label;
  label = new QLabel(tr("Start level"), this);
  Layout -> addWidget(label, i, 0);

  FStartLevel = new QSpinBox(1, 26, 1, this);
  Layout -> addWidget(FStartLevel, i ++, 1);

  FIncreaseLevel = new QCheckBox(tr("Increase level in the game"), this);
  Layout -> addWidget(FIncreaseLevel, i ++, 0);

  FTrashLines = new QCheckBox(tr("Pop trash lines"),this);
  Layout -> addWidget(FTrashLines, i ++, 0);

  label = new QLabel(tr("Computer speed (in Human vs Computer)"), this);
  Layout -> addWidget(label, i, 0);

  FComputerSpeed = new QSpinBox(1, 26, 1, this);
  Layout -> addWidget(FComputerSpeed, i ++, 1);

  /*label = new QLabel(tr("Field height"), this);
  Layout -> addWidget(label, i, 0);

  FFieldHeight = new QSpinBox(10, 25, 1, this);
  Layout -> addWidget(FFieldHeight, i ++, 1);*/

  label = new QLabel(tr("Figure collection"), this);
  Layout -> addWidget(label, i, 0);

  FFigureCollection = new QComboBox(this);
  for (int j = 0; j < FigureCollections::instance()->collectionNumber() - 1; j ++)
  {
    FFigureCollection->insertItem(FigureCollections::instance()->at(j)->name());
  }

  Layout -> addWidget(FFigureCollection, i ++, 1);

  QPushButton * OkButton = new QPushButton(tr("OK"), this);
  Layout -> addWidget(OkButton, i, 0, AlignRight);
  QPushButton * CancelButton = new QPushButton(tr("Cancel"), this);
  Layout -> addWidget(CancelButton, i, 1, AlignLeft);

  connect(OkButton, SIGNAL(clicked()), this, SLOT(accept()));
  connect(CancelButton, SIGNAL(clicked()), this, SLOT(reject()));
  OkButton -> setDefault(true);
}

SkillOptionsDlg::~SkillOptionsDlg(){
}

void SkillOptionsDlg::editOptions(SkillOptions & opt)
{
  FStartLevel -> setValue(opt.StartLevel);
  FIncreaseLevel -> setChecked(opt.IncreaseLevel);
  FTrashLines -> setChecked(opt.TrashLines);
  //FFieldHeight -> setValue(opt.FieldHeight);
  FFigureCollection -> setCurrentItem(opt.FigureCollectionNumber);
  FComputerSpeed->setValue(opt.ComputerSpeed);

  if (exec() == Accepted)
  {
    opt.StartLevel = FStartLevel -> value();
    opt.IncreaseLevel = FIncreaseLevel -> isChecked();
    opt.TrashLines = FTrashLines -> isChecked();
    //opt.FieldHeight = FFieldHeight -> value();
    opt.FigureCollectionNumber = FFigureCollection -> currentItem();
    opt.ComputerSpeed = FComputerSpeed->value();
  }
}

void bindSkillOptions(ConfigFile & file, const QString & sect, SkillOptions & opt)
{
  file.bindIntValue(sect, "StartLevel", & opt.StartLevel);
  file.bindBoolValue(sect, "IncreaseLevel", & opt.IncreaseLevel);
  file.bindBoolValue(sect, "TrashLines", & opt.TrashLines);
  //file.bindIntValue(sect, "FieldHeight", & opt.FieldHeight);
  file.bindIntValue(sect, "FigureCollectionNumber", & opt.FigureCollectionNumber);
  file.bindIntValue(sect, "ComputerSpeed", & opt.ComputerSpeed);
}

void editSkillOptions(QWidget * parent, SkillOptions & opt)
{
  SkillOptionsDlg dlg(parent);
  dlg.editOptions(opt);
}
